@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _ASTNODE_HPP
@@ -36,6 +34,7 @@ class StaticAnalysis;
 class PendingUpdateList;
 class EventHandler;
 class StaticTyper;
+struct BoolResult;
 
 class XQILLA_API ASTNode : public LocationInfo
 {
@@ -51,7 +50,6 @@ public:
     VARIABLE,
     STEP,
     IF,
-    INSTANCE_OF,
     CASTABLE_AS,
     CAST_AS,
     TREAT_AS,
@@ -64,7 +62,7 @@ public:
     FUNCTION_CALL,
     USER_FUNCTION,
     ORDERING_CHANGE,
-    XPATH1_CONVERT,
+    FUNCTION_COERCION,
     PROMOTE_UNTYPED,
     PROMOTE_NUMERIC,
     PROMOTE_ANY_URI,
@@ -97,6 +95,7 @@ public:
     INLINE_FUNCTION,
     FUNCTION_REF,
     FUNCTION_DEREF,
+    PARTIAL_APPLY,
     COPY_OF,
     COPY,
     MAP,
@@ -139,6 +138,9 @@ public:
       each of the given context items in turn. */
   virtual Result iterateResult(const Result &contextItems, DynamicContext* context) const = 0;
 
+  /** Returns the result of this expression as a boolean */
+  virtual BoolResult boolResult(DynamicContext* context) const = 0;
+
   /** Returns the result of this expression via the EventHandler provided.
       An EventGenerator may be returned to be called as a tail call optimization */
   virtual EventGenerator::Ptr generateEvents(EventHandler *events, DynamicContext *context,
