@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _ASTNODEIMPL_HPP
@@ -27,7 +25,6 @@ class Numeric;
 
 #include <xqilla/ast/ASTNode.hpp>
 #include <xqilla/runtime/Result.hpp>
-#include <xqilla/runtime/LazySequenceResult.hpp>
 #include <xqilla/ast/StaticAnalysis.hpp>
 
 /** class which is used as base for all the different data types */
@@ -58,6 +55,9 @@ public:
   /// Default - returns an empty update list
   virtual PendingUpdateList createUpdateList(DynamicContext *context) const;
 
+  /// Default implementation calls ATBooleanOrDerived::isTrue()
+  virtual BoolResult boolResult(DynamicContext* context) const;
+
   /**
    * Can be overridden by derived classes. Default implementation returns a
    * NavStepResult constructed using contextItems and this expression.
