@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQCASTABLEAS_HPP
@@ -25,7 +23,6 @@
 #include <xqilla/framework/XQillaExport.hpp>
 
 #include <xqilla/ast/ASTNodeImpl.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 
 class SequenceType;
 
@@ -35,9 +32,10 @@ public:
   XQCastableAs(ASTNode* expr, SequenceType* exprType, XPath2MemoryManager* memMgr);
   XQCastableAs(ASTNode* expr, SequenceType* exprType, bool isPrimitive, AnyAtomicType::AtomicObjectType typeIndex, XPath2MemoryManager* memMgr);
 
-  virtual Result createResult(DynamicContext* context, int flags=0) const;
   virtual ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 
   ASTNode *getExpression() const;
   SequenceType *getSequenceType() const;
@@ -47,17 +45,6 @@ public:
   void setExpression(ASTNode *item);
 
 protected:
-  class CastableAsResult : public SingleResult
-  {
-  public:
-    CastableAsResult(const XQCastableAs *di);
-
-    Item::Ptr getSingleResult(DynamicContext *context) const;
-
-  private:
-    const XQCastableAs *_di;
-  };
-
   ASTNode* _expr;
   SequenceType* _exprType;
   bool _isPrimitive;
