@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQCONTEXTITEM_HPP
@@ -25,7 +23,6 @@
 #include <xqilla/framework/XQillaExport.hpp>
 
 #include <xqilla/ast/ASTNodeImpl.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 
 /** Returns the context item. Raises [err:XP0002] if the context item is null. */
 class XQILLA_API XQContextItem : public ASTNodeImpl
@@ -42,13 +39,7 @@ public:
   virtual ASTNode *staticTypingImpl(StaticContext *context);
 
   virtual Result createResult(DynamicContext* context, int flags=0) const;
-};
-
-class XQILLA_API ContextItemResult : public SingleResult
-{
-public:
-  ContextItemResult(const LocationInfo *location);
-  Item::Ptr getSingleResult(DynamicContext *context) const;
+  static Result result(DynamicContext *context, const LocationInfo *info);
 };
 
 #endif
