@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQDOCUMENTORDER_HPP
@@ -24,7 +22,6 @@
 
 #include <xqilla/framework/XQillaExport.hpp>
 #include <xqilla/ast/ASTNodeImpl.hpp>
-#include <xqilla/runtime/LazySequenceResult.hpp>
 #include <xqilla/items/Node.hpp>
 
 #include <set>
@@ -48,15 +45,14 @@ protected:
   bool unordered_;
 };
 
-class XQILLA_API DocumentOrderResult : public LazySequenceResult
+class XQILLA_API DocumentOrderResult : public ResultImpl
 {
 public:
-  DocumentOrderResult(const LocationInfo *location, const Result &parent, DynamicContext *context)
-    : LazySequenceResult(location, context), parent_(parent) {}
-  void getResult(Sequence &toFill, DynamicContext *context) const;
-  std::string asString(DynamicContext *context, int indent) const { return "documentorderresult"; }
+  DocumentOrderResult(const LocationInfo *location, const Result &parent)
+    : ResultImpl(location), parent_(parent) {}
+  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
 private:
-  mutable Result parent_;
+  Result parent_;
 };
 
 class XQILLA_API UniqueNodesResult : public ResultImpl
@@ -66,7 +62,6 @@ public:
     : ResultImpl(doc), parent_(parent), nTypeOfItemsInLastStep_(0),
       noDups_(uniqueLessThanCompareFn(context)) {}
   Item::Ptr next(DynamicContext *context);
-  std::string asString(DynamicContext *context, int indent) const { return "uniquenodesresult"; }
 
 private:
   class uniqueLessThanCompareFn {
