@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef XQFUNCTIONCALL_HPP
@@ -28,17 +26,22 @@ class XQILLA_API XQFunctionCall : public ASTNodeImpl
 {
 public:
   XQFunctionCall(const XMLCh *qname, VectorOfASTNodes *args, XPath2MemoryManager *mm);
+  XQFunctionCall(const XMLCh *prefix, const XMLCh *uri, const XMLCh *name, VectorOfASTNodes *args, XPath2MemoryManager *mm);
 
   virtual ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
 
   virtual Result createResult(DynamicContext* context, int flags=0) const;
 
-  const XMLCh *getQName() const { return qname_; }
+  const XMLCh *getPrefix() const { return prefix_; }
+  const XMLCh *getURI() const { return uri_; }
+  const XMLCh *getName() const { return name_; }
   const VectorOfASTNodes *getArguments() const { return args_; }
 
 private:
-  const XMLCh *qname_;
+  const XMLCh *prefix_;
+  const XMLCh *uri_;
+  const XMLCh *name_;
   VectorOfASTNodes *args_;
 };
 
