@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef XQGLOBALVARIABLE_HPP
@@ -32,8 +30,10 @@ public:
   XQGlobalVariable(const XMLCh* varQName, SequenceType* seqType, ASTNode* value, XPath2MemoryManager *mm, bool isParam = false);
 
   void execute(DynamicContext* context) const;
+  void resolveName(StaticContext *context);
   void staticResolution(StaticContext *context);
   void staticTypingOnce(StaticContext *context, StaticTyper *styper);
+  void resetStaticTypingOnce();
   void staticTyping(StaticContext *context, StaticTyper *styper);
 
   const XMLCh* getVariableName() const;
@@ -64,7 +64,7 @@ protected:
   SequenceType* m_Type;
   ASTNode* m_Value;
   StaticAnalysis _src;
-  bool staticTyped_;
+  enum { BEFORE, DURING, AFTER } staticTyped_;
 };
 
 #endif
