@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQIF_HPP
@@ -48,17 +46,6 @@ public:
   void setWhenFalse(ASTNode *item);
 
 protected:
-  class IfResult : public ResultImpl
-  {
-  public:
-    IfResult(const XQIf *di);
-
-    Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
-
-  private:
-    const XQIf *_di;
-  };
-
   ASTNode* _test,* _whenTrue,* _whenFalse;
 };
 
