@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef XQINLINEFUNCTION_HPP
@@ -25,32 +23,40 @@
 #include <xqilla/ast/ASTNodeImpl.hpp>
 
 class XQUserFunction;
-class InlineFunctionResult;
+class FunctionSignature;
 
 class XQILLA_API XQInlineFunction : public ASTNodeImpl
 {
 public:
   XQInlineFunction(XQUserFunction *func, XPath2MemoryManager *mm);
-  XQInlineFunction(XQUserFunction *func, unsigned int numArgs, ASTNode *instance, XPath2MemoryManager *mm);
+  XQInlineFunction(XQUserFunction *func, const XMLCh *prefix, const XMLCh *uri, const XMLCh *name,
+                   unsigned int numArgs, FunctionSignature *signature, ASTNode *instance,
+                   XPath2MemoryManager *mm);
 
   virtual ASTNode *staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
   virtual Result createResult(DynamicContext* context, int flags=0) const;
 
+  const XMLCh *getPrefix() const { return prefix_; }
+  const XMLCh *getURI() const { return uri_; }
+  const XMLCh *getName() const { return name_; }
   unsigned int getNumArgs() const { return numArgs_; }
 
   XQUserFunction *getUserFunction() const { return func_; }
   void setUserFunction(XQUserFunction *func) { func_ = func; }
 
+  FunctionSignature *getSignature() const { return signature_; }
+  void setSignature(FunctionSignature *s) { signature_ = s; }
+
   ASTNode *getInstance() const { return instance_; }
   void setInstance(ASTNode *i) { instance_ = i; }
 
 private:
   XQUserFunction *func_;
+  const XMLCh *prefix_, *uri_, *name_;
   unsigned int numArgs_;
+  FunctionSignature *signature_;
   ASTNode *instance_;
-
-  friend class InlineFunctionResult;
 };
 
 #endif
