@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQPREDICATE_HPP
@@ -100,14 +98,12 @@ private:
 class NumericPredicateFilterResult : public ResultImpl
 {
 public:
-	NumericPredicateFilterResult(const Result &parent, const ASTNode *pred, size_t contextSize);
-	Item::Ptr next(DynamicContext *context);
-	std::string asString(DynamicContext *context, int indent) const;
+  NumericPredicateFilterResult(const Result &parent, const ASTNode *pred, size_t contextSize);
+  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
 private:
-	bool todo_;
-	Result parent_;
-	const ASTNode *pred_;
-	size_t contextSize_;
+  Result parent_;
+  const ASTNode *pred_;
+  size_t contextSize_;
 };
 
 #endif
