@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _DYNAMICCONTEXT_HPP
@@ -26,6 +24,7 @@
 #include <xqilla/items/ATDurationOrDerived.hpp>
 
 class Sequence;
+class Result;
 class Item;
 class URIResolver;
 class VariableStore;
@@ -71,9 +70,9 @@ public:
   /** set the variable store for globally scoped variables */
   virtual void setGlobalVariableStore(const VariableStore *store) = 0;
   /** set the value of an external global variable with the given uri/localname pair */
-  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Sequence &value) = 0;
+  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Result &value) = 0;
   /** set the value of an external global variable with the given QName */
-  virtual void setExternalVariable(const XMLCh *qname, const Sequence &value) = 0;
+  virtual void setExternalVariable(const XMLCh *qname, const Result &value) = 0;
 
   /** get the regex group store */
   virtual const RegexGroupStore* getRegexGroupStore() const = 0;
