@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _VARIABLETYPESTORE_HPP
@@ -29,6 +27,7 @@
 class SequenceType;
 class XPath2MemoryManager;
 class StaticAnalysis;
+class XQGlobalVariable;
 
 /** This is the wrapper class for the variable store, which implements the 
     lookup and scoping of simple variables. */
@@ -51,11 +50,11 @@ public:
 
   /** Declares and/or sets a variable in the global scope. */
   virtual void declareGlobalVar(const XMLCh* namespaceURI, const XMLCh* name,
-                                const StaticAnalysis &src) = 0;
+                                const StaticAnalysis &src, XQGlobalVariable *global) = 0;
 
   /** Gets a variable from the global scope */
-  virtual const StaticAnalysis* getGlobalVar(const XMLCh* namespaceURI,
-                                                      const XMLCh* name) const = 0;
+  virtual const StaticAnalysis* getGlobalVar(const XMLCh* namespaceURI, const XMLCh* name,
+                                             XQGlobalVariable **global = 0) const = 0;
 
   /** Declare a var in the top level scope (A full set of
       these namespaceURI/name pair methods should be made) */
@@ -65,8 +64,8 @@ public:
   /** Looks up the value of a variable in the current scope, using ident as an
       qname. Returns a boolean (true if successful), and the SequenceType value
       of the variable*/
-  virtual const StaticAnalysis *getVar(const XMLCh* namespaceURI,
-                                                const XMLCh* name) const = 0;
+  virtual const StaticAnalysis *getVar(const XMLCh* namespaceURI, const XMLCh* name,
+                                       XQGlobalVariable **global = 0) const = 0;
 };
 
 #endif
