@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef ITEMFACTORYIMPL_HPP
@@ -33,6 +31,7 @@ class XQILLA_API ItemFactoryImpl : public ItemFactory
 {
 public:
   ItemFactoryImpl(const DocumentCache* dc, XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager* memMgr);
+  ItemFactoryImpl(DatatypeLookup *datatypeLookup);
   virtual ~ItemFactoryImpl();
 
   /* @name Atomic type factory methods */
@@ -98,6 +97,7 @@ public:
   virtual ATUntypedAtomic::Ptr createUntypedAtomic(const XMLCh* value, const DynamicContext* context);
 
   virtual ATBooleanOrDerived::Ptr createBoolean(bool value, const DynamicContext* context);
+  virtual ATBooleanOrDerived::Ptr createBoolean(BoolResult value, const DynamicContext* context);
   virtual ATBooleanOrDerived::Ptr createBoolean(const XMLCh* value, const DynamicContext* context);
   virtual ATBooleanOrDerived::Ptr createBooleanOrDerived(const XMLCh* typeURI, const XMLCh* typeName, bool value, const DynamicContext* context);
   virtual ATBooleanOrDerived::Ptr createBooleanOrDerived(const XMLCh* typeURI, const XMLCh* typeName, const XMLCh* value, const DynamicContext* context);
@@ -109,10 +109,7 @@ public:
                                                      const XMLCh *name, const DynamicContext* context);
 
 protected:
-  ItemFactoryImpl(DatatypeLookup *dl);
-
   DatatypeLookup *datatypeLookup_;
-  bool datatypeLookupOwned_;
 };
 
 #endif
