@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: XQContextImpl.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 #ifndef XQCONTEXTIMPL_H
@@ -121,8 +119,8 @@ public:
   virtual void setVariableStore(const VariableStore *store);
   virtual const VariableStore* getGlobalVariableStore() const;
   virtual void setGlobalVariableStore(const VariableStore *store);
-  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Sequence &value);
-  virtual void setExternalVariable(const XMLCh *qname, const Sequence &value);
+  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Result &value);
+  virtual void setExternalVariable(const XMLCh *qname, const Result &value);
 
   virtual const RegexGroupStore* getRegexGroupStore() const;
   virtual void setRegexGroupStore(const RegexGroupStore *store);
@@ -203,7 +201,7 @@ public:
 
   virtual void addCustomFunction(FuncFactory *func);
   virtual void removeCustomFunction(FuncFactory *func);
-  virtual ASTNode *lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v) const;
+  virtual ASTNode *lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v, const LocationInfo *location) const;
 
   virtual void addExternalFunction(const ExternalFunction *func);
   virtual const ExternalFunction *lookUpExternalFunction(const XMLCh *uri, const XMLCh *name, size_t numArgs) const;
@@ -258,6 +256,12 @@ public:
   /** Gets the listener for warning and trace messages */
   virtual MessageListener *getMessageListener() const;
 
+  virtual void setFTCase(FTCaseOption option) { _ftCaseOption = option; }
+  virtual FTCaseOption getFTCase() { return _ftCaseOption; }
+
+  virtual void setModule(XQQuery *module) { _module = module; }
+  virtual XQQuery *getModule() const { return _module; }
+
   /////////////////////////////////////////
   //  XQilla context specific accessors  //
   /////////////////////////////////////////
@@ -393,6 +397,9 @@ protected:
   /** The message listener, for warnings and trace messages */
   MessageListener *_messageListener;
 
+  /** The module */
+  XQQuery *_module;
+
   /** Base URI. This is an absolute URI, used when necessary in the
    * resolution of relative URIs (for example, by the fn:resolve- uri
    * function.) The base URI is always provided by the external environment. */
@@ -470,6 +477,9 @@ protected:
 
   // used for memory management in expression evaluation
   XPath2MemoryManager* _memMgr;
+
+  //Full text related
+  FTCaseOption _ftCaseOption;
 };
 
 #endif
