@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: XQDynamicContextImpl.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 #ifndef XQDYNAMICCONTEXTIMPL_H
@@ -33,7 +31,7 @@
 class XQILLA_API XQDynamicContextImpl : public DynamicContext
 {
 public:
-  XQDynamicContextImpl(XQillaConfiguration *conf, const StaticContext *staticContext,
+  XQDynamicContextImpl(XQillaConfiguration *conf, const StaticContext *staticContext, VarStoreImpl *defaultVarStore,
                        XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager* memMgr);
   ~XQDynamicContextImpl();
 
@@ -114,8 +112,8 @@ public:
   virtual void setVariableStore(const VariableStore *store);
   virtual const VariableStore* getGlobalVariableStore() const;
   virtual void setGlobalVariableStore(const VariableStore *store);
-  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Sequence &value);
-  virtual void setExternalVariable(const XMLCh *qname, const Sequence &value);
+  virtual void setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Result &value);
+  virtual void setExternalVariable(const XMLCh *qname, const Result &value);
 
   virtual const RegexGroupStore* getRegexGroupStore() const;
   virtual void setRegexGroupStore(const RegexGroupStore *store);
@@ -196,7 +194,7 @@ public:
 
   virtual void addCustomFunction(FuncFactory *func);
   virtual void removeCustomFunction(FuncFactory *func);
-  virtual ASTNode *lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v) const;
+  virtual ASTNode *lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v, const LocationInfo *location) const;
 
   virtual void addExternalFunction(const ExternalFunction *func);
   virtual const ExternalFunction *lookUpExternalFunction(const XMLCh *uri, const XMLCh *name, size_t numArgs) const;
@@ -251,6 +249,13 @@ public:
   /** Gets the listener for warning and trace messages */
   virtual MessageListener *getMessageListener() const;
 
+  virtual void setModule(XQQuery *module);
+  virtual XQQuery *getModule() const;
+
+  virtual void setFTCase(FTCaseOption option);
+  virtual FTCaseOption getFTCase();
+
+
   /////////////////////////////////////////
   //  XQilla context specific accessors  //
   /////////////////////////////////////////
@@ -310,9 +315,10 @@ protected:
    * available for reference within the expression. The QName
    * represents the name of the variable, and the Sequence represents its
    * value */
+  VarStoreImpl *_defaultVarStore;
+  bool _defaultVarStoreOwned;
   const VariableStore *_varStore;
   const VariableStore *_globalVarStore;
-  VarStoreImpl _defaultVarStore;
 
   const RegexGroupStore *_regexStore;
 
@@ -331,6 +337,7 @@ protected:
    * a constructor function in the in-scope functions. Constructor 
    * functions are discussed in 3.10.4 Constructor Functions */
   ItemFactory* _itemFactory;
+  bool _itemFactoryOwned;
 
   struct ResolverEntry {
     ResolverEntry() : resolver(0), adopt(false) {}
@@ -406,6 +413,8 @@ inline void XQDynamicContextImpl::setNamespaceBinding(const XMLCh* prefix, const
 { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("You cannot change the static context when using a proxying dynamic context")); }
 inline void XQDynamicContextImpl::setDoLintWarnings(bool enabled)
 { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("You cannot change the static context when using a proxying dynamic context")); }
+inline void XQDynamicContextImpl::setModule(XQQuery *module)
+{ XQThrow2(ContextException,X("XQDynamicContextImpl"), X("You cannot change the static context when using a proxying dynamic context")); }
 
 
 inline const StaticType &XQDynamicContextImpl::getContextItemType() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
@@ -413,14 +422,17 @@ inline const XMLCh* XQDynamicContextImpl::getDefaultFuncNS() const { XQThrow2(Co
 inline StaticContext::NodeSetOrdering XQDynamicContextImpl::getNodeSetOrdering() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline ModuleResolver * XQDynamicContextImpl::getModuleResolver() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline ExternalFunctionResolver * XQDynamicContextImpl::getExternalFunctionResolver() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
-inline ASTNode* XQDynamicContextImpl::lookUpFunction(const XMLCh* uri, const XMLCh* name, const VectorOfASTNodes& v) const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
+inline ASTNode* XQDynamicContextImpl::lookUpFunction(const XMLCh* uri, const XMLCh* name, const VectorOfASTNodes& v, const LocationInfo *location) const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline const ExternalFunction *XQDynamicContextImpl::lookUpExternalFunction(const XMLCh *uri, const XMLCh *name, size_t numArgs) const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline StaticContext::FLWOROrderingMode XQDynamicContextImpl::getDefaultFLWOROrderingMode() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline bool XQDynamicContextImpl::getXPath1CompatibilityMode() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline bool XQDynamicContextImpl::getPreserveBoundarySpace() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline DocumentCache::ValidationMode XQDynamicContextImpl::getRevalidationMode() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 inline bool XQDynamicContextImpl::getDoLintWarnings() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
+inline XQQuery *XQDynamicContextImpl::getModule() const { XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 
+inline void XQDynamicContextImpl::setFTCase(FTCaseOption option){ XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
+inline XQDynamicContextImpl::FTCaseOption XQDynamicContextImpl::getFTCase(){ XQThrow2(ContextException,X("XQDynamicContextImpl"), X("The static context is not available from a proxying dynamic context")); }
 
 inline XQilla::Language XQDynamicContextImpl::getLanguage() const { return _staticContext->getLanguage(); }
 inline const XMLCh* XQDynamicContextImpl::getBaseURI() const { return _staticContext->getBaseURI(); }
