@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _EVENTGENERATOR_HPP
@@ -51,6 +49,8 @@ public:
     }
   }
 
+  static Result createResult(const ASTNode *ast, const QueryPathNode *projection,
+    DynamicContext *context);
 };
 
 class XQILLA_API ASTNodeEventGenerator : public EventGenerator
@@ -65,16 +65,4 @@ protected:
   bool preserveNS_, preserveType_;
 };
 
-class XQILLA_API GenerateEventsResult : public ResultImpl
-{
-public:
-  GenerateEventsResult(const ASTNode *ast, const QueryPathNode *projection);
-
-  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
-
-private:
-  const ASTNode *ast_;
-  const QueryPathNode *projection_;
-};
-
 #endif
