@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef __XPATH2MEMORYMANAGER_HPP
@@ -24,6 +22,7 @@
 
 #include <algorithm>
 #include <assert.h>
+#include <cstddef>
 
 #include <xqilla/framework/XQillaExport.hpp>
 
@@ -165,6 +164,21 @@ public:
     return 0xFFFFFFFF;
   }
 
+  bool operator==(const XQillaAllocator<_Tp>& o) const
+  {
+    return &o == this;
+  }
+
+  bool operator!=(const XQillaAllocator<_Tp>& o) const
+  {
+    return &o != this;
+  }
+
+  bool operator!=(XQillaAllocator<_Tp>& o)
+  {
+    return _memMgr != o._memMgr;
+  }
+
   char _singleton[sizeof(_Tp)];
   XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager* _memMgr;
 };
