@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _DEFAULTTOKENSTORE_HPP
@@ -37,7 +35,7 @@ class DynamicContext;
 class XQILLA_API DefaultTokenStore : public TokenStore
 {
 public:
-  typedef std::vector<TokenInfo::Ptr> Tokens;
+  typedef std::vector<TokenInfo> Tokens;
 
   class TokenEntryStream : public TokenStream
   {
@@ -45,11 +43,13 @@ public:
     TokenEntryStream(const Tokens &tokens)
       : it_(tokens.begin()), end_(tokens.end()) {}
 
-    virtual TokenInfo::Ptr next()
+    virtual TokenInfo next()
     {
       if(it_ != end_)
         return *it_++;
-      return 0;
+      TokenInfo token;
+      memset(&token, 0, sizeof(TokenInfo));
+      return token;
     }
 
   private:
@@ -70,7 +70,7 @@ private:
   class TokenEntry
   {
   public:
-    void addToken(const TokenInfo::Ptr &token)
+    void addToken(const TokenInfo &token)
     {
       tokens_.push_back(token);
     }
@@ -87,8 +87,7 @@ private:
   XERCES_CPP_NAMESPACE_QUALIFIER RefHashTableOf<TokenEntry> tokens_;
   unsigned int numTokens_;
   XPath2MemoryManager *mm_;
-
-  mutable XERCES_CPP_NAMESPACE_QUALIFIER XMLBuffer buffer_;
+  DynamicContext *context_;
 };
 
 #endif
