@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTCONTAINS_HPP
@@ -25,7 +23,6 @@
 #include <xqilla/framework/XQillaExport.hpp>
 
 #include <xqilla/ast/ASTNodeImpl.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 
 class FTSelection;
 
@@ -37,7 +34,8 @@ public:
 
   virtual ASTNode *staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
-  virtual Result createResult(DynamicContext *context, int flags=0) const;
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 
   ASTNode *getArgument() const { return argument_; }
   FTSelection *getSelection() const { return selection_; }
@@ -47,18 +45,6 @@ public:
   void setSelection(FTSelection *s) { selection_ = s; }
   void setIgnore(ASTNode *a) { ignore_ = a; }
 
-private:
-  class XQILLA_API FTContainsResult : public SingleResult
-  {
-  public:
-    FTContainsResult(const FTContains *parent);
-    Item::Ptr getSingleResult(DynamicContext *context) const;
-    std::string asString(DynamicContext *context, int indent) const;
-
-  private:
-    const FTContains *parent_;
-  };
-
 private:
   ASTNode *argument_;
   FTSelection *selection_;
