@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTCONTENT_HPP
@@ -39,12 +37,12 @@ public:
     : FTOption(FTSelection::CONTENT, memMgr), type_(type), arg_(arg) {}
 
   virtual FTSelection *staticResolution(StaticContext *context);
-  virtual FTSelection *staticTyping(StaticContext *context, StaticTyper *styper);
-  virtual FTSelection *optimize(FTContext *context, bool execute) const;
-  virtual AllMatches::Ptr execute(FTContext *ftcontext) const;
+  virtual FTSelection *staticTypingImpl(StaticContext *context);
+  virtual FTSelection *optimize(FTContext *context) const;
+  virtual AllMatches *execute(FTContext *ftcontext) const;
 
   virtual void setArgument(FTSelection *arg) { arg_ = arg; }
-  const FTSelection *getArgument() const { return arg_; }
+  FTSelection *getArgument() const { return arg_; }
   Type getType() const { return type_; }
 
 private:
@@ -55,23 +53,31 @@ private:
 class FTContentAtStartMatches : public AllMatches
 {
 public:
-  FTContentAtStartMatches(const LocationInfo *info, const AllMatches::Ptr &arg)
-    : AllMatches(info), arg_(arg) {}
-  Match::Ptr next(DynamicContext *context);
+  FTContentAtStartMatches(const LocationInfo *info, AllMatches *arg)
+    : AllMatches(info), arg_(arg), includes_() {}
+  ~FTContentAtStartMatches();
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
 
 private:
-  AllMatches::Ptr arg_;
+  AllMatches *arg_;
+  StringMatches includes_;
 };
 
 class FTContentAtEndMatches : public AllMatches
 {
 public:
-  FTContentAtEndMatches(const LocationInfo *info, const AllMatches::Ptr &arg)
-    : AllMatches(info), arg_(arg) {}
-  Match::Ptr next(DynamicContext *context);
+  FTContentAtEndMatches(const LocationInfo *info, AllMatches *arg)
+    : AllMatches(info), arg_(arg), includes_() {}
+  ~FTContentAtEndMatches();
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
 
 private:
-  AllMatches::Ptr arg_;
+  AllMatches *arg_;
+  StringMatches includes_;
 };
 
 #endif
