@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTOPTION_HPP
@@ -36,32 +34,41 @@ public:
 
   virtual void setArgument(FTSelection *arg) = 0;
 
-  static unsigned int tokenUnit(const TokenInfo::Ptr t, FTUnit unit)
+  static unsigned int tokenUnit(const TokenInfo &t, FTUnit unit)
   {
     switch(unit) {
     case WORDS:
-      return t->getPosition();
+      return t.position_;
     case SENTENCES:
-      return t->getSentence();
+      return t.sentence_;
     case PARAGRAPHS:
-      return t->getParagraph();
+      return t.paragraph_;
     }
     return 0;
   }
 
-  static unsigned int tokenDistance(const TokenInfo::Ptr a, const TokenInfo::Ptr b, FTUnit unit)
+  static unsigned int tokenDistance(const TokenInfo &a, const TokenInfo &b, FTUnit unit)
   {
     switch(unit) {
     case WORDS:
-      return (unsigned int)labs((long)a->getPosition() - (long)b->getPosition()) - 1;
+      return (unsigned int)labs((long)a.position_ - (long)b.position_) - 1;
     case SENTENCES:
-      return (unsigned int)labs((long)a->getSentence() - (long)b->getSentence());
+      return (unsigned int)labs((long)a.sentence_ - (long)b.sentence_);
     case PARAGRAPHS:
-      return (unsigned int)labs((long)a->getParagraph() - (long)b->getParagraph());
+      return (unsigned int)labs((long)a.paragraph_ - (long)b.paragraph_);
     }
     return 0;
   }
 
+  static bool lessThanFunc(const TokenInfo &a, const TokenInfo &b)
+  {
+    if (a.paragraph_ != b.paragraph_)
+      return a.paragraph_ < b.paragraph_;
+    if (a.sentence_ != b.sentence_)
+      return a.sentence_ < b.sentence_;
+    return a.position_ < b.position_;
+  }
+
 protected:
   FTOption(Type type, XPath2MemoryManager *memMgr)
     : FTSelection(type, memMgr) {}
