@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTOR_HPP
@@ -33,9 +31,9 @@ public:
   FTOr(FTSelection *left, FTSelection *right, XPath2MemoryManager *memMgr);
 
   virtual FTSelection *staticResolution(StaticContext *context);
-  virtual FTSelection *staticTyping(StaticContext *context, StaticTyper *styper);
-  virtual FTSelection *optimize(FTContext *context, bool execute) const;
-  virtual AllMatches::Ptr execute(FTContext *ftcontext) const;
+  virtual FTSelection *staticTypingImpl(StaticContext *context);
+  virtual FTSelection *optimize(FTContext *context) const;
+  virtual AllMatches *execute(FTContext *ftcontext) const;
 
   const VectorOfFTSelections &getArguments() const { return args_; }
   void addArg(FTSelection *sel) { args_.push_back(sel); }
@@ -48,18 +46,21 @@ class FTDisjunctionMatches : public AllMatches
 {
 public:
   FTDisjunctionMatches(const LocationInfo *info);
+  ~FTDisjunctionMatches();
 
-  void addMatches(const AllMatches::Ptr &m)
+  void addMatches(AllMatches *m)
   {
     args_.push_back(m);
   }
 
-  Match::Ptr next(DynamicContext *context);
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
 
 private:
   bool toDo_;
-  std::vector<AllMatches::Ptr> args_;
-  std::vector<AllMatches::Ptr>::iterator it_;
+  std::vector<AllMatches*> args_;
+  std::vector<AllMatches*>::iterator it_;
 };
 
 #endif
