@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,13 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTSCOPE_HPP
 #define _FTSCOPE_HPP
 
 #include <xqilla/fulltext/FTOption.hpp>
+#include <set>
 
 class XQILLA_API FTScope : public FTOption
 {
@@ -38,12 +37,12 @@ public:
     : FTOption(FTSelection::SCOPE, memMgr), type_(type), unit_(unit), arg_(arg) {}
 
   virtual FTSelection *staticResolution(StaticContext *context);
-  virtual FTSelection *staticTyping(StaticContext *context, StaticTyper *styper);
-  virtual FTSelection *optimize(FTContext *context, bool execute) const;
-  virtual AllMatches::Ptr execute(FTContext *ftcontext) const;
+  virtual FTSelection *staticTypingImpl(StaticContext *context);
+  virtual FTSelection *optimize(FTContext *context) const;
+  virtual AllMatches *execute(FTContext *ftcontext) const;
 
   virtual void setArgument(FTSelection *arg) { arg_ = arg; }
-  const FTSelection *getArgument() const { return arg_; }
+  FTSelection *getArgument() const { return arg_; }
   Type getType() const { return type_; }
   FTUnit getUnit() const { return unit_; }
 
@@ -56,25 +55,35 @@ private:
 class FTScopeSameMatches : public AllMatches
 {
 public:
-  FTScopeSameMatches(const LocationInfo *info, FTOption::FTUnit unit, const AllMatches::Ptr &arg)
-    : AllMatches(info), unit_(unit), arg_(arg) {}
-  Match::Ptr next(DynamicContext *context);
+  FTScopeSameMatches(const LocationInfo *info, FTOption::FTUnit unit, AllMatches *arg)
+    : AllMatches(info), unit_(unit), arg_(arg), excludes_(), unitValue_(0) {}
+  ~FTScopeSameMatches();
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
 
 private:
   FTOption::FTUnit unit_;
-  AllMatches::Ptr arg_;
+  AllMatches *arg_;
+  StringMatches excludes_;
+  unsigned int unitValue_;
 };
 
 class FTScopeDifferentMatches : public AllMatches
 {
 public:
-  FTScopeDifferentMatches(const LocationInfo *info, FTOption::FTUnit unit, const AllMatches::Ptr &arg)
-    : AllMatches(info), unit_(unit), arg_(arg) {}
-  Match::Ptr next(DynamicContext *context);
+  FTScopeDifferentMatches(const LocationInfo *info, FTOption::FTUnit unit, AllMatches *arg)
+    : AllMatches(info), unit_(unit), arg_(arg), excludes_(), unitValuesSeen_() {}
+  ~FTScopeDifferentMatches();
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
 
 private:
   FTOption::FTUnit unit_;
-  AllMatches::Ptr arg_;
+  AllMatches *arg_;
+  StringMatches excludes_;
+  std::set<unsigned int> unitValuesSeen_;
 };
 
 #endif
