@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FTUNARYNOT_HPP
@@ -30,11 +28,12 @@ public:
   FTUnaryNot(FTSelection *arg, XPath2MemoryManager *memMgr);
 
   virtual FTSelection *staticResolution(StaticContext *context);
-  virtual FTSelection *staticTyping(StaticContext *context, StaticTyper *styper);
-  virtual FTSelection *optimize(FTContext *context, bool execute) const;
-  virtual AllMatches::Ptr execute(FTContext *ftcontext) const;
+  virtual FTSelection *staticTypingImpl(StaticContext *context);
+  virtual FTSelection *optimize(FTContext *context) const;
+  virtual AllMatches *execute(FTContext *ftcontext) const;
 
-  const FTSelection *getArgument() const { return arg_; }
+  FTSelection *getArgument() const { return arg_; }
+  void setArgument(FTSelection *expr) { arg_ = expr; }
 
 private:
   FTSelection *arg_;
@@ -43,13 +42,22 @@ private:
 class FTUnaryNotMatches : public AllMatches
 {
 public:
-  FTUnaryNotMatches(const LocationInfo *info, const AllMatches::Ptr &arg)
-    : AllMatches(info), toDo_(true), arg_(arg) {}
-  Match::Ptr next(DynamicContext *context);
+  FTUnaryNotMatches(const LocationInfo *info, AllMatches *arg)
+    : AllMatches(info), toDo_(true), arg_(arg), includes_(), excludes_() {}
+  ~FTUnaryNotMatches();
+  bool next(DynamicContext *context);
+  const StringMatches &getStringIncludes();
+  const StringMatches &getStringExcludes();
+
+private:
+  void addStringIncludes(const StringMatches &sMatches);
+  void addStringExcludes(const StringMatches &sMatches);
 
 private:
   bool toDo_;
-  AllMatches::Ptr arg_;
+  AllMatches *arg_;
+  StringMatches includes_;
+  StringMatches excludes_;
 };
 
 #endif
