@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _MATCH_HPP
@@ -33,11 +31,10 @@ class XQILLA_API StringMatch
 {
 public:
   StringMatch()
-    : queryString(0), queryPos(0), tokenInfo(0), startToken(false), endToken(false) {}
-  StringMatch(const XMLCh *qString, unsigned int qPos, const TokenInfo::Ptr &tInfo)
-    : queryString(qString), queryPos(qPos), tokenInfo(tInfo), startToken(false), endToken(false) {}
+    : queryPos(0), tokenInfo(0), startToken(false), endToken(false) {}
+  StringMatch(unsigned int qPos, const TokenInfo::Ptr &tInfo)
+    : queryPos(qPos), tokenInfo(tInfo), startToken(false), endToken(false) {}
 
-  const XMLCh *queryString;
   unsigned int queryPos;
   TokenInfo::Ptr tokenInfo;
   bool startToken, endToken;
@@ -55,13 +52,13 @@ public:
   const StringMatches &getStringIncludes() const { return includes_; }
   const StringMatches &getStringExcludes() const { return excludes_; }
 
-  void addStringInclude(const XMLCh *queryString, unsigned int queryPos, const TokenInfo::Ptr &tokenInfo)
+  void addStringInclude(unsigned int queryPos, const TokenInfo::Ptr &tokenInfo)
   {
-    includes_.push_back(StringMatch(queryString, queryPos, tokenInfo));
+    includes_.push_back(StringMatch(queryPos, tokenInfo));
   }
-  void addStringExclude(const XMLCh *queryString, unsigned int queryPos, const TokenInfo::Ptr &tokenInfo)
+  void addStringExclude(unsigned int queryPos, const TokenInfo::Ptr &tokenInfo)
   {
-    excludes_.push_back(StringMatch(queryString, queryPos, tokenInfo));
+    excludes_.push_back(StringMatch(queryPos, tokenInfo));
   }
   void addStringInclude(const StringMatch &match)
   {
