@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _TOKENINFO_HPP
@@ -27,20 +25,29 @@
 
 #include <stdlib.h>
 
-class XQILLA_API TokenInfo : public ReferenceCounted
+struct XQILLA_API TokenInfo
 {
-public:
-  typedef RefCountPointer<const TokenInfo> Ptr;
+  const XMLCh *word_;
+  unsigned int position_;
+  unsigned int sentence_;
+  unsigned int paragraph_; 
 
-  virtual ~TokenInfo() {}
+  TokenInfo() 
+  {
+    word_ = 0;
+    position_ = 0;
+    sentence_ = 0;
+    paragraph_ = 0;
+  }
 
-  virtual const XMLCh *getWord() const = 0;
-  virtual unsigned int getPosition() const = 0;
-  virtual unsigned int getSentence() const = 0;
-  virtual unsigned int getParagraph() const = 0;
-
-protected:
-  TokenInfo() {}
+  TokenInfo(const XMLCh *word, unsigned int position, 
+                         unsigned int sentence, unsigned int paragraph) 
+  {
+    word_ = word;
+    position_ = position;
+    sentence_ = sentence;
+    paragraph_ = paragraph;
+  }
 };
 
 class XQILLA_API TokenStream : public ReferenceCounted
@@ -50,7 +57,7 @@ public:
 
   virtual ~TokenStream() {}
 
-  virtual TokenInfo::Ptr next() = 0;
+  virtual TokenInfo next() = 0;
 
 protected:
   TokenStream() {}
