@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,8 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionAbs.hpp 475 2008-01-08 18:47:44Z jpcs $
  */
 
-/*
- fn:abs($arg as numeric?) as numeric?
-*/
-
 #ifndef _FUNCTIONABS_HPP
 #define _FUNCTIONABS_HPP
 
@@ -39,8 +33,8 @@ public:
 
   FunctionAbs(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
 
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
+  virtual Result createResult(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONABS_HPP
+#endif
 
