@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _FUNCTIONADJUSTDATETIMETOTIMEZONE_HPP
@@ -26,12 +24,6 @@
 
 #include <xqilla/ast/XQFunction.hpp>
 
-/**
- * Adjusts a dateTime to a specific timezone, or to no timezone at all
- *
- *  fn:adjust-dateTime-to-timezone(dateTime? $srcval) => dateTime?
- *  fn:adjust-dateTime-to-timezone(dateTime? $srcval, dayTimeDuration? $timezone) => dateTime?
- */
 class XQILLA_API FunctionAdjustDateTimeToTimezone : public XQFunction
 {
 public:
@@ -39,26 +31,14 @@ public:
   static const unsigned int minArgs;
   static const unsigned int maxArgs;
 
-  /**
-   * Constructor
-   */
    FunctionAdjustDateTimeToTimezone(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
 
-  /**
-   * Called during static analysis to determine if statically correct.
-   * Performs constant folding if the function has two arguments, and
-   * they are constant.
-   */
-  virtual ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
 
-  /**
-   * Adjusts a dateTime to a specific timezone, or to no timezone at all
-   */
   Sequence createSequence(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONADJUSTDATETIMETOTIMEZONE_HPP
+#endif
 
 
 
