@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,26 +15,15 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionContains.hpp 475 2008-01-08 18:47:44Z jpcs $
  */
 
-/*
-
-  Contains function
-
-*/
-
 #ifndef _FUNCTIONCONTAINS_HPP
 #define _FUNCTIONCONTAINS_HPP
 
-#include <xqilla/framework/XQillaExport.hpp>
-
-
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
 /** Contains function. */
-class XQILLA_API FunctionContains : public ConstantFoldingFunction
+class XQILLA_API FunctionContains : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -42,9 +31,8 @@ public:
   static const unsigned int maxArgs;
 
   FunctionContains(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
-  
-  /** XPath function. returns true if string1 conatins string2 **/
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
-#endif // _FUNCTIONCONTAINS_HPP
+#endif
