@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,24 +15,14 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
-/*
-
-   default-collation function
-
-*/
-
-
 #ifndef _FUNCTIONDEFAULTCOLLATION_HPP
 #define _FUNCTIONDEFAULTCOLLATION_HPP
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-/** default-collation function*/
-class XQILLA_API FunctionDefaultCollation : public ConstantFoldingFunction
+class XQILLA_API FunctionDefaultCollation : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -41,7 +31,8 @@ public:
 
   FunctionDefaultCollation(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
+  virtual ASTNode *staticResolution(StaticContext *context);
+  virtual Sequence createSequence(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONDEFAULTCOLLATION_HPP
+#endif
