@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,16 +15,8 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionDistinctValues.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
-/*
-
-  Distinct-values function
-  
-*/
-
 #ifndef _FUNCTIONDISTINCTVALUES_HPP
 #define _FUNCTIONDISTINCTVALUES_HPP
 
@@ -32,8 +24,6 @@
 
 #include <xqilla/ast/XQFunction.hpp>
 
-// Distinct values is not really an aggregate function, but since its behaviour is so
-// similar (in terms of sequence validation) we will extend from it.
 class XQILLA_API FunctionDistinctValues : public XQFunction
 {
 public:
@@ -43,7 +33,6 @@ public:
 
   FunctionDistinctValues(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  virtual ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
   virtual Result createResult(DynamicContext* context, int flags=0) const;
 };
