@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,24 +15,14 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionEmpty.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
-/*
-
-  Empty function
-
-*/
-
 #ifndef _FUNCTIONEMPTY_HPP
 #define _FUNCTIONEMPTY_HPP
 
-#include <xqilla/framework/XQillaExport.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
-
-class XQILLA_API FunctionEmpty : public ConstantFoldingFunction
+class XQILLA_API FunctionEmpty : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -43,10 +33,8 @@ public:
 
   ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
-#endif // _FUNCTIONEMPTY_HPP
-
-
-
+#endif
