@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,25 +15,14 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
-/*
-  
-  xf:ends-with(string? $operand1, string? $operand2) => boolean?
-  xf:ends-with(string? $operand1, string? $operand2, anyURI $collationLiteral) => boolean?
-  
-*/
-
 #ifndef _FUNCTIONENDSWITH_HPP
 #define _FUNCTIONENDSWITH_HPP
 
-#include <xqilla/framework/XQillaExport.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
-
-class XQILLA_API FunctionEndsWith : public ConstantFoldingFunction
+class XQILLA_API FunctionEndsWith : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -41,11 +30,8 @@ public:
   static const unsigned int maxArgs;
 
   FunctionEndsWith(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
-
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
-#endif // _FUNCTIONENDSWITH_HPP
-
-
-
+#endif
