@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,16 +15,8 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
-/*
-
-  Error function
-
-*/
-
 #ifndef _FUNCTIONError_HPP
 #define _FUNCTIONError_HPP
 
@@ -35,20 +27,21 @@
 class XQILLA_API FunctionError : public XQFunction
 {
 public:
+  static const XMLCh XMLChXQueryErrorURI[];
+
   static const XMLCh name[];
   static const unsigned int minArgs;
   static const unsigned int maxArgs;
 
   FunctionError(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
 
-  ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
 
   Sequence createSequence(DynamicContext* context, int flags=0) const;
   virtual PendingUpdateList createUpdateList(DynamicContext *context) const;
 };
 
-#endif // _FUNCTIONError_HPP
+#endif
 
 
 
