@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionIdref.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 #ifndef _FUNCTIONIDREF_HPP
@@ -28,12 +26,6 @@
 
 #include <vector>
 
-/** 
- * Function idref
- * 
- * fn:idref(string* $srcval) => element*
- *
- */
 class XQILLA_API FunctionIdref : public XQFunction
 {
 public:
@@ -41,24 +33,12 @@ public:
   static const unsigned int minArgs;
   static const unsigned int maxArgs;
 
-  /**
-   * Constructor
-   */
   FunctionIdref(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
   virtual ASTNode* staticResolution(StaticContext *context);
-  virtual ASTNode *staticTypingImpl(StaticContext *context);
 
-  /** 
-   * Returns the sequence of elements nodes having either an IDREF attribute 
-   * whose value matches the value of one of the items in the value of $srcval 
-   * or an IDREFS attribute whose value contains an IDREF value that matches 
-   * the value of one of the items in the value of $srcval. 
-   *
-   * This function allows reverse navigation from IDs to IDREFs.
-   */
   Sequence createSequence(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONIDREF_HPP
+#endif
 
