@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,23 +15,13 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionImplicitTimezone.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
-/*
-
-   implicit-timezone function
-
-*/
-
-
 #ifndef _FUNCTIONIMPLICITTIMEZONE_HPP
 #define _FUNCTIONIMPLICITTIMEZONE_HPP
 
 #include <xqilla/ast/XQFunction.hpp>
 
-/** implicit-timezone function*/
 class XQILLA_API FunctionImplicitTimezone : public XQFunction
 {
 public:
@@ -41,10 +31,9 @@ public:
 
   FunctionImplicitTimezone(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  virtual ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
 
   Sequence createSequence(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONIMPLICITTIMEZONE_HPP
+#endif
