@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,16 +15,8 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionName.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
-/*
-
-   Name function
-
-*/
-
 #ifndef _FUNCTIONNAME_HPP
 #define _FUNCTIONNAME_HPP
 
@@ -33,7 +25,6 @@
 #include <xqilla/ast/XQFunction.hpp>
 #include <xqilla/runtime/Sequence.hpp>
 
-/** Name function */
 class XQILLA_API FunctionName : public XQFunction
 {
 public:
@@ -43,20 +34,11 @@ public:
 
   FunctionName(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  /**
-   * Called during static analysis to determine if statically correct.
-   * Performs constant folding if the function has an argument, and it
-   * is constant.
-   */
   virtual ASTNode* staticResolution(StaticContext *context);
-  virtual ASTNode *staticTypingImpl(StaticContext *context);
 
-  /** Returns the namespace unaware name of the node. Takes an optional nodelist,
-      and returns an empty string if the node has no name. Uses the current context
-      if a nodelist is not specified. */
   Sequence createSequence(DynamicContext* context, int flags=0) const;
 
 };
 
-#endif // _FUNCTIONNAME_HPP
+#endif
 
