@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,16 +15,8 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
-/*
-
-   Namespace-uri function
-
-*/
-
 #ifndef _FUNCTIONNAMESPACEURI_HPP
 #define _FUNCTIONNAMESPACEURI_HPP
 
@@ -32,7 +24,6 @@
 
 #include <xqilla/ast/XQFunction.hpp>
 
-/** Namespace-uri function */
 class XQILLA_API FunctionNamespaceUri : public XQFunction
 {
 public:
@@ -42,21 +33,11 @@ public:
 
   FunctionNamespaceUri(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  /**
-   * Called during static analysis to determine if statically correct.
-   * Performs constant folding if the function has an argument, and it
-   * is constant.
-   */
   virtual ASTNode* staticResolution(StaticContext *context);
-  virtual ASTNode *staticTypingImpl(StaticContext *context);
 
-  /** Returns the namespace of the node. Takes an optional node,
-      and returns an empty string if the node has no name. Uses the current context
-      if a node is not specified. */
   Sequence createSequence(DynamicContext* context, int flags=0) const;
-
 };
 
-#endif // _FUNCTIONNAMESPACEURI_HPP
+#endif
 
 
