@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,25 +15,16 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionNot.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
-/*
-
-  Not function
-
-*/
-
 #ifndef _FUNCTIONNOT_HPP
 #define _FUNCTIONNOT_HPP
 
 #include <xqilla/framework/XQillaExport.hpp>
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-/** Not operator function. */
-class XQILLA_API FunctionNot : public ConstantFoldingFunction
+class XQILLA_API FunctionNot : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -43,11 +34,9 @@ public:
   FunctionNot(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
   ASTNode* staticResolution(StaticContext *context);
-  virtual ASTNode *staticTypingImpl(StaticContext *context);
-  /** negates the argument **/
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
-
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
-#endif // _FUNCTIONNOT_HPP
+#endif
 
