@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionResolveURI.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 
@@ -26,12 +24,6 @@
 #include <xqilla/framework/XQillaExport.hpp>
 #include <xqilla/ast/XQFunction.hpp>
 
-/***
- *  Returns an absolute anyURI given a base URI and a relative URI.
- *
- *  fn:resolve-uri(anyURI $relative) => anyURI
- *  fn:resolve-uri(anyURI $relative, anyURI $base) => anyURI
- */
 class XQILLA_API FunctionResolveURI : public XQFunction
 {
 public:
@@ -41,19 +33,12 @@ public:
 
   FunctionResolveURI(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
   
-  ASTNode* staticResolution(StaticContext *context);
-  virtual ASTNode *staticTypingImpl(StaticContext *context);
+  virtual ASTNode *staticResolution(StaticContext *context);
 
- /** 
-  * Returns an absolute anyURI given a base URI and a relative URI.
-  **/
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
-
-private:
-  const XMLCh *baseURI_;
+  virtual Sequence createSequence(DynamicContext* context, int flags=0) const;
 };
 
-#endif // _FUNCTIONRESOLVEURI_HPP
+#endif
 
 
 
