@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,23 +15,14 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionStartsWith.hpp 475 2008-01-08 18:47:44Z jpcs $
  */
 
-/*
-
-  StartsWith function
-
-*/
-
 #ifndef _FUNCTIONSTARTSWITH_HPP
 #define _FUNCTIONSTARTSWITH_HPP
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-/** StartsWith function. */
-class XQILLA_API FunctionStartsWith : public ConstantFoldingFunction
+class XQILLA_API FunctionStartsWith : public XQFunction
 {
 public:
   static const XMLCh name[];
@@ -39,10 +30,8 @@ public:
   static const unsigned int maxArgs;
 
   FunctionStartsWith(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr);
-  
-  /** XPath function. returns true if string1 starts with string2 **/
-  Sequence createSequence(DynamicContext* context, int flags=0) const;
-
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
-#endif // _FUNCTIONSTARTSWITH_HPP
+#endif
