@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FunctionUpperCase.hpp 475 2008-01-08 18:47:44Z jpcs $
  */
 
 #ifndef _FUNCTIONUPPERCASE_HPP
@@ -24,7 +22,7 @@
 
 #include <xqilla/framework/XQillaExport.hpp>
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
 /**
  * Returns the given value after translating every lower-case letter to its
@@ -33,7 +31,7 @@
  * xf:upper-case(string? $srcval) =>  string?
  * 
  */
-class XQILLA_API FunctionUpperCase : public ConstantFoldingFunction
+class XQILLA_API FunctionUpperCase : public XQFunction
 {
 public:
   static const XMLCh name[];
