@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,24 +15,21 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef XQILLAFUNCTION_HPP
 #define XQILLAFUNCTION_HPP
 
-#include <xqilla/ast/ConstantFoldingFunction.hpp>
+#include <xqilla/ast/XQFunction.hpp>
 
-class XQILLA_API XQillaFunction : public ConstantFoldingFunction
+class XQILLA_API XQillaFunction : public XQFunction
 {
 public:
   static const XMLCh XQillaPrefix[];
   static const XMLCh XMLChFunctionURI[];
 
 protected:
-  XQillaFunction(const XMLCh *name, unsigned int argsFrom, unsigned int argsTo,
-    const char *paramDecl, const VectorOfASTNodes &args, XPath2MemoryManager *memMgr);
+  XQillaFunction(const XMLCh *name, const char *signature, const VectorOfASTNodes &args, XPath2MemoryManager *memMgr);
 };
 
 #endif
