@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQILLAFUNCTIONREF_HPP
@@ -30,6 +28,7 @@
 
 class DynamicContext;
 class LocationInfo;
+class FunctionSignature;
 
 typedef std::vector<Result> VectorOfResults;
 
@@ -47,6 +46,7 @@ public:
 
   virtual ATQNameOrDerived::Ptr getName(const DynamicContext *context) const = 0;
   virtual size_t getNumArgs() const = 0;
+  virtual const FunctionSignature *getSignature() const = 0;
   virtual Result execute(const VectorOfResults &args, DynamicContext *context, const LocationInfo *location) const = 0;
   virtual FunctionRef::Ptr partialApply(const Result &arg, unsigned int argNum, DynamicContext *context, const LocationInfo *location) const = 0;
 
