@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _ATDECIMALORDERIVEDIMPL_HPP
@@ -100,6 +98,35 @@ public:
   /** Returns the absolute value of this Numeric */
   virtual Numeric::Ptr abs(const DynamicContext* context) const;
 
+  /** Returns the square root of this Numeric */
+  virtual Numeric::Ptr sqrt(const DynamicContext* context) const;
+
+  /** Returns the square root of this Numeric */
+  virtual Numeric::Ptr sin(const DynamicContext* context) const;
+
+  /** Returns the cosinus of this Numeric */
+ virtual Numeric::Ptr cos(const DynamicContext* context) const;
+
+ /** Returns the tangens of this Numeric */
+ virtual Numeric::Ptr tan(const DynamicContext* context) const;
+
+ /** Returns the arcsinus of this Numeric */
+ virtual Numeric::Ptr asin(const DynamicContext* context) const;
+
+ /** Returns the arccosinux of this Numeric */
+ virtual Numeric::Ptr acos(const DynamicContext* context) const;
+
+ /** Returns the arctangens of this Numeric */
+ virtual Numeric::Ptr atan(const DynamicContext* context) const;
+
+  /** Returns the ln of this Numeric */
+  virtual Numeric::Ptr log(const DynamicContext* context) const;
+
+  /** Returns the exp of this Numeric */
+  virtual Numeric::Ptr exp(const DynamicContext* context) const;
+
+  virtual Numeric::Ptr power(const Numeric::Ptr &other, const DynamicContext* context) const;
+
   /** Does this Numeric have value 0? */
   virtual bool isZero() const;
 
