@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: Except.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 /*
@@ -47,20 +45,6 @@ public:
   Result createResult(DynamicContext* context, int flags=0) const;
 
 private:
-  class ExceptResult : public ResultImpl
-  {
-  public:
-    ExceptResult(const Except *op, int flags);
-
-    Item::Ptr next(DynamicContext *context);
-  private:
-    const Except *_op;
-    unsigned int _flags;
-    bool _toDo;
-    Result _result;
-    ResultBuffer _excpt;
-  };
-
   bool sortAdded_;
 };
 
