@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: NodeComparison.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 #ifndef _NODECOMPARISON_HPP
@@ -25,9 +23,7 @@
 #include <xqilla/framework/XQillaExport.hpp>
 
 #include <xqilla/ast/XQOperator.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 
-/** NodeComparison operator function*/
 class XQILLA_API NodeComparison : public XQOperator
 {
 public:
@@ -37,18 +33,8 @@ public:
   
   ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
-  Result createResult(DynamicContext* context, int flags=0) const;
-
-private:
-  class NodeComparisonResult : public SingleResult
-  {
-  public:
-    NodeComparisonResult(const NodeComparison *op);
-
-    Item::Ptr getSingleResult(DynamicContext *context) const;
-  private:
-    const NodeComparison *_op;
-  };
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 };
 
 #endif
