@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: OrderComparison.hpp 659 2008-10-06 00:11:22Z jpcs $
  */
 
 #ifndef _ORDERCOMPARISON_HPP
@@ -25,7 +23,6 @@
 #include <xqilla/framework/XQillaExport.hpp>
 
 #include <xqilla/ast/XQOperator.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 
 class XQILLA_API OrderComparison : public XQOperator
 {
@@ -37,21 +34,12 @@ public:
 
   ASTNode* staticResolution(StaticContext *context);
   virtual ASTNode *staticTypingImpl(StaticContext *context);
-  Result createResult(DynamicContext* context, int flags=0) const;
+  virtual BoolResult boolResult(DynamicContext* context) const;
+  virtual Result createResult(DynamicContext* context, int flags) const;
 
   bool getTestBefore() const;
 
 private:
-  class OrderComparisonResult : public SingleResult
-  {
-  public:
-    OrderComparisonResult(const OrderComparison *op);
-
-    Item::Ptr getSingleResult(DynamicContext *context) const;
-  private:
-    const OrderComparison *_op;
-  };
-
   bool _testBefore;
 };
 
