@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _ASTVISITOR_HPP
@@ -33,10 +31,10 @@
 #include <xqilla/ast/XQStep.hpp>
 #include <xqilla/ast/XQVariable.hpp>
 #include <xqilla/ast/XQIf.hpp>
-#include <xqilla/ast/XQInstanceOf.hpp>
 #include <xqilla/ast/XQCastableAs.hpp>
 #include <xqilla/ast/XQCastAs.hpp>
 #include <xqilla/ast/XQTreatAs.hpp>
+#include <xqilla/ast/XQFunctionCoercion.hpp>
 #include <xqilla/ast/XQOperator.hpp>
 #include <xqilla/ast/XQContextItem.hpp>
 #include <xqilla/ast/XQDOMConstructor.hpp>
@@ -49,7 +47,6 @@
 #include <xqilla/ast/XQOrderingChange.hpp>
 #include <xqilla/ast/XQAtomize.hpp>
 #include <xqilla/ast/XQEffectiveBooleanValue.hpp>
-#include <xqilla/ast/XPath1Compat.hpp>
 #include <xqilla/ast/ConvertFunctionArg.hpp>
 #include <xqilla/ast/XQDocumentOrder.hpp>
 #include <xqilla/ast/XQPredicate.hpp>
@@ -64,12 +61,14 @@
 #include <xqilla/ast/XQInlineFunction.hpp>
 #include <xqilla/ast/XQFunctionRef.hpp>
 #include <xqilla/ast/XQFunctionDeref.hpp>
+#include <xqilla/ast/XQPartialApply.hpp>
 #include <xqilla/ast/XQMap.hpp>
 
 #include <xqilla/ast/ContextTuple.hpp>
 #include <xqilla/ast/ForTuple.hpp>
 #include <xqilla/ast/LetTuple.hpp>
 #include <xqilla/ast/WhereTuple.hpp>
+#include <xqilla/ast/CountTuple.hpp>
 #include <xqilla/ast/OrderByTuple.hpp>
 #include <xqilla/debug/TupleDebugHook.hpp>
 
@@ -88,6 +87,18 @@
 #include <xqilla/update/UApplyUpdates.hpp>
 
 #include <xqilla/fulltext/FTContains.hpp>
+#include <xqilla/fulltext/FTSelection.hpp>
+#include <xqilla/fulltext/FTWords.hpp>
+#include <xqilla/fulltext/FTOr.hpp>
+#include <xqilla/fulltext/FTAnd.hpp>
+#include <xqilla/fulltext/FTMildnot.hpp>
+#include <xqilla/fulltext/FTUnaryNot.hpp>
+#include <xqilla/fulltext/FTOrder.hpp>
+#include <xqilla/fulltext/FTDistance.hpp>
+#include <xqilla/fulltext/FTRange.hpp>
+#include <xqilla/fulltext/FTScope.hpp>
+#include <xqilla/fulltext/FTContent.hpp>
+#include <xqilla/fulltext/FTWindow.hpp>
 
 #define ALL_ASTVISITOR_METHODS() \
   virtual XQGlobalVariable *optimizeGlobalVar(XQGlobalVariable *item); \
@@ -103,10 +114,10 @@
   virtual ASTNode *optimizeStep(XQStep *item); \
   virtual ASTNode *optimizeVariable(XQVariable *item); \
   virtual ASTNode *optimizeIf(XQIf *item); \
-  virtual ASTNode *optimizeInstanceOf(XQInstanceOf *item); \
   virtual ASTNode *optimizeCastableAs(XQCastableAs *item); \
   virtual ASTNode *optimizeCastAs(XQCastAs *item); \
   virtual ASTNode *optimizeTreatAs(XQTreatAs *item); \
+  virtual ASTNode *optimizeFunctionCoercion(XQFunctionCoercion *item); \
   virtual ASTNode *optimizeOperator(XQOperator *item); \
   virtual ASTNode *optimizeContextItem(XQContextItem *item); \
   virtual ASTNode *optimizeReturn(XQReturn *item); \
@@ -119,7 +130,6 @@
   virtual ASTNode *optimizeOrderingChange(XQOrderingChange *item); \
   virtual ASTNode *optimizeAtomize(XQAtomize *item); \
   virtual ASTNode *optimizeEffectiveBooleanValue(XQEffectiveBooleanValue *item); \
-  virtual ASTNode *optimizeXPath1CompatConvertFunctionArg(XPath1CompatConvertFunctionArg *item); \
   virtual ASTNode *optimizePromoteUntyped(XQPromoteUntyped *item); \
   virtual ASTNode *optimizePromoteNumeric(XQPromoteNumeric *item); \
   virtual ASTNode *optimizePromoteAnyURI(XQPromoteAnyURI *item); \
@@ -140,6 +150,7 @@
   virtual ASTNode *optimizeInlineFunction(XQInlineFunction *item); \
   virtual ASTNode *optimizeFunctionRef(XQFunctionRef *item); \
   virtual ASTNode *optimizeFunctionDeref(XQFunctionDeref *item); \
+  virtual ASTNode *optimizePartialApply(XQPartialApply *item); \
   virtual ASTNode *optimizeMap(XQMap *item); \
 \
   virtual ASTNode *optimizeUDelete(UDelete *item); \
@@ -155,6 +166,20 @@
   virtual ASTNode *optimizeUApplyUpdates(UApplyUpdates *item); \
 \
   virtual ASTNode *optimizeFTContains(FTContains *item); \
+  virtual FTSelection *optimizeUnknownFTSelection(FTSelection *item); \
+  virtual FTSelection *optimizeFTWords(FTWords *selection); \
+  virtual FTSelection *optimizeFTWord(FTWord *selection); \
+  virtual FTSelection *optimizeFTOr(FTOr *selection); \
+  virtual FTSelection *optimizeFTAnd(FTAnd *selection); \
+  virtual FTSelection *optimizeFTMildnot(FTMildnot *selection); \
+  virtual FTSelection *optimizeFTUnaryNot(FTUnaryNot *selection); \
+  virtual FTSelection *optimizeFTOrder(FTOrder *selection); \
+  virtual FTSelection *optimizeFTDistance(FTDistance *selection); \
+  virtual FTSelection *optimizeFTDistanceLiteral(FTDistanceLiteral *selection); \
+  virtual FTSelection *optimizeFTScope(FTScope *selection); \
+  virtual FTSelection *optimizeFTContent(FTContent *selection); \
+  virtual FTSelection *optimizeFTWindow(FTWindow *selection); \
+  virtual FTSelection *optimizeFTWindowLiteral(FTWindowLiteral *selection); \
 \
   virtual TupleNode *optimizeUnknownTupleNode(TupleNode *item); \
   virtual TupleNode *optimizeContextTuple(ContextTuple *item); \
@@ -162,6 +187,7 @@
   virtual TupleNode *optimizeLetTuple(LetTuple *item); \
   virtual TupleNode *optimizeWhereTuple(WhereTuple *item); \
   virtual TupleNode *optimizeOrderByTuple(OrderByTuple *item); \
+  virtual TupleNode *optimizeCountTuple(CountTuple *item); \
   virtual TupleNode *optimizeTupleDebugHook(TupleDebugHook *item);
 
 
@@ -175,6 +201,7 @@ protected:
   virtual void optimize(XQQuery *query);
   virtual ASTNode *optimize(ASTNode *item);
   virtual TupleNode *optimizeTupleNode(TupleNode *item);
+  virtual FTSelection *optimizeFTSelection(FTSelection *selection);
 
   ALL_ASTVISITOR_METHODS()
 };
