@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _OPTIMIZER_HPP
@@ -28,6 +26,7 @@ class ASTNode;
 class DynamicContext;
 
 class XQQuery;
+class XQUserFunction;
 
 class XQILLA_API Optimizer
 {
@@ -37,11 +36,13 @@ public:
 
   void startOptimize(XQQuery *query);
   ASTNode *startOptimize(ASTNode *item);
+  XQUserFunction *startOptimize(XQUserFunction *item);
   void reset();
 
 protected:
   virtual void optimize(XQQuery *query) = 0;
   virtual ASTNode *optimize(ASTNode *item) = 0;
+  virtual XQUserFunction *optimizeFunctionDef(XQUserFunction *item) = 0;
   virtual void resetInternal();
 
 private:
@@ -56,12 +57,11 @@ class XQILLA_API StaticResolver : public Optimizer
 public:
   StaticResolver(DynamicContext *xpc, Optimizer *parent = 0)
     : Optimizer(parent), xpc_(xpc) {}
-  virtual ~StaticResolver() {}
 
 protected:
   virtual void optimize(XQQuery *query);
   virtual ASTNode *optimize(ASTNode *item);
-  virtual void resetInternal() {}
+  virtual XQUserFunction *optimizeFunctionDef(XQUserFunction *item);
 
 private:
   DynamicContext *xpc_;
