@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _EMPTYRESULT_HPP
@@ -32,7 +30,8 @@ public:
   Item::Ptr next(DynamicContext *context);
 
   virtual Sequence toSequence(DynamicContext *context);
-  virtual ResultBufferImpl *toResultBuffer(unsigned int readCount);
+  virtual void toResultBuffer(unsigned int readCount, ResultBuffer &buffer);
+  virtual void skip(unsigned count, DynamicContext *context);
 };
 
 #endif
