@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _RESULTBUFFER_HPP
@@ -31,7 +29,7 @@ class XQILLA_API ResultBuffer
 public:
   ResultBuffer(const Result &result, unsigned int readCount = ResultBufferImpl::UNLIMITED_COUNT);
   ResultBuffer(const Item::Ptr &item, unsigned int readCount = ResultBufferImpl::UNLIMITED_COUNT);
-  ResultBuffer(ResultBufferImpl *impl);
+  ResultBuffer(ResultBufferImpl *impl, unsigned start = 0);
 
   /// Creates a result that returns the same items as the one used to construct this buffer
   Result createResult();
@@ -43,6 +41,7 @@ public:
 
 private:
   ResultBufferImpl::Ptr _impl;
+  unsigned _start;
 };
 
 #endif
