@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _RESULTIMPL_HPP
@@ -29,7 +27,7 @@
 class DynamicContext;
 class Sequence;
 class Result;
-class ResultBufferImpl;
+class ResultBuffer;
 
 /** A lazily evaluated query result */
 class XQILLA_API ResultImpl : public LocationInfo
@@ -46,11 +44,14 @@ public:
   /// Gets the next item, or returns a tail call Result which will give the next item.
   virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
 
-  /// Cast operator to a Sequence. Only works properly before next() has been called.
+  /// Cast operator to a Sequence
   virtual Sequence toSequence(DynamicContext *context);
 
   /// Returns a result buffer of the result
-  virtual ResultBufferImpl *toResultBuffer(unsigned int readCount);
+  virtual void toResultBuffer(unsigned int readCount, ResultBuffer &buffer);
+
+  /// Skip ahead
+  virtual void skip(unsigned count, DynamicContext *context);
 
 protected:
   ResultImpl(const LocationInfo *o);
