@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _SEQUENCERESULT_HPP
@@ -32,14 +30,16 @@ public:
   SequenceResult(const LocationInfo *o, const Sequence &seq);
 
   /// Get the next item from the iterator
-  virtual Item::Ptr next(DynamicContext *context);
+  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context);
 
   /// Cast operator to a Sequence
   virtual Sequence toSequence(DynamicContext *context = 0);
 
+  virtual void skip(unsigned count, DynamicContext *context);
+
 private:
-  Sequence::iterator it_;
   Sequence seq_;
+  size_t i_;
 };
 
 #endif
