@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQILLA_07637_HPP
@@ -29,15 +27,21 @@
 #include <xercesc/util/XMemory.hpp>
 
 class DynamicContext;
-class XQQuery;
 class XPath2MemoryManager;
 class XQillaConfiguration;
+class Optimizer;
+class DelayedFuncFactory;
+class XQQuery;
 
 XERCES_CPP_NAMESPACE_BEGIN
 class InputSource;
 class XMLBuffer;
 XERCES_CPP_NAMESPACE_END
 
+namespace CompileDelayedModule {
+void compile(const XMLCh *queryFile);
+}
+
 /**
  * Provides factory methods for creating XQQuery and DynamicContext objects.
  * 
@@ -53,7 +57,8 @@ public:
     NO_STATIC_RESOLUTION = 0x1, ///< Don't perform static resolution or any optimization
     NO_ADOPT_CONTEXT     = 0x2, ///< Don't adopt the context and delete it when the XQQuery is deleted
     DEBUG_QUERY          = 0x4, ///< Build debugging hooks into the query to enable the use of DebugListener
-    NO_OPTIMIZATION      = 0x8  ///< Don't perform optimization
+    NO_OPTIMIZATION      = 0x8, ///< Don't perform optimization
+    NO_DEFAULT_MODULES   = 0x10 ///< Don't automatically import the default modules
   };
 
   /// Enumeration used to select a language to parse
@@ -64,12 +69,22 @@ public:
     UPDATE     = 0x04,
     EXTENSIONS = 0x08,
     XSLT2      = 0x10,
+    VERSION3   = 0x20,
 
-    XQUERY_FULLTEXT        = (XQUERY | FULLTEXT),
-    XPATH2_FULLTEXT        = (XPATH2 | FULLTEXT),
-    XQUERY_UPDATE          = (XQUERY | UPDATE),
-    XQUERY_FULLTEXT_UPDATE = (XQUERY | FULLTEXT | UPDATE),
-    XSLT2_FULLTEXT         = (XSLT2 | FULLTEXT)
+    XQUERY_FULLTEXT          = (XQUERY | FULLTEXT),
+    XQUERY_UPDATE            = (XQUERY | UPDATE),
+    XQUERY_FULLTEXT_UPDATE   = (XQUERY | FULLTEXT | UPDATE),
+    XPATH2_FULLTEXT          = (XPATH2 | FULLTEXT),
+    XSLT2_FULLTEXT           = (XSLT2 | FULLTEXT),
+
+    XQUERY3                  = (XQUERY | VERSION3),
+    XQUERY3_FULLTEXT         = (XQUERY | VERSION3 | FULLTEXT),
+    XQUERY3_UPDATE           = (XQUERY | VERSION3 | UPDATE),
+    XQUERY3_FULLTEXT_UPDATE  = (XQUERY | VERSION3 | FULLTEXT | UPDATE),
+    XPATH3                   = (XPATH2 | VERSION3),
+    XPATH3_FULLTEXT          = (XPATH2 | VERSION3 | FULLTEXT),
+    XSLT3                    = (XSLT2 | VERSION3),
+    XSLT3_FULLTEXT           = (XSLT2 | VERSION3 | FULLTEXT),
   };
                                                                                                                                                               
   /**
@@ -110,7 +125,8 @@ public:
                         const XMLCh* queryFile = NULL,
                         unsigned int flags = 0,
                         XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *memMgr =
-                        XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager);
+                        XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager,
+                        XQQuery *result = 0);
 
   /**
    * Parse the expression from the given InputSource.
@@ -130,7 +146,8 @@ public:
                         DynamicContext* context = 0,
                         unsigned int flags = 0,
                         XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *memMgr =
-                        XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager);
+                        XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager,
+                        XQQuery *result = 0);
 
   /**
    * Parse the expression residing at the given URL.
@@ -150,7 +167,8 @@ public:
                                DynamicContext* context = 0,
                                unsigned int flags = 0,
                                XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *memMgr =
-                               XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager);
+                               XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager,
+                               XQQuery *result = 0);
 
   //@}
 
@@ -176,6 +194,13 @@ private:
   static bool readQuery(const XERCES_CPP_NAMESPACE_QUALIFIER InputSource& querySrc, 
                         XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager* memMgr, 
                         XERCES_CPP_NAMESPACE_QUALIFIER XMLBuffer& queryText);
+  static Optimizer *createOptimizer(DynamicContext *context, unsigned int flags);
+
+  static void compileDelayedModule(const XMLCh* queryFile, XERCES_CPP_NAMESPACE_QUALIFIER MemoryManager *memMgr =
+                                   XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::fgMemoryManager);
+
+  friend void CompileDelayedModule::compile(const XMLCh *queryFile);
+  friend class DelayedFuncFactory;
 };
 
 #endif
