@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _UTRANSFORM_HPP
@@ -65,23 +63,6 @@ public:
   DocumentCache::ValidationMode getRevalidationMode() const { return valMode_; }
 
 protected:
-  class TransformResult : public ResultImpl
-  {
-  public:
-    TransformResult(const UTransform *transform, DynamicContext *context);
-
-    Item::Ptr next(DynamicContext *context);
-    std::string asString(DynamicContext *context, int indent) const
-    { return ""; }
-
-  private:
-    const UTransform *transform_;
-    bool toDo_;
-
-    VarStoreImpl scope_;
-    Result result_;
-  };
-
   VectorOfCopyBinding* bindings_;
   ASTNode *modify_;
   ASTNode *return_;
