@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,14 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/ForTuple.hpp>
 #include <xqilla/ast/LetTuple.hpp>
+#include <xqilla/ast/CountTuple.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/context/VariableTypeStore.hpp>
+#include <xqilla/context/ContextHelpers.hpp>
 #include <xqilla/runtime/ResultBuffer.hpp>
 #include <xqilla/runtime/ClosureResult.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
@@ -82,12 +82,18 @@ static bool canPushLetBack(TupleNode *ancestor, const StaticAnalysis &exprSrc)
       return canPushLetBack(f->getParent(), exprSrc);
     break;
   }
+  case TupleNode::COUNT: {
+    CountTuple *f = (CountTuple*)ancestor;
+    if(!exprSrc.isVariableUsed(f->getVarURI(), f->getVarName()))
+      return canPushLetBack(f->getParent(), exprSrc);
+    break;
+  }
   case TupleNode::ORDER_BY:
   case TupleNode::WHERE:
     return canPushLetBack(ancestor->getParent(), exprSrc);
   case TupleNode::CONTEXT_TUPLE:
     break;
-  default:
+  case TupleNode::DEBUG_HOOK:
     return true;
   }
   return false;
@@ -161,7 +167,8 @@ public:
       return false;
 
     // TBD Use counts for the variable - jpcs
-    values_ = ResultBuffer(ClosureResult::create(ast_->getExpression(), context, parent_));
+    AutoVariableStoreReset vsReset(context, parent_);
+    values_ = ResultBuffer(ast_->getExpression()->createResult(context));
     return true;
   }
 
