@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <assert.h>
@@ -27,9 +25,10 @@
 #include <xqilla/context/ItemFactory.hpp>
 #include <xqilla/context/StaticContext.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/utils/XStr.hpp>
 
+#include <xercesc/validators/schema/SchemaSymbols.hpp>
+
 XERCES_CPP_NAMESPACE_USE;
 
 const unsigned int StaticType::UNLIMITED = (unsigned int)-1;
@@ -38,7 +37,7 @@ StaticType StaticType::create(const XMLCh *uri, const XMLCh *name, const StaticC
                               bool &isExact)
 {
   if(XPath2Utils::equals(name, AnyAtomicType::fgDT_ANYATOMICTYPE) &&
-     XPath2Utils::equals(uri, FunctionConstructor::XMLChXPath2DatatypesURI)) {
+     XPath2Utils::equals(uri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
     isExact = true;
     return ANY_ATOMIC_TYPE;
   }
