@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -186,7 +184,7 @@ Item::Ptr AnalyzeStringResult::next(DynamicContext *context)
 
       if(tokStart < matchStart) {
         const XMLCh *str = XPath2Utils::subString(input_, tokStart, matchStart - tokStart, mm);
-        strings_.push_back(pair<const XMLCh*, Match*>(str, 0));
+        strings_.push_back(pair<const XMLCh*, Match*>(str, static_cast<Match*>(0)));
       }
 
       const XMLCh *str = XPath2Utils::subString(input_, matchStart, matchEnd - matchStart, mm);
@@ -198,7 +196,7 @@ Item::Ptr AnalyzeStringResult::next(DynamicContext *context)
     if(tokStart < (int) length) {
       const XMLCh *str = XPath2Utils::subString(input_, tokStart,
                                                 (unsigned int)(length - tokStart), mm);
-      strings_.push_back(pair<const XMLCh*, Match*>(str, 0));
+      strings_.push_back(pair<const XMLCh*, Match*>(str, static_cast<Match*>(0)));
     }
   }
 
@@ -275,12 +273,8 @@ public:
     context->setContextPosition(matchPos);
     context->setContextItem(context->getItemFactory()->createString(matchString, context));
 
-    if(match) {
-      return ClosureResult::create(ast_->getMatch(), context);
-    }
-    else {
-      return ClosureResult::create(ast_->getNonMatch(), context);
-    }
+    if(match) return ast_->getMatch()->createResult(context);
+    return ast_->getNonMatch()->createResult(context);
   }
 
 private:
@@ -289,6 +283,6 @@ private:
 
 Result XQAnalyzeString::createResult(DynamicContext* context, int flags) const
 {
-  return new XslAnalyzeStringResult(this);
+  return ClosureResult::create(getStaticAnalysis(), context, new XslAnalyzeStringResult(this));
 }
 
