@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQApplyTemplates.hpp>
@@ -28,6 +26,8 @@
 #include <xqilla/utils/XPath2NSUtils.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
 #include <xqilla/runtime/ClosureResult.hpp>
+#include <xqilla/schema/SequenceType.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 
 #include <xercesc/framework/XMLBuffer.hpp>
 
@@ -196,9 +196,9 @@ Result XQApplyTemplates::executeTemplate(const XQUserFunction *tplt, const Templ
   if(scope == 0) scope = &localscope;
 
   // Check to see if we set the correct parameters for the template
-  if(tplt->getArgumentSpecs() != 0) {
-    XQUserFunction::ArgumentSpecs::const_iterator argIt = tplt->getArgumentSpecs()->begin();
-    for(; argIt != tplt->getArgumentSpecs()->end(); ++argIt) {
+  if(tplt->getSignature()->argSpecs != 0) {
+    ArgumentSpecs::const_iterator argIt = tplt->getSignature()->argSpecs->begin();
+    for(; argIt != tplt->getSignature()->argSpecs->end(); ++argIt) {
       bool found = false;
 
       if(args != 0) {
@@ -207,7 +207,7 @@ Result XQApplyTemplates::executeTemplate(const XQUserFunction *tplt, const Templ
           if(XPath2Utils::equals((*it)->name, (*argIt)->getName()) &&
              XPath2Utils::equals((*it)->uri, (*argIt)->getURI())) {
             if(scope == &localscope)
-              localscope.setVar((*it)->uri, (*it)->name, ClosureResult::create((*it)->value, context));
+              localscope.setVar((*it)->uri, (*it)->name, (*it)->value->createResult(context));
             found = true;
             break;
           }
@@ -231,7 +231,8 @@ Result XQApplyTemplates::executeTemplate(const XQUserFunction *tplt, const Templ
     }
   }
 
-  return ClosureResult::create(tplt->getTemplateInstance(), context, scope);
+  AutoVariableStoreReset vsReset(context, scope);
+  return tplt->getTemplateInstance()->createResult(context);
 }
 
 void XQApplyTemplates::evaluateArguments(VarStoreImpl &scope, DynamicContext *context) const
@@ -241,7 +242,7 @@ void XQApplyTemplates::evaluateArguments(VarStoreImpl &scope, DynamicContext *co
   TemplateArguments::const_iterator it;
   for(it = args_->begin(); it != args_->end(); ++it) {
     // TBD variable use count - jpcs
-    scope.setVar((*it)->uri, (*it)->name, ClosureResult::create((*it)->value, context));
+    scope.setVar((*it)->uri, (*it)->name, (*it)->value->createResult(context));
   }
 }
 
@@ -352,5 +353,6 @@ private:
 
 Result XQApplyTemplates::createResult(DynamicContext *context, int flags) const
 {
-  return new ApplyTemplatesResult(this, expr_->createResult(context), templates_, context);
+  return ClosureResult::create(getStaticAnalysis(), context,
+    new ApplyTemplatesResult(this, expr_->createResult(context), templates_, context));
 }
