@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQCallTemplate.hpp>
@@ -27,6 +25,7 @@
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 
 #include <xercesc/framework/XMLBuffer.hpp>
 
@@ -146,9 +145,9 @@ ASTNode* XQCallTemplate::staticResolution(StaticContext *context)
   vector<bool> argUsed(args_ ? args_->size() : 0, false);
   VectorOfASTNodes newArgs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
 
-  if(tplt->getArgumentSpecs() != 0) {
-    XQUserFunction::ArgumentSpecs::const_iterator argIt;
-    for(argIt = tplt->getArgumentSpecs()->begin(); argIt != tplt->getArgumentSpecs()->end(); ++argIt) {
+  if(tplt->getSignature()->argSpecs != 0) {
+    ArgumentSpecs::const_iterator argIt;
+    for(argIt = tplt->getSignature()->argSpecs->begin(); argIt != tplt->getSignature()->argSpecs->end(); ++argIt) {
 
       bool found = false;
       if(args_ != 0) {
