@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <assert.h>
@@ -65,22 +63,21 @@ ASTNode *XQContextItem::staticTypingImpl(StaticContext *context)
 }
 
 Result XQContextItem::createResult(DynamicContext* context, int flags) const
-{
-  return new ContextItemResult(this);
-}
-
-ContextItemResult::ContextItemResult(const LocationInfo *location)
-  : SingleResult(location)
-{
-}
-
-Item::Ptr ContextItemResult::getSingleResult(DynamicContext *context) const
 {
   const Item::Ptr item = context->getContextItem();
-  if(item == NULLRCP) {
-    XQThrow(DynamicErrorException,X("XQContextItem::ContextItemResult::getSingleResult"),
+  if(item.isNull()) {
+    XQThrow(DynamicErrorException,X("XQContextItem::ContextItemResult::createResult"),
              X("It is an error for the context item to be undefined when using it [err:XPDY0002]"));
   }
   return item;
 }
 
+Result XQContextItem::result(DynamicContext *context, const LocationInfo *info)
+{
+  const Item::Ptr item = context->getContextItem();
+  if(item.isNull()) {
+    XQThrow3(DynamicErrorException,X("XQContextItem::result"),
+             X("It is an error for the context item to be undefined when using it [err:XPDY0002]"), info);
+  }
+  return item;
+}
