@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQFunctionDeref.hpp>
@@ -38,17 +36,15 @@ XQFunctionDeref::XQFunctionDeref(ASTNode *expr, VectorOfASTNodes *args, XPath2Me
 {
 }
 
-static const XMLCh err_TBD[] = { 'e', 'r', 'r', ':', 'T', 'B', 'D', 0 };
-
 ASTNode *XQFunctionDeref::staticResolution(StaticContext *context)
 {
   XPath2MemoryManager *mm = context->getMemoryManager();
 
   SequenceType *seqType = new (mm) SequenceType(new (mm) SequenceType::ItemType(SequenceType::ItemType::TEST_FUNCTION),
-                                                SequenceType::QUESTION_MARK);
+                                                SequenceType::EXACTLY_ONE);
   seqType->setLocationInfo(this);
 
-  expr_ = new (mm) XQTreatAs(expr_, seqType, mm, err_TBD);
+  expr_ = new (mm) XQTreatAs(expr_, seqType, mm);
   expr_->setLocationInfo(this);
   expr_ = expr_->staticResolution(context);
 
@@ -80,11 +76,6 @@ ASTNode *XQFunctionDeref::staticTypingImpl(StaticContext *context)
     _src.getStaticType() = StaticType(StaticType::ITEM_TYPE, 0, StaticType::UNLIMITED);
   }
 
-  // TBD Work out if these are really needed - jpcs
-  _src.contextItemUsed(true);
-  _src.contextPositionUsed(true);
-  _src.contextSizeUsed(true);
-
   return this;
 }
 
@@ -119,5 +110,15 @@ private:
 
 Result XQFunctionDeref::createResult(DynamicContext *context, int flags) const
 {
-  return new FunctionDerefResult(this);
+  FunctionRef::Ptr func = (FunctionRef*)getExpression()->createResult(context)->next(context).get();
+  if(func.isNull()) return 0;
+
+  VectorOfResults args;
+  if(getArguments()) {
+    for(VectorOfASTNodes::iterator i = getArguments()->begin(); i != getArguments()->end(); ++i) {
+      args.push_back(ClosureResult::create(*i, context));
+    }
+  }
+
+  return func->execute(args, context, this);
 }
