@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,11 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQFunctionRef.hpp>
+#include <xqilla/ast/XQInlineFunction.hpp>
 #include <xqilla/context/DynamicContext.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 #include <xqilla/context/VariableTypeStore.hpp>
 #include <xqilla/context/ContextHelpers.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
@@ -29,6 +27,7 @@
 #include <xqilla/utils/XStr.hpp>
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 
 #include "../items/impl/FunctionRefImpl.hpp"
 
@@ -38,20 +37,7 @@ using namespace std;
 XQFunctionRef::XQFunctionRef(const XMLCh *qname, unsigned int numArgs, XPath2MemoryManager *mm)
   : ASTNodeImpl(FUNCTION_REF, mm),
     qname_(qname),
-    uri_(0),
-    name_(0),
-    numArgs_(numArgs),
-    instance_(0)
-{
-}
-
-XQFunctionRef::XQFunctionRef(const XMLCh *uri, const XMLCh *name, unsigned int numArgs, ASTNode *instance, XPath2MemoryManager *mm)
-  : ASTNodeImpl(FUNCTION_REF, mm),
-    qname_(0),
-    uri_(uri),
-    name_(name),
-    numArgs_(numArgs),
-    instance_(instance)
+    numArgs_(numArgs)
 {
 }
 
@@ -59,23 +45,25 @@ ASTNode *XQFunctionRef::staticResolution(StaticContext *context)
 {
   XPath2MemoryManager *mm = context->getMemoryManager();
 
-  prefix_ = XPath2NSUtils::getPrefix(qname_, mm);
-  name_ = XPath2NSUtils::getLocalName(qname_);
+  const XMLCh *prefix = XPath2NSUtils::getPrefix(qname_, mm);
+  const XMLCh *name = XPath2NSUtils::getLocalName(qname_);
 
-  if(prefix_ == 0 || *prefix_ == 0) {
-    uri_ = context->getDefaultFuncNS();
+  const XMLCh *uri;
+  if(prefix == 0 || *prefix == 0) {
+    uri = context->getDefaultFuncNS();
   }
   else {
-    uri_ = context->getUriBoundToPrefix(prefix_, this);
+    uri = context->getUriBoundToPrefix(prefix, this);
   }
 
-  instance_ = FunctionRefImpl::createInstance(uri_, name_, numArgs_, context, this);
-  if(instance_ == 0) {
+  FunctionSignature *signature = 0;
+  ASTNode *instance = FunctionRefImpl::createInstance(uri, name, numArgs_, context, this, signature);
+  if(instance == 0) {
     XMLBuffer buf;
     buf.set(X("A function called {"));
-    buf.append(uri_);
+    buf.append(uri);
     buf.append(X("}"));
-    buf.append(name_);
+    buf.append(name);
     buf.append(X(" with "));
     XPath2Utils::numToBuf(numArgs_, buf);
     buf.append(X(" arguments is not defined [err:XPST0017]"));
@@ -83,25 +71,24 @@ ASTNode *XQFunctionRef::staticResolution(StaticContext *context)
     XQThrow(StaticErrorException, X("XQFunctionRef::staticResolution"), buf.getRawBuffer());
   }
 
-  instance_ = instance_->staticResolution(context);
+  instance = instance->staticResolution(context);
 
-  return this;
+  ASTNode *result = new (mm) XQInlineFunction(0, prefix, uri, name, numArgs_, signature, instance, mm);
+  result->setLocationInfo(result);
+  this->release();
+  return result; // Don't call staticResolution() on result
 }
 
 ASTNode *XQFunctionRef::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
-
-  // TBD Using getMemoryManager() might not be thread safe in DB XML - jpcs
-
-  XPath2MemoryManager *mm = getMemoryManager();
-  _src.getStaticType() = StaticType(mm, numArgs_, instance_->getStaticAnalysis().getStaticType());
-
+  // Should never happen
+  assert(false);
   return this;
 }
 
 Result XQFunctionRef::createResult(DynamicContext *context, int flags) const
 {
-  return (Item::Ptr)new FunctionRefImpl(prefix_, uri_, name_,
-                                        instance_, numArgs_, context);
+  // Should never happen
+  assert(false);
+  return 0;
 }
