@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,17 +15,15 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQInlineFunction.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/context/ContextHelpers.hpp>
 #include <xqilla/functions/XQUserFunction.hpp>
-#include <xqilla/runtime/SingleResult.hpp>
 #include <xqilla/context/VariableTypeStore.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 
 #include "../items/impl/FunctionRefImpl.hpp"
 
@@ -35,15 +33,25 @@ using namespace std;
 XQInlineFunction::XQInlineFunction(XQUserFunction *func, XPath2MemoryManager *mm)
   : ASTNodeImpl(INLINE_FUNCTION, mm),
     func_(func),
-    numArgs_((unsigned int)(func->getArgumentSpecs() ? func->getArgumentSpecs()->size() : 0)),
+    prefix_(0),
+    uri_(0),
+    name_(0),
+    numArgs_((unsigned int)(func->getSignature()->argSpecs ? func->getSignature()->argSpecs->size() : 0)),
+    signature_(new (mm) FunctionSignature(func->getSignature(), mm)),
     instance_(0)
 {
 }
 
-XQInlineFunction::XQInlineFunction(XQUserFunction *func, unsigned int numArgs, ASTNode *instance, XPath2MemoryManager *mm)
+XQInlineFunction::XQInlineFunction(XQUserFunction *func, const XMLCh *prefix, const XMLCh *uri, const XMLCh *name,
+                                   unsigned int numArgs, FunctionSignature *signature, ASTNode *instance,
+                                   XPath2MemoryManager *mm)
   : ASTNodeImpl(INLINE_FUNCTION, mm),
     func_(func),
+    prefix_(prefix),
+    uri_(uri),
+    name_(name),
     numArgs_(numArgs),
+    signature_(signature),
     instance_(instance)
 {
 }
@@ -53,8 +61,9 @@ ASTNode *XQInlineFunction::staticResolution(StaticContext *context)
   XPath2MemoryManager *mm = context->getMemoryManager();
 
   func_->staticResolutionStage1(context);
+  signature_->staticResolution(context);
 
-  instance_ = FunctionRefImpl::createInstance(func_, numArgs_, mm, this);
+  instance_ = FunctionRefImpl::createInstance(func_, signature_, mm, this);
   instance_ = instance_->staticResolution(context);
 
   func_->staticResolutionStage2(context);
@@ -68,24 +77,21 @@ ASTNode *XQInlineFunction::staticTypingImpl(StaticContext *context)
 
   _src.addExceptContextFlags(instance_->getStaticAnalysis());
 
-  // TBD Using getMemoryManager() might not be thread safe in DB XML - jpcs
-
   // Remove the argument variables
-  XPath2MemoryManager *mm = getMemoryManager();
-  for(unsigned int i = 0; i < numArgs_; ++i) {
-    XMLBuffer buf(20);
-    buf.set(FunctionRefImpl::argVarPrefix);
-    XPath2Utils::numToBuf(i, buf);
-
-    _src.removeVariable(0, mm->getPooledString(buf.getRawBuffer()));
+  if(signature_->argSpecs) {
+    ArgumentSpecs::const_iterator argsIt = signature_->argSpecs->begin();
+    for(; argsIt != signature_->argSpecs->end(); ++argsIt) {
+      _src.removeVariable((*argsIt)->getURI(), (*argsIt)->getName());
+    }
   }
 
-  _src.getStaticType() = StaticType(mm, numArgs_, instance_->getStaticAnalysis().getStaticType());
+  // TBD Using getMemoryManager() might not be thread safe in DB XML - jpcs
+  _src.getStaticType() = StaticType(getMemoryManager(), numArgs_, instance_->getStaticAnalysis().getStaticType());
 
   return this;
 }
 
 Result XQInlineFunction::createResult(DynamicContext *context, int flags) const
 {
-  return (Item::Ptr)new FunctionRefImpl(instance_, numArgs_, _src, context);
+  return (Item::Ptr)new FunctionRefImpl(prefix_, uri_, name_, signature_, instance_, _src, context);
 }
