@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -32,6 +30,7 @@
 #include <xqilla/items/impl/ATDoubleOrDerivedImpl.hpp>
 #include <xqilla/items/impl/ATFloatOrDerivedImpl.hpp>
 #include <xqilla/exceptions/IllegalArgumentException.hpp>
+#include <xqilla/utils/XPath2Utils.hpp>
 
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
 
@@ -73,7 +72,7 @@ ASTNode *XQLiteral::create(const Item::Ptr &item, DynamicContext *context, XPath
     return result;
   }
   else {
-    XQThrow2(IllegalArgumentException, X("XQLiteral::create"), X("Cannot create an ASTNode literal for a non atomic item"));
+    XQThrow2(::IllegalArgumentException, X("XQLiteral::create"), X("Cannot create an ASTNode literal for a non atomic item"));
   }
 }
 
@@ -148,6 +147,12 @@ ASTNode *XQLiteral::staticTypingImpl(StaticContext *context)
   return this;
 }
 
+BoolResult XQLiteral::boolResult(DynamicContext* context) const
+{
+  assert(primitiveType_ == AnyAtomicType::BOOLEAN);
+  return XPath2Utils::equals(SchemaSymbols::fgATTVAL_TRUE, value_);
+}
+
 Result XQLiteral::createResult(DynamicContext* context, int flags) const
 {
   return (Item::Ptr)context->getItemFactory()->createDerivedFromAtomicType(primitiveType_, typeURI_, typeName_, value_, context).get();
@@ -259,11 +264,11 @@ Result XQNumericLiteral::createResult(DynamicContext* context, int flags) const
 {
   switch(primitiveType_) {
   case AnyAtomicType::DECIMAL:
-    return (Item::Ptr)context->getItemFactory()->createDecimalOrDerived(typeURI_, typeName_, getValue(), context);
+    return (Item::Ptr)new ATDecimalOrDerivedImpl(typeURI_, typeName_, getValue(), context);
   case AnyAtomicType::FLOAT:
-    return (Item::Ptr)context->getItemFactory()->createFloatOrDerived(typeURI_, typeName_, getValue(), context);
+    return (Item::Ptr)new ATFloatOrDerivedImpl(typeURI_, typeName_, getValue(), context);
   case AnyAtomicType::DOUBLE:
-    return (Item::Ptr)context->getItemFactory()->createDoubleOrDerived(typeURI_, typeName_, getValue(), context);
+    return (Item::Ptr)new ATDoubleOrDerivedImpl(typeURI_, typeName_, getValue(), context);
   default: break;
   }
   return 0;
