@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQReturn.hpp>
@@ -26,6 +24,7 @@
 #include <xqilla/update/PendingUpdateList.hpp>
 #include <xqilla/optimizer/ASTVisitor.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
+#include <xqilla/runtime/ClosureResult.hpp>
 
 XQReturn::XQReturn(TupleNode *parent, ASTNode *expr, XPath2MemoryManager *mm)
   : ASTNodeImpl(RETURN, mm),
@@ -97,7 +96,8 @@ private:
 
 Result XQReturn::createResult(DynamicContext* context, int flags) const
 {
-  return new ReturnResult(this, parent_->createResult(context));
+  return ClosureResult::create(getStaticAnalysis(), context,
+    new ReturnResult(this, parent_->createResult(context)));
 }
 
 EventGenerator::Ptr XQReturn::generateEvents(EventHandler *events, DynamicContext *context,
