@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -29,7 +27,7 @@
 #include <xqilla/exceptions/StaticErrorException.hpp>
 #include <xqilla/runtime/ClosureResult.hpp>
 
-static const unsigned int CONSTANT_FOLD_LIMIT = 30;
+static const unsigned int CONSTANT_FOLD_LIMIT = 4;
 
 XQSequence *XQSequence::constantFold(Result &result, DynamicContext *context, XPath2MemoryManager* memMgr,
                                      const LocationInfo *location)
@@ -186,7 +184,7 @@ public:
 
       if(i_ == ast_->getChildren().end()) {
         // Tail call optimisation
-        tail = ClosureResult::create(ast, context);
+        tail = ast->createResult(context);
         return 0;
       }
 
@@ -206,6 +204,12 @@ private:
 Result XQSequence::createResult(DynamicContext* context, int flags) const
 {
   if(_astNodes.empty()) return 0;
-  return new XQSequenceResult(this);
+  return ClosureResult::create(getStaticAnalysis(), context, new XQSequenceResult(this));
 }
 
+BoolResult XQSequence::boolResult(DynamicContext* context) const
+{
+  if(_astNodes.empty()) return BoolResult::Null;
+  if(_astNodes.size() == 1) return _astNodes[0]->boolResult(context);
+  return ASTNodeImpl::boolResult(context);
+}
