@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/ast/XQValidate.hpp>
@@ -41,7 +39,7 @@ XQValidate::XQValidate(ASTNode *expr, DocumentCache::ValidationMode mode, XPath2
 
 Result XQValidate::createResult(DynamicContext* context, int flags) const
 {
-  return new GenerateEventsResult(this, 0);
+  return EventGenerator::createResult(this, 0, context);
 }
 
 EventGenerator::Ptr XQValidate::generateEvents(EventHandler *events, DynamicContext *context,
