@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: ItemFactoryImpl.cpp 623 2008-08-26 20:22:24Z jpcs $
  */
 
 #include <xqilla/context/impl/ItemFactoryImpl.hpp>
@@ -43,20 +41,18 @@ XERCES_CPP_NAMESPACE_USE
 #endif
 
 ItemFactoryImpl::ItemFactoryImpl(const DocumentCache* dc, MemoryManager* memMgr)
-  : datatypeLookup_(new (memMgr) DatatypeLookup(dc, memMgr)),
-    datatypeLookupOwned_(true)
+  : datatypeLookup_(new (memMgr) DatatypeLookup(dc, memMgr))
 {
 }
 
-ItemFactoryImpl::ItemFactoryImpl(DatatypeLookup *dl)
-  : datatypeLookup_(dl),
-    datatypeLookupOwned_(false)
+ItemFactoryImpl::ItemFactoryImpl(DatatypeLookup *datatypeLookup)
+  : datatypeLookup_(datatypeLookup)
 {
 }
 
 ItemFactoryImpl::~ItemFactoryImpl()
 {
-  if(datatypeLookupOwned_) delete datatypeLookup_;
+  delete datatypeLookup_;
 }
 
 AnyAtomicType::AtomicObjectType ItemFactoryImpl::getPrimitiveTypeIndex(const XMLCh* typeURI, const XMLCh* typeName, bool &isPrimitive) const
@@ -147,6 +143,15 @@ ATBooleanOrDerived::Ptr ItemFactoryImpl::createBoolean(bool value, const Dynamic
     value, context);  
 }
 
+ATBooleanOrDerived::Ptr ItemFactoryImpl::createBoolean(BoolResult value, const DynamicContext* context)
+{
+  if(value == BoolResult::Null) return 0;
+  return createBooleanOrDerived(
+    SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
+    SchemaSymbols::fgDT_BOOLEAN,
+    value, context);  
+}
+
 ATBooleanOrDerived::Ptr ItemFactoryImpl::createBoolean(const XMLCh* value, const DynamicContext* context) {
   return (ATBooleanOrDerived*)datatypeLookup_->getBooleanFactory()->createInstance(value, context).get();
 }
