@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../../config/xqilla_config.h"
@@ -44,12 +42,12 @@ void VarTypeStoreImpl::clear()
 
 void VarTypeStoreImpl::addLocalScope()
 {
-  _store.addScope(Scope<const StaticAnalysis*>::LOCAL_SCOPE);
+  _store.addScope(Scope<VarType>::LOCAL_SCOPE);
 }
 
 void VarTypeStoreImpl::addLogicalBlockScope()
 {
-  _store.addScope(Scope<const StaticAnalysis*>::LOGICAL_BLOCK_SCOPE);
+  _store.addScope(Scope<VarType>::LOGICAL_BLOCK_SCOPE);
 }
 
 void VarTypeStoreImpl::removeScope()
@@ -59,32 +57,41 @@ void VarTypeStoreImpl::removeScope()
 
 void VarTypeStoreImpl::declareGlobalVar(const XMLCh* namespaceURI,
                                         const XMLCh* name,
-                                        const StaticAnalysis &src)
+                                        const StaticAnalysis &src,
+                                        XQGlobalVariable *global)
 {
-  _store.setGlobalVar(namespaceURI, name, &src);
+  _store.setGlobalVar(namespaceURI, name, VarType(&src, global));
 }
 
 void VarTypeStoreImpl::declareVar(const XMLCh* namespaceURI,
                                   const XMLCh* name,
                                   const StaticAnalysis &src)
 {
-  _store.declareVar(namespaceURI, name, &src);
+  _store.declareVar(namespaceURI, name, VarType(&src, 0));
 }
 
 const StaticAnalysis *VarTypeStoreImpl::getVar(const XMLCh* namespaceURI,
-                                                        const XMLCh* name) const
+                                               const XMLCh* name,
+                                               XQGlobalVariable **global) const
 {
-  VarHashEntry<const StaticAnalysis*>* result = _store.getVar(namespaceURI, name);
-  if(result)
-    return result->getValue();
+  VarHashEntry<VarType>* result = _store.getVar(namespaceURI, name);
+  if(result) {
+    if(global) *global = result->getValue().global;
+    return result->getValue().type;
+  }
+  if(global) *global = 0;
   return 0;
 }
 
 const StaticAnalysis* VarTypeStoreImpl::getGlobalVar(const XMLCh* namespaceURI,
-                                                              const XMLCh* name) const
+                                                     const XMLCh* name,
+                                                     XQGlobalVariable **global) const
 {
-  VarHashEntry<const StaticAnalysis*>* result = _store.getGlobalVar(namespaceURI, name);
-  if(result)
-    return result->getValue();
+  VarHashEntry<VarType>* result = _store.getGlobalVar(namespaceURI, name);
+  if(result) {
+    if(global) *global = result->getValue().global;
+    return result->getValue().type;
+  }
+  if(global) *global = 0;
   return 0;
 }
