@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 /*
@@ -61,13 +59,15 @@ public:
   /** Declares and/or sets a variable in the global scope. */
   virtual void declareGlobalVar(const XMLCh* namespaceURI,
                                 const XMLCh* name,
-                                const StaticAnalysis &src);
+                                const StaticAnalysis &src,
+                                XQGlobalVariable *global);
 
   /** Looks up the value of a variable in the current scope.
       Returns a boolean (true if successful), and the SequenceType
       value of the variable*/
   virtual const StaticAnalysis* getGlobalVar(const XMLCh* namespaceURI,
-                                                      const XMLCh* name) const;
+                                             const XMLCh* name,
+                                             XQGlobalVariable **global = 0) const;
 
   ///Declares a var in the top level scope
   virtual void declareVar(const XMLCh* namespaceURI,
@@ -78,10 +78,20 @@ public:
       Returns a boolean (true if successful), and the SequenceType
       value of the variable*/
   virtual const StaticAnalysis* getVar(const XMLCh* namespaceURI,
-                                                const XMLCh* name) const;
+                                       const XMLCh* name,
+                                       XQGlobalVariable **global = 0) const;
 
 private:
-  VariableStoreTemplate<const StaticAnalysis*> _store;
+  struct VarType
+  {
+    VarType(const StaticAnalysis *t, XQGlobalVariable *g)
+      : type(t), global(g) {}
+
+    const StaticAnalysis *type;
+    XQGlobalVariable *global;
+  };
+
+  VariableStoreTemplate<VarType> _store;
 };
 
 #endif
