@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
@@ -35,6 +33,7 @@
 #include <xqilla/ast/XQFunction.hpp>
 #include <xqilla/ast/XQSequence.hpp>
 #include <xqilla/ast/XQVariable.hpp>
+#include <xqilla/ast/XQCastAs.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
 #include <xqilla/utils/ContextUtils.hpp>
@@ -47,19 +46,13 @@
 #include <xqilla/items/DatatypeLookup.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
 #include <xqilla/functions/FunctionLookup.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/functions/XQUserFunction.hpp>
-#include <xqilla/functions/XQillaFunction.hpp>
 #include <xqilla/dom-api/impl/XQillaNSResolverImpl.hpp>
 #include <xqilla/simple-api/XQillaConfiguration.hpp>
+#include <xqilla/schema/SequenceType.hpp>
 
 XERCES_CPP_NAMESPACE_USE;
 
-const XMLCh XMLChXS[]    = { chLatin_x, chLatin_s, chNull };
-const XMLCh XMLChXSI[]   = { chLatin_x, chLatin_s, chLatin_i, chNull };
-const XMLCh XMLChFN[]    = { chLatin_f, chLatin_n, chNull };
-const XMLCh XMLChLOCAL[] = { chLatin_l, chLatin_o, chLatin_c, chLatin_a, chLatin_l, chNull };
-
 static CodepointCollation g_codepointCollation;
 
 XQContextImpl::XQContextImpl(XQillaConfiguration *conf, XQilla::Language language, MemoryManager* memMgr)
@@ -77,6 +70,7 @@ XQContextImpl::XQContextImpl(XQillaConfiguration *conf, XQilla::Language languag
     _bPreserveBoundarySpace(false),
     _revalidationMode(DocumentCache::VALIDATION_LAX),
     _messageListener(0),
+    _module(0),
     _varStore(&_defaultVarStore),
     _globalVarStore(&_defaultVarStore),
     _defaultVarStore(&_internalMM),
@@ -89,7 +83,8 @@ XQContextImpl::XQContextImpl(XQillaConfiguration *conf, XQilla::Language languag
     _stackFrame(0),
     _projection(true),
     _lint(false),
-    _tmpVarCounter(0)
+    _tmpVarCounter(0),
+    _ftCaseOption(StaticContext::CASE_INSENSITIVE)
 {
   _memMgr = &_internalMM;
 
@@ -150,20 +145,6 @@ XQContextImpl::XQContextImpl(XQillaConfiguration *conf, XQilla::Language languag
   if(_defaultResolver.resolver != 0) {
     _defaultResolver.adopt = true;
   }
-
-  if((language & XQilla::XPATH2) == 0 &&
-     (language & XQilla::XSLT2) == 0) {
-    // XQuery defines these predefined namespace bindings
-    setNamespaceBinding(XMLChXS, SchemaSymbols::fgURI_SCHEMAFORSCHEMA);
-    setNamespaceBinding(XMLChXSI, SchemaSymbols::fgURI_XSI);
-    setNamespaceBinding(XMLChFN, XQFunction::XMLChFunctionURI);
-    setNamespaceBinding(XMLChLOCAL, XQUserFunction::XMLChXQueryLocalFunctionsURI);
-  }
-
-  if((language & XQilla::XSLT2) == 0) {
-    // Predefine the namespace for xqilla extension functions
-    setNamespaceBinding(XQillaFunction::XQillaPrefix, XQillaFunction::XMLChFunctionURI);
-  }
 }
 
 XQContextImpl::~XQContextImpl()
@@ -228,11 +209,15 @@ DynamicContext *XQContextImpl::createModuleContext(MemoryManager *memMgr) const
 
 DynamicContext *XQContextImpl::createModuleDynamicContext(const DynamicContext* moduleCtx, MemoryManager *memMgr) const
 {
-  DynamicContext* moduleDCtx = new (memMgr) XQDynamicContextImpl(_conf, moduleCtx, memMgr);
+  XQDynamicContextImpl* moduleDCtx = new (memMgr) XQDynamicContextImpl(_conf, moduleCtx, (VarStoreImpl*)&_defaultVarStore, memMgr);
 
   // Force the context to use our memory manager
   moduleDCtx->setMemoryManager(getMemoryManager());
 
+  // Use the variables from our context
+  moduleDCtx->setGlobalVariableStore(getGlobalVariableStore());
+  moduleDCtx->setVariableStore(getVariableStore());
+
   // Add our URIResolvers to the module context
   moduleDCtx->setDefaultURIResolver(_defaultResolver.resolver, /*adopt*/false);
   std::vector<ResolverEntry, XQillaAllocator<ResolverEntry> >::const_iterator end = _resolvers.end();
@@ -256,7 +241,7 @@ DynamicContext *XQContextImpl::createModuleDynamicContext(const DynamicContext*
 
 DynamicContext *XQContextImpl::createDynamicContext(MemoryManager *memMgr) const
 {
-  DynamicContext *result = new (memMgr) XQDynamicContextImpl(_conf, this, memMgr);
+  DynamicContext *result = new (memMgr) XQDynamicContextImpl(_conf, this, 0, memMgr);
   _conf->populateDynamicContext(result);
   return result;
 }
@@ -285,7 +270,7 @@ DynamicContext *XQContextImpl::createDebugQueryContext(const Item::Ptr &contextI
   variables->getInScopeVariables(inScopeVars);
   std::vector<std::pair<const XMLCh *, const XMLCh*> >::iterator i = inScopeVars.begin();
   for(; i != inScopeVars.end(); ++i) {
-    store->declareGlobalVar(i->first, i->second, *src);
+    store->declareGlobalVar(i->first, i->second, *src, 0);
   }
 
   // Set up all the in-scope namespaces
@@ -610,12 +595,12 @@ void XQContextImpl::setGlobalVariableStore(const VariableStore *store)
   _globalVarStore = store;
 }
 
-void XQContextImpl::setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Sequence &value)
+void XQContextImpl::setExternalVariable(const XMLCh *namespaceURI, const XMLCh *name, const Result &value)
 {
   _defaultVarStore.setVar(namespaceURI, name, value);
 }
 
-void XQContextImpl::setExternalVariable(const XMLCh *qname, const Sequence &value)
+void XQContextImpl::setExternalVariable(const XMLCh *qname, const Result &value)
 {
   const XMLCh *uri = getUriBoundToPrefix(XPath2NSUtils::getPrefix(qname, getMemoryManager()), 0);
   const XMLCh *name = XPath2NSUtils::getLocalName(qname);
@@ -705,22 +690,37 @@ Collation* XQContextImpl::getDefaultCollation(const LocationInfo *location) cons
   return getCollation(_defaultCollation, location);
 }
 
-ASTNode *XQContextImpl::lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v) const
+ASTNode *XQContextImpl::lookUpFunction(const XMLCh *uri, const XMLCh* name, const VectorOfASTNodes &v, const LocationInfo *location) const
 {
   ASTNode* functionImpl = FunctionLookup::lookUpGlobalFunction(uri, name, v, getMemoryManager(), _functionTable);
-
-  if(functionImpl == NULL && v.size() == 1) {
-    // maybe it's not a function, but a datatype
-    try {
-      bool isPrimitive;
-      _itemFactory->getPrimitiveTypeIndex(uri, name, isPrimitive);
-      functionImpl = new (getMemoryManager())
-        FunctionConstructor(uri, name, v, getMemoryManager());
-    }
-    catch(TypeNotFoundException&) {
-      // ignore this exception: it means the type has not been found
-    }
+  if(functionImpl) {
+    functionImpl->setLocationInfo(location);
+    return functionImpl;
   }
+
+  if(v.size() != 1) return 0;
+
+  // maybe it's not a function, but a datatype
+  try {
+    bool isPrimitive;
+    _itemFactory->getPrimitiveTypeIndex(uri, name, isPrimitive);
+  }
+  catch(TypeNotFoundException&) {
+    // ignore this exception: it means the type has not been found
+    return 0;
+  }
+
+  if((XPath2Utils::equals(name, XMLUni::fgNotationString) || XPath2Utils::equals(name, AnyAtomicType::fgDT_ANYATOMICTYPE)) &&
+     XPath2Utils::equals(uri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA))
+    return 0;
+
+  XPath2MemoryManager *mm = getMemoryManager();
+
+  SequenceType *seqType = new (mm) SequenceType(uri, name, SequenceType::QUESTION_MARK, mm);
+  seqType->setLocationInfo(location);
+
+  functionImpl = new (mm) XQCastAs(v[0], seqType, mm);
+  functionImpl->setLocationInfo(location);
   return functionImpl;
 }
 
