@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../../config/xqilla_config.h"
@@ -50,7 +48,6 @@
 #include <xercesc/dom/DOMAttr.hpp>
 #include <xercesc/dom/DOMNode.hpp>
 #include <xercesc/validators/schema/SchemaSymbols.hpp>
-#include <xercesc/util/XMLString.hpp>
 #include <xercesc/dom/impl/DOMDocumentImpl.hpp>
 #include <xercesc/util/XMLDOMMsg.hpp>
 
@@ -78,7 +75,7 @@ XPath2ResultImpl::XPath2ResultImpl(DOMNode* contextNode,
                                    MemoryManager* memMgr,
                                    XQillaExpressionImpl *exprImpl)
   : _createdWith(memMgr),
-    _context(staticContext->createDynamicContext(_createdWith)),
+    _context(exprImpl != 0 ? staticContext : staticContext->createDynamicContext(_createdWith)),
     _currentItem(0),
     _exprToDelete(exprImpl)
 {
@@ -98,8 +95,8 @@ XPath2ResultImpl::XPath2ResultImpl(DOMNode* contextNode,
 
 XPath2ResultImpl::~XPath2ResultImpl() { 
   _currentItem = 0;
-  delete _context;
   if(_exprToDelete) _exprToDelete->release();
+  else delete _context;
 }
 
 const DOMTypeInfo *XPath2ResultImpl::getTypeInfo() const
@@ -150,7 +147,7 @@ const XMLCh* XPath2ResultImpl::getNamespace() const
 int XPath2ResultImpl::getIntegerValue() const
 {
   if(_currentItem.isNull()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("There is no current result in the result")); 
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("There is no current result in the result")); 
   }
 
   return FunctionNumber::number(_currentItem, _context, 0)->asInt();
@@ -159,7 +156,7 @@ int XPath2ResultImpl::getIntegerValue() const
 double XPath2ResultImpl::getNumberValue() const
 {
   if(_currentItem.isNull()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("There is no current result in the result"));
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("There is no current result in the result"));
   }
 
   return FunctionNumber::number(_currentItem, _context, 0)->asDouble();
@@ -168,7 +165,7 @@ double XPath2ResultImpl::getNumberValue() const
 const XMLCh* XPath2ResultImpl::getStringValue() const
 {
   if(_currentItem.isNull()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("There is no current result in the result"));
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("There is no current result in the result"));
   }
 
   return FunctionString::string(_currentItem, _context);
@@ -177,7 +174,7 @@ const XMLCh* XPath2ResultImpl::getStringValue() const
 bool XPath2ResultImpl::getBooleanValue() const
 {
   if(_currentItem.isNull()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("There is no current result in the result"));
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("There is no current result in the result"));
   }
 
   if(!_currentItem->isAtomicValue()) {
@@ -197,7 +194,7 @@ bool XPath2ResultImpl::getBooleanValue() const
 DOMNode* XPath2ResultImpl::getNodeValue() const
 {
   if(_currentItem.isNull()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("There is no current result in the result"));
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("There is no current result in the result"));
   }
 
   if(!_currentItem->isNode()) {
@@ -484,7 +481,7 @@ bool XPath2IteratorResultImpl::iterateNext()
 {
   // check for document changes
   if(getInvalidIteratorState()) {
-    throw XQillaException(DOMException::INVALID_STATE_ERR, XMLString::transcode("Document has changed"));
+    throw XQillaException(DOMException::INVALID_STATE_ERR, X("Document has changed"));
   }
 
   try {
