@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <xqilla/events/EventGenerator.hpp>
@@ -24,6 +22,7 @@
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/events/QueryPathTreeFilter.hpp>
 #include <xqilla/events/SequenceBuilder.hpp>
+#include <xqilla/runtime/Sequence.hpp>
 
 ASTNodeEventGenerator::ASTNodeEventGenerator(const ASTNode *ast, bool preserveNS, bool preserveType)
   : ast_(ast),
@@ -37,27 +36,19 @@ EventGenerator::Ptr ASTNodeEventGenerator::generateEvents(EventHandler *events,
   return ast_->generateEvents(events, context, preserveNS_, preserveType_);
 }
 
-GenerateEventsResult::GenerateEventsResult(const ASTNode *ast, const QueryPathNode *projection)
-  : ResultImpl(ast),
-    ast_(ast),
-    projection_(projection)
-{
-}
-
-Item::Ptr GenerateEventsResult::nextOrTail(Result &tail, DynamicContext *context)
+Result EventGenerator::createResult(const ASTNode *ast, const QueryPathNode *projection,
+  DynamicContext *context)
 {
   AutoDelete<SequenceBuilder> builder(context->createSequenceBuilder());
-  if(context->getProjection() && projection_ != 0) {
-    QueryPathTreeFilter qptf(projection_, builder.get());
-    ast_->generateAndTailCall(&qptf, context, true, true);
+  if(context->getProjection() && projection != 0) {
+    QueryPathTreeFilter qptf(projection, builder.get());
+    ast->generateAndTailCall(&qptf, context, true, true);
     qptf.endEvent();
   }
   else {
-    ast_->generateAndTailCall(builder.get(), context, true, true);
+    ast->generateAndTailCall(builder.get(), context, true, true);
     builder->endEvent();
   }
 
-  tail = builder->getSequence();
-  return 0;
+  return builder->getSequence();
 }
-
