@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -30,6 +28,7 @@
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/context/URIResolver.hpp>
 #include <xqilla/context/impl/ItemFactoryImpl.hpp>
+#include <xqilla/optimizer/ASTToXML.hpp>
 
 #include <xercesc/util/RefHashTableOf.hpp>
 #include <xercesc/util/NoSuchElementException.hpp>
@@ -178,3 +177,8 @@ URIResolver *FastXDMConfiguration::createDefaultURIResolver(MemoryManager *memMg
   return new (memMgr) FastXDMURIResolver(memMgr);
 }
 
+ASTToXML *FastXDMConfiguration::createASTToXML()
+{
+  return new ASTToXML();
+}
+
