@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: FastXDMNodeImpl.cpp 734 2009-01-30 18:08:37Z gmfeinberg $
  */
 
 #include "../config/xqilla_config.h"
@@ -29,15 +27,14 @@
 #include <xqilla/exceptions/ItemException.hpp>
 #include <xqilla/items/ATQNameOrDerived.hpp>
 #include <xqilla/items/ATUntypedAtomic.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/schema/DocumentCache.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
 #include <xqilla/utils/XMLChCompare.hpp>
 #include <xqilla/dom-api/XQillaNSResolver.hpp>
-#include <xqilla/axis/SelfAxis.hpp>
 #include <xqilla/axis/NodeTest.hpp>
 #include <xqilla/events/EventSerializer.hpp>
 #include <xqilla/events/NSFixupFilter.hpp>
+#include <xqilla/context/ItemFactory.hpp>
 
 #include <xercesc/framework/MemBufFormatTarget.hpp>
 #include <xercesc/validators/datatype/ListDatatypeValidator.hpp>
@@ -1051,7 +1048,9 @@ public:
         }
         break;
       case DO_XML:
-        result = new FastXDMNamespaceNodeImpl(owner_, XMLUni::fgXMLString, XMLUni::fgXMLURIName, order_++);
+        if(done_.insert(XMLUni::fgXMLString).second) {
+          result = new FastXDMNamespaceNodeImpl(owner_, XMLUni::fgXMLString, XMLUni::fgXMLURIName, order_++);
+        }
         state_ = DONE;
         owner_ = 0;
         break;
@@ -1489,7 +1488,7 @@ Result FastXDMAttributeNodeImpl::getAxisResult(XQStep::Axis axis, const NodeTest
   }
   case XQStep::DESCENDANT_OR_SELF:
   case XQStep::SELF: {
-    return nodeTest->filterResult(new SelfAxis(info, this), info);
+    return nodeTest->filterResult((Item::Ptr)this, info);
   }
   case XQStep::ATTRIBUTE:
   case XQStep::CHILD:
@@ -1734,7 +1733,8 @@ protected:
 
 Node::Ptr FastXDMNamespaceNodeImpl::root(const DynamicContext* context) const
 {
-  return owner_.isNull() ? this : owner_->root(context);
+  if(owner_.isNull()) return this;
+  return owner_->root(context);
 }
 
 Node::Ptr FastXDMNamespaceNodeImpl::dmParent(const DynamicContext* context) const
@@ -1769,7 +1769,7 @@ Result FastXDMNamespaceNodeImpl::getAxisResult(XQStep::Axis axis, const NodeTest
     if(owner_.notNull())
       return new FastXDMNamespaceAncestorOrSelfAxis(info, this, nodeTest);
     else 
-      return nodeTest->filterResult(new SelfAxis(info, this), info);
+      return nodeTest->filterResult((Item::Ptr)this, info);
   }
   case XQStep::FOLLOWING: {
     if(owner_.notNull())
@@ -1778,7 +1778,7 @@ Result FastXDMNamespaceNodeImpl::getAxisResult(XQStep::Axis axis, const NodeTest
   }
   case XQStep::PARENT: {
     if(owner_.notNull())
-      return nodeTest->filterResult(new SelfAxis(info, owner_), info);
+      return nodeTest->filterResult((Item::Ptr)owner_, info);
     break;
   }
   case XQStep::PRECEDING: {
@@ -1788,7 +1788,7 @@ Result FastXDMNamespaceNodeImpl::getAxisResult(XQStep::Axis axis, const NodeTest
   }
   case XQStep::DESCENDANT_OR_SELF:
   case XQStep::SELF: {
-    return nodeTest->filterResult(new SelfAxis(info, this), info);
+    return nodeTest->filterResult((Item::Ptr)this, info);
   }
   case XQStep::ATTRIBUTE:
   case XQStep::CHILD:
