@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -24,7 +22,10 @@
 #include <xqilla/fulltext/Tokenizer.hpp>
 #include <xqilla/framework/XPath2MemoryManager.hpp>
 #include <xqilla/context/DynamicContext.hpp>
-#include <xqilla/utils/UCANormalizer.hpp>
+#include <xqilla/utils/UnicodeTransformer.hpp>
+extern "C" {
+#include <xqilla/utils/utf8proc.h>
+}
 
 #include <xercesc/util/XMLString.hpp>
 
@@ -37,19 +38,25 @@ DefaultTokenStore::DefaultTokenStore(const Node::Ptr &node, const Tokenizer *tok
   : tokens_(73, /*adoptElems*/true, context->getMemoryManager()),
     numTokens_(0),
     mm_(context->getMemoryManager()),
-    buffer_(1023, context->getMemoryManager())
+    context_(context)
 {
   TokenStream::Ptr stream = tokenizer->tokenize(node, context);
-  TokenInfo::Ptr token;
-  while((token = stream->next()).notNull()) {
-    ++numTokens_;
-    buffer_.reset();
-    Normalizer::caseFoldAndRemoveDiacritics(token->getWord(), buffer_);
+  TokenInfo token;
+	
+  int options = 0;
+  //consider stem, stop words, thesaurus, case options
+  if(context_->getFTCase() == StaticContext::CASE_INSENSITIVE) {
+    options = UTF8PROC_CASEFOLD;
+  }
 
-    TokenEntry *entry = tokens_.get(buffer_.getRawBuffer());
+
+  while((token = stream->next()).word_ != 0) {
+    ++numTokens_;
+    AutoDeallocate<XMLCh> buf(UnicodeTransformer::transform(token.word_, options, mm_), mm_);
+    TokenEntry *entry = tokens_.get(buf.get());
     if(entry == 0) {
       entry = new TokenEntry();
-      tokens_.put((void*)mm_->getPooledString(buffer_.getRawBuffer()), entry);
+      tokens_.put((void*)mm_->getPooledString(buf.get()), entry);
     }
     entry->addToken(token);
   }
@@ -58,17 +65,23 @@ DefaultTokenStore::DefaultTokenStore(const Node::Ptr &node, const Tokenizer *tok
 DefaultTokenStore::DefaultTokenStore(XPath2MemoryManager *mm)
   : tokens_(3, /*adoptElems*/true, mm),
     numTokens_(0),
-    mm_(mm),
-    buffer_(0, mm)
+    mm_(mm)
 {
 }
 
 TokenStream::Ptr DefaultTokenStore::findTokens(const XMLCh *searchString) const
 {
-  buffer_.reset();
-  Normalizer::caseFoldAndRemoveDiacritics(searchString, buffer_);
+  int options = 0;
+  if(context_->getFTCase() == StaticContext::CASE_INSENSITIVE){
+    options = UTF8PROC_CASEFOLD;
+  } else if(context_->getFTCase() == StaticContext::UPPERCASE){
+    options = UTF8PROC_UPPERCASE;
+  } else if(context_->getFTCase() == StaticContext::LOWERCASE){
+    options = UTF8PROC_LOWERCASE;
+  }
 
-  const TokenEntry *entry = tokens_.get(buffer_.getRawBuffer());
+  AutoDeallocate<XMLCh> buf(UnicodeTransformer::transform(searchString, options, mm_), mm_);
+  const TokenEntry *entry = tokens_.get(buf.get());
   if(entry == 0) return 0;
   return entry->getTokenStream();
 }
