@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -49,20 +47,19 @@ FTSelection *FTOr::staticResolution(StaticContext *context)
   return this;
 }
 
-FTSelection *FTOr::staticTyping(StaticContext *context, StaticTyper *styper)
+FTSelection *FTOr::staticTypingImpl(StaticContext *context)
 {
   src_.clear();
 
   for(VectorOfFTSelections::iterator i = args_.begin();
       i != args_.end(); ++i) {
-    *i = (*i)->staticTyping(context, styper);
     src_.add((*i)->getStaticAnalysis());
   }
 
   return this;
 }
 
-FTSelection *FTOr::optimize(FTContext *ftcontext, bool execute) const
+FTSelection *FTOr::optimize(FTContext *ftcontext) const
 {
   XPath2MemoryManager *mm = ftcontext->context->getMemoryManager();
 
@@ -71,7 +68,7 @@ FTSelection *FTOr::optimize(FTContext *ftcontext, bool execute) const
 
   for(VectorOfFTSelections::const_iterator i = args_.begin();
       i != args_.end(); ++i) {
-    FTSelection *arg = (*i)->optimize(ftcontext, execute);
+    FTSelection *arg = (*i)->optimize(ftcontext);
     if(arg != 0)
       ftor->addArg(arg);
   }
@@ -86,10 +83,10 @@ FTSelection *FTOr::optimize(FTContext *ftcontext, bool execute) const
   return ftor;
 }
 
-AllMatches::Ptr FTOr::execute(FTContext *ftcontext) const
+AllMatches *FTOr::execute(FTContext *ftcontext) const
 {
   FTDisjunctionMatches *disjunction = new FTDisjunctionMatches(this);
-  AllMatches::Ptr result(disjunction);
+  AllMatches *result(disjunction);
 
   for(VectorOfFTSelections::const_iterator i = args_.begin();
       i != args_.end(); ++i) {
@@ -105,26 +102,44 @@ FTDisjunctionMatches::FTDisjunctionMatches(const LocationInfo *info)
 {
 }
 
-Match::Ptr FTDisjunctionMatches::next(DynamicContext *context)
+FTDisjunctionMatches::~FTDisjunctionMatches()
+{
+  for(it_ = args_.begin(); it_ != args_.end(); it_++) {
+    delete *(it_);
+  }
+}
+
+bool FTDisjunctionMatches::next(DynamicContext *context)
 {
   // TBD AllMatches normalization
-
   if(toDo_) {
     toDo_ = false;
     it_ = args_.begin();
   }
 
-  Match::Ptr result(0);
+  bool found = false;
   while(it_ != args_.end()) {
-    result = (*it_)->next(context);
-    if(result.isNull()) {
+    if(!(*it_)->next(context)) {
+      delete *(it_);
       *it_ = 0;
       ++it_;
     }
     else {
+      found = true;
       break;
     }
   }
 
-  return result;
+  return found;
+}
+
+const StringMatches &FTDisjunctionMatches::getStringIncludes()
+{
+  assert(it_ != args_.end());
+  return (*it_)->getStringIncludes();
+}
+const StringMatches &FTDisjunctionMatches::getStringExcludes()
+{
+  assert(it_ != args_.end());
+  return (*it_)->getStringExcludes();
 }
