@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -66,45 +64,39 @@ FTSelection *FTWords::staticResolution(StaticContext *context)
   return this;
 }
 
-FTSelection *FTWords::staticTyping(StaticContext *context, StaticTyper *styper)
+FTSelection *FTWords::staticTypingImpl(StaticContext *context)
 {
   src_.clear();
-
-  expr_ = expr_->staticTyping(context, styper);
   src_.add(expr_->getStaticAnalysis());
-
   return this;
 }
 
-FTSelection *FTWords::optimize(FTContext *ftcontext, bool execute) const
+FTSelection *FTWords::optimize(FTContext *ftcontext) const
 {
-  if(execute || expr_->isConstant()) {
+  if(expr_->isConstant()) {
     Result strings = expr_->createResult(ftcontext->context);
 
     switch(option_) {
     case ANY_WORD: {
       return optimizeAnyWord(strings, ftcontext)->
-        optimize(ftcontext, execute);
+        optimize(ftcontext);
     }
     case ALL_WORDS: {
       return optimizeAllWords(strings, ftcontext)->
-        optimize(ftcontext, execute);
+        optimize(ftcontext);
     }
     case PHRASE: {
       return optimizePhrase(strings, ftcontext)->
-        optimize(ftcontext, execute);
+        optimize(ftcontext);
     }
     case ANY: {
       return optimizeAny(strings, ftcontext)->
-        optimize(ftcontext, execute);
+        optimize(ftcontext);
     }
     case ALL: {
       return optimizeAll(strings, ftcontext)->
-        optimize(ftcontext, execute);
+        optimize(ftcontext);
     }
-    default:
-      assert(0);
-      break;
     }
   }
 
@@ -122,9 +114,9 @@ FTSelection *FTWords::optimizeAnyWord(Result strings, FTContext *ftcontext) cons
   while((item = strings->next(ftcontext->context)).notNull()) {
     TokenStream::Ptr stream = ftcontext->tokenizer->
       tokenize(item->asString(ftcontext->context), ftcontext->context->getMemoryManager());
-    TokenInfo::Ptr token;
-    while((token = stream->next()).notNull()) {
-      FTSelection *word = new (mm) FTWord(token->getWord(), mm);
+    TokenInfo token;
+    while((token = stream->next()).word_ != 0) {
+      FTSelection *word = new (mm) FTWord(token.word_, mm);
       word->setLocationInfo(this);
       ftor->addArg(word);
     }
@@ -133,6 +125,24 @@ FTSelection *FTWords::optimizeAnyWord(Result strings, FTContext *ftcontext) cons
   return ftor;
 }
 
+AllMatches *FTWords::executeAnyWord(Result strings, FTContext *ftcontext) const
+{
+  FTDisjunctionMatches *disjunction = new FTDisjunctionMatches(this);
+  AllMatches *result(disjunction);
+
+  Item::Ptr item;
+  while((item = strings->next(ftcontext->context)).notNull()) {
+    TokenStream::Ptr stream = ftcontext->tokenizer->
+      tokenize(item->asString(ftcontext->context), ftcontext->context->getMemoryManager());
+    TokenInfo token;
+    while((token = stream->next()).word_ != 0) {
+      disjunction->addMatches(new FTStringSearchMatches(this, token.word_, ftcontext));
+    }
+  }
+
+  return result;
+}
+
 FTSelection *FTWords::optimizeAllWords(Result strings, FTContext *ftcontext) const
 {
   XPath2MemoryManager *mm = ftcontext->context->getMemoryManager();
@@ -144,9 +154,9 @@ FTSelection *FTWords::optimizeAllWords(Result strings, FTContext *ftcontext) con
   while((item = strings->next(ftcontext->context)).notNull()) {
     TokenStream::Ptr stream = ftcontext->tokenizer->
       tokenize(item->asString(ftcontext->context), ftcontext->context->getMemoryManager());
-    TokenInfo::Ptr token;
-    while((token = stream->next()).notNull()) {
-      FTSelection *word = new (mm) FTWord(token->getWord(), mm);
+    TokenInfo token;
+    while((token = stream->next()).word_ != 0) {
+      FTSelection *word = new (mm) FTWord(token.word_, mm);
       word->setLocationInfo(this);
       ftand->addArg(word);
     }
@@ -155,6 +165,24 @@ FTSelection *FTWords::optimizeAllWords(Result strings, FTContext *ftcontext) con
   return ftand;
 }
 
+AllMatches *FTWords::executeAllWords(Result strings, FTContext *ftcontext) const
+{
+  FTConjunctionMatches *conjunction = new FTConjunctionMatches(this);
+  AllMatches *result(conjunction);
+
+  Item::Ptr item;
+  while((item = strings->next(ftcontext->context)).notNull()) {
+    TokenStream::Ptr stream = ftcontext->tokenizer->
+      tokenize(item->asString(ftcontext->context), ftcontext->context->getMemoryManager());
+    TokenInfo token;
+    while((token = stream->next()).word_ != 0) {
+      conjunction->addMatches(new FTStringSearchMatches(this, token.word_, ftcontext));
+    }
+  }
+
+  return result;
+}
+
 FTSelection *FTWords::optimizePhrase(Result strings, FTContext *ftcontext) const
 {
   XPath2MemoryManager *mm = ftcontext->context->getMemoryManager();
@@ -168,6 +196,14 @@ FTSelection *FTWords::optimizePhrase(Result strings, FTContext *ftcontext) const
   return result;
 }
 
+AllMatches *FTWords::executePhrase(Result strings, FTContext *ftcontext) const
+{
+  AllMatches *result = executeAllWords(strings, ftcontext);
+  result = new FTOrderMatches(this, result);
+  result = new FTDistanceExactlyMatches(this, 0, FTOption::WORDS, result);
+  return result;
+}
+
 FTSelection *FTWords::optimizeAny(Result strings, FTContext *ftcontext) const
 {
   XPath2MemoryManager *mm = ftcontext->context->getMemoryManager();
@@ -177,12 +213,25 @@ FTSelection *FTWords::optimizeAny(Result strings, FTContext *ftcontext) const
 
   Item::Ptr item;
   while((item = strings->next(ftcontext->context)).notNull()) {
-    ftor->addArg(optimizePhrase(Sequence(item), ftcontext));
+    ftor->addArg(optimizePhrase(item, ftcontext));
   }
 
   return ftor;
 }
 
+AllMatches *FTWords::executeAny(Result strings, FTContext *ftcontext) const
+{
+  FTDisjunctionMatches *disjunction = new FTDisjunctionMatches(this);
+  AllMatches *result(disjunction);
+
+  Item::Ptr item;
+  while((item = strings->next(ftcontext->context)).notNull()) {
+    disjunction->addMatches(executePhrase(item, ftcontext));
+  }
+
+  return result;
+}
+
 FTSelection *FTWords::optimizeAll(Result strings, FTContext *ftcontext) const
 {
   XPath2MemoryManager *mm = ftcontext->context->getMemoryManager();
@@ -192,15 +241,46 @@ FTSelection *FTWords::optimizeAll(Result strings, FTContext *ftcontext) const
 
   Item::Ptr item;
   while((item = strings->next(ftcontext->context)).notNull()) {
-    ftand->addArg(optimizePhrase(Sequence(item), ftcontext));
+    ftand->addArg(optimizePhrase(item, ftcontext));
   }
 
   return ftand;
 }
 
-AllMatches::Ptr FTWords::execute(FTContext *ftcontext) const
+AllMatches *FTWords::executeAll(Result strings, FTContext *ftcontext) const
 {
-  assert(0);
+  FTConjunctionMatches *conjunction = new FTConjunctionMatches(this);
+  AllMatches *result(conjunction);
+
+  Item::Ptr item;
+  while((item = strings->next(ftcontext->context)).notNull()) {
+    conjunction->addMatches(executePhrase(item, ftcontext));
+  }
+
+  return result;
+}
+
+AllMatches *FTWords::execute(FTContext *ftcontext) const
+{
+  Result strings = expr_->createResult(ftcontext->context);
+
+  switch(option_) {
+  case ANY_WORD: {
+    return executeAnyWord(strings, ftcontext);
+  }
+  case ALL_WORDS: {
+    return executeAllWords(strings, ftcontext);
+  }
+  case PHRASE: {
+    return executePhrase(strings, ftcontext);
+  }
+  case ANY: {
+    return executeAny(strings, ftcontext);
+  }
+  case ALL: {
+    return executeAll(strings, ftcontext);
+  }
+  }
   return 0;
 }
 
@@ -217,18 +297,18 @@ FTSelection *FTWord::staticResolution(StaticContext *context)
   return this;
 }
 
-FTSelection *FTWord::staticTyping(StaticContext *context, StaticTyper *styper)
+FTSelection *FTWord::staticTypingImpl(StaticContext *context)
 {
   src_.clear();
   return this;
 }
 
-FTSelection *FTWord::optimize(FTContext *context, bool execute) const
+FTSelection *FTWord::optimize(FTContext *context) const
 {
   return const_cast<FTWord*>(this);
 }
 
-AllMatches::Ptr FTWord::execute(FTContext *ftcontext) const
+AllMatches *FTWord::execute(FTContext *ftcontext) const
 {
   return new FTStringSearchMatches(this, queryString_, ftcontext);
 }
@@ -237,29 +317,36 @@ AllMatches::Ptr FTWord::execute(FTContext *ftcontext) const
 
 FTStringSearchMatches::FTStringSearchMatches(const LocationInfo *info, const XMLCh *queryString, FTContext *ftcontext)
   : AllMatches(info),
-    queryString_(queryString),
     queryPos_(ftcontext->queryPos++),
-    tokenStream_(ftcontext->tokenStore->findTokens(queryString))
+    tokenStream_(ftcontext->tokenStore->findTokens(queryString)),
+    includes_(),
+    excludes_()
 {
 }
 
-Match::Ptr FTStringSearchMatches::next(DynamicContext *context)
+bool FTStringSearchMatches::next(DynamicContext *context)
 {
-  if(tokenStream_.isNull()) return 0;
+  includes_.clear();
+  if(tokenStream_.isNull()) return false;
 
-  TokenInfo::Ptr token = tokenStream_->next();
-  if(token.isNull()) {
+  TokenInfo token = tokenStream_->next();
+  if(token.word_ == 0) {
     tokenStream_ = 0;
-    return 0;
+    return false;
   }
 
   // TBD query position
-  Match::Ptr match = new Match();
-  match->addStringInclude(queryString_, queryPos_, token);
-  return match;
+  StringMatch sm(queryPos_, token);
+  includes_.push_back(sm);
+  return true;
 }
 
-AllMatches::Ptr FTStringSearchMatches::optimize()
+const StringMatches &FTStringSearchMatches::getStringIncludes()
 {
-  return this;
+  return includes_;
+}
+
+const StringMatches &FTStringSearchMatches::getStringExcludes()
+{
+  return excludes_;
 }
