@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -35,10 +33,12 @@
 #include <xercesc/util/XMLUni.hpp>
 #include <xqilla/context/ItemFactory.hpp>
 
+XERCES_CPP_NAMESPACE_USE;
+
 const XMLCh FunctionContains::name[] = {
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chNull 
+  chLatin_c, chLatin_o, chLatin_n, 
+  chLatin_t, chLatin_a, chLatin_i, 
+  chLatin_n, chLatin_s, chNull 
 };
 const unsigned int FunctionContains::minArgs = 2;
 const unsigned int FunctionContains::maxArgs = 3;
@@ -49,12 +49,11 @@ const unsigned int FunctionContains::maxArgs = 3;
 **/
 
 FunctionContains::FunctionContains(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : ConstantFoldingFunction(name, minArgs, maxArgs, "string?,string?,string", args, memMgr)
+  : XQFunction(name, "($arg1 as xs:string?, $arg2 as xs:string?, $collation as xs:string) as xs:boolean", args, memMgr)
 {
-  _src.getStaticType() = StaticType::BOOLEAN_TYPE;
 }
 
-Sequence FunctionContains::createSequence(DynamicContext* context, int flags) const
+BoolResult FunctionContains::boolResult(DynamicContext* context) const
 {
     Sequence str1 = getParamNumber(1,context)->toSequence(context);
     Sequence str2 = getParamNumber(2,context)->toSequence(context);
@@ -75,18 +74,20 @@ Sequence FunctionContains::createSequence(DynamicContext* context, int flags) co
     else
         collation=context->getCollation(CodepointCollation::getCodepointCollationName(), this);
 
-    const XMLCh* container = XERCES_CPP_NAMESPACE_QUALIFIER XMLUni::fgZeroLenString;
+    const XMLCh* container = XMLUni::fgZeroLenString;
     if(!str1.isEmpty())
         container=str1.first()->asString(context);
-    const XMLCh* pattern = XERCES_CPP_NAMESPACE_QUALIFIER XMLUni::fgZeroLenString;
+    const XMLCh* pattern = XMLUni::fgZeroLenString;
     if(!str2.isEmpty())
         pattern=str2.first()->asString(context);
 
-    if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(pattern)==0)
-        return Sequence(context->getItemFactory()->createBoolean(true, context), context->getMemoryManager());
-    else if(XERCES_CPP_NAMESPACE_QUALIFIER XMLString::stringLen(container)==0)
-        return Sequence(context->getItemFactory()->createBoolean(false, context), context->getMemoryManager());
+    if(XMLString::stringLen(pattern)==0) return true;
+    else if(XMLString::stringLen(container)==0) return false;
 
-    bool contains = XERCES_CPP_NAMESPACE_QUALIFIER XMLString::patternMatch(container, pattern) > -1;
-    return Sequence(context->getItemFactory()->createBoolean(contains, context), context->getMemoryManager());
+    return XMLString::patternMatch(container, pattern) > -1;
+}
+
+Result FunctionContains::createResult(DynamicContext* context, int flags) const
+{
+  return (Item::Ptr)context->getItemFactory()->createBoolean(boolResult(context), context);
 }
