@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -31,10 +29,13 @@
 #include <xqilla/ast/StaticAnalysis.hpp>
 #include <xqilla/framework/XPath2MemoryManagerImpl.hpp>
 #include <xqilla/context/ItemFactory.hpp>
+#include <xqilla/ast/XQContextItem.hpp>
+
+XERCES_CPP_NAMESPACE_USE;
 
 const XMLCh FunctionIdref::name[] = {
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_d, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_r, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_e, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_f, XERCES_CPP_NAMESPACE_QUALIFIER chNull 
+  chLatin_i, chLatin_d, chLatin_r, 
+  chLatin_e, chLatin_f, chNull 
 };
 const unsigned int FunctionIdref::minArgs = 1;
 const unsigned int FunctionIdref::maxArgs = 2;
@@ -45,46 +46,27 @@ const unsigned int FunctionIdref::maxArgs = 2;
  **/
 
 FunctionIdref::FunctionIdref(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "string*, node()", args, memMgr)
+  : XQFunction(name, "($arg as xs:string*, $node as node()) as node()*", args, memMgr)
 {
 }
 
-ASTNode* FunctionIdref::staticResolution(StaticContext *context) {
-  if(_args.size()==2 && _args.back()->getType()==ASTNode::CONTEXT_ITEM)
-      _args.pop_back();
-  return resolveArguments(context);
-}
-
-ASTNode *FunctionIdref::staticTypingImpl(StaticContext *context)
+ASTNode* FunctionIdref::staticResolution(StaticContext *context)
 {
-  _src.clear();
+  XPath2MemoryManager *mm = context->getMemoryManager();
 
-  _src.getStaticType() = StaticType(StaticType::NODE_TYPE, 0, StaticType::UNLIMITED);
-  if(_args.size()==1)
-    _src.contextItemUsed(true);
-  return calculateSRCForArguments(context);
+  if(_args.size() == 1) {
+    XQContextItem *ci = new (mm) XQContextItem(mm);
+    ci->setLocationInfo(this);
+    _args.push_back(ci);
+  }
+
+  resolveArguments(context);
+  return this;
 }
 
 Sequence FunctionIdref::createSequence(DynamicContext* context, int flags) const
 {
-  Node::Ptr ctxNode;
-  if(getNumArgs() == 2)
-  {
-    Sequence arg=getParamNumber(2,context)->toSequence(context);
-    ctxNode=arg.first();
-  }
-  else
-  {
-    const Item::Ptr item = context->getContextItem();
-    if(item==NULLRCP)
-      XQThrow(FunctionException, X("FunctionIdref::createSequence"),X("Undefined context item in fn:idref [err:XPDY0002]"));
-    if(!item->isNode())
-      XQThrow(FunctionException, X("FunctionIdref::createSequence"),X("The context item is not a node [err:XPTY0004]"));
-    ctxNode=item;
-  }
-
-  Node::Ptr root = ctxNode->root(context);
-
+  Node::Ptr root = ((Node*)getParamNumber(2, context)->next(context).get())->root(context);
   if(root->dmNodeKind() != Node::document_string) {
     XQThrow(FunctionException,X("FunctionIdref::createSequence"), X("Current context doesn't belong to a document [err:FODC0001]"));
   }
@@ -102,7 +84,7 @@ Sequence FunctionIdref::createSequence(DynamicContext* context, int flags) const
     //for each string check that it is lexically a xs:ID, if not ignore it
     bool validID = true;
     try {
-      context->getItemFactory()->createStringOrDerived(XERCES_CPP_NAMESPACE_QUALIFIER SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XERCES_CPP_NAMESPACE_QUALIFIER XMLUni::fgIDString, str, context);
+      context->getItemFactory()->createStringOrDerived(SchemaSymbols::fgURI_SCHEMAFORSCHEMA, XMLUni::fgIDString, str, context);
     } catch (InvalidLexicalSpaceException &e) {
       validID = false;
     }
