@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -24,12 +22,14 @@
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/ATDecimalOrDerived.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
-#include <xqilla/exceptions/FunctionException.hpp>
+#include <xqilla/exceptions/DynamicErrorException.hpp>
 #include <xqilla/ast/StaticAnalysis.hpp>
 
+XERCES_CPP_NAMESPACE_USE;
+
 const XMLCh FunctionLast::name[] = {
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_l, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_a, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chNull 
+  chLatin_l, chLatin_a, chLatin_s, 
+  chLatin_t, chNull 
 };
 const unsigned int FunctionLast::minArgs = 0;
 const unsigned int FunctionLast::maxArgs = 0;
@@ -39,21 +39,22 @@ const unsigned int FunctionLast::maxArgs = 0;
 **/
 
 FunctionLast::FunctionLast(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "empty()", args, memMgr)
+  : XQFunction(name, "() as xs:integer", args, memMgr)
 {
 }
 
-ASTNode* FunctionLast::staticResolution(StaticContext *context) {
-  return resolveArguments(context);
-}
-
 ASTNode *FunctionLast::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
+  _src.clearExceptType();
+
+  if(context && !context->getContextItemType().containsType(StaticType::ITEM_TYPE)) {
+    XQThrow(DynamicErrorException,X("XQContextItem::staticTyping"),
+            X("It is an error for the context item to be undefined when using it [err:XPDY0002]"));
+  }
 
-  _src.getStaticType() = StaticType::DECIMAL_TYPE;
   _src.contextSizeUsed(true);
-  return calculateSRCForArguments(context);
+  calculateSRCForArguments(context);
+  return this;
 }
 
 Sequence FunctionLast::createSequence(DynamicContext* context, int flags) const
@@ -61,7 +62,7 @@ Sequence FunctionLast::createSequence(DynamicContext* context, int flags) const
   XPath2MemoryManager* memMgr = context->getMemoryManager();
 
   if(context->getContextItem()==NULLRCP)
-    XQThrow(FunctionException,X("FunctionPosition::createSequence"), X("Undefined context item in fn:last [err:XPDY0002]"));
+    XQThrow(DynamicErrorException,X("FunctionPosition::createSequence"), X("Undefined context item in fn:last [err:XPDY0002]"));
   return Sequence(context->getItemFactory()->createInteger((long)context->getContextSize(), context), memMgr);
 }
 
