@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -26,6 +24,7 @@
 #include <xqilla/functions/ExternalFunction.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
 #include <xqilla/framework/XPath2MemoryManagerImpl.hpp>
+#include <xqilla/utils/XPath2Utils.hpp>
 
 XERCES_CPP_NAMESPACE_USE
 
@@ -46,8 +45,6 @@ FunctionLookup::~FunctionLookup()
 
 void FunctionLookup::insertFunction(FuncFactory *func)
 {
-  size_t secondaryKey = SECONDARY_KEY(func);
-
   // Use similar algorithm to lookup in order to detect overlaps
   // in argument numbers
   RefHash2KeysTableOfEnumerator<FuncFactory> iterator(const_cast<RefHash2KeysTableOf< FuncFactory >* >(&_funcTable));
@@ -68,14 +65,17 @@ void FunctionLookup::insertFunction(FuncFactory *func)
       buf.append(func->getURI());
       buf.append(X("}"));
       buf.append(func->getName());
-      buf.append(X("/"));
-      XMLCh szInt[10];
-      XMLString::binToText((unsigned int)secondaryKey,szInt,9,10);
-      buf.append(szInt);
+      buf.append(X("#"));
+      if(func->getMinArgs() >= entry->getMinArgs() &&
+         func->getMinArgs() <= entry->getMaxArgs())
+        XPath2Utils::numToBuf((unsigned int)func->getMinArgs(), buf);
+      else
+        XPath2Utils::numToBuf((unsigned int)entry->getMinArgs(), buf);
       buf.append(X(" [err:XQST0034]."));
       XQThrow2(StaticErrorException,X("FunctionLookup::insertFunction"), buf.getRawBuffer());
     }
   // Ok to add function
+  size_t secondaryKey = SECONDARY_KEY(func);
   _funcTable.put((void*)func->getURINameHash(), (int)secondaryKey, func);
 }
 
@@ -102,6 +102,7 @@ ASTNode* FunctionLookup::lookUpFunction(const XMLCh* URI, const XMLCh* fname,
   //
   XMLBuffer key;
   key.set(fname);
+  key.append(':');
   key.append(URI);
   iterator.setPrimaryKey(key.getRawBuffer());
   size_t nargs = args.size();
@@ -129,6 +130,7 @@ const ExternalFunction *FunctionLookup::lookUpExternalFunction(
   size_t secondaryKey = numArgs;
   XMLBuffer key;
   key.set(fname);
+  key.append(':');
   key.append(URI);
   return _exFuncTable.get(key.getRawBuffer(), (int)secondaryKey);
 }
@@ -146,6 +148,12 @@ void FunctionLookup::copyExternalFunctionsTo(DynamicContext *context) const
  */
 static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr);
 
+// static
+XPath2MemoryManager* FunctionLookup::getMemoryManager()
+{
+  return g_memMgr;
+}
+
 // static
 void FunctionLookup::initialize()
 {
@@ -196,12 +204,9 @@ ASTNode* FunctionLookup::lookUpGlobalFunction(
                                               XPath2MemoryManager* memMgr,
                                               const FunctionLookup *contextTable)
 {
-  ASTNode *ast = g_globalFunctionTable->lookUpFunction(
-                                                       URI, fname, args, memMgr);
-  if (!ast && contextTable)
-    ast = contextTable->lookUpFunction(
-                                       URI, fname, args, memMgr);
-  return ast;
+  if(contextTable)
+    return contextTable->lookUpFunction(URI, fname, args, memMgr);
+  return g_globalFunctionTable->lookUpFunction(URI, fname, args, memMgr);
 }
 
 // static
@@ -218,25 +223,19 @@ const ExternalFunction *FunctionLookup::lookUpGlobalExternalFunction(
   return ef;
 }
 
-#include "FuncFactoryTemplate.hpp"
+#include <xqilla/functions/FuncFactoryTemplate.hpp>
 #include <xqilla/functions/FunctionAbs.hpp>
-#include <xqilla/functions/FunctionAvg.hpp>
 #include <xqilla/functions/FunctionBaseURI.hpp>
-#include <xqilla/functions/FunctionBoolean.hpp>
 #include <xqilla/functions/FunctionCeiling.hpp>
 #include <xqilla/functions/FunctionCollection.hpp>
 #include <xqilla/functions/FunctionCompare.hpp>
-#include <xqilla/functions/FunctionCodepointEqual.hpp>
 #include <xqilla/functions/FunctionConcat.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/functions/FunctionContains.hpp>
 #include <xqilla/functions/FunctionCount.hpp>
 #include <xqilla/functions/FunctionCurrentDate.hpp>
 #include <xqilla/functions/FunctionCurrentDateTime.hpp>
 #include <xqilla/functions/FunctionCurrentTime.hpp>
-#include <xqilla/functions/FunctionData.hpp>
 #include <xqilla/functions/FunctionDateTime.hpp>
-#include <xqilla/functions/FunctionDeepEqual.hpp>
 #include <xqilla/functions/FunctionDefaultCollation.hpp>
 #include <xqilla/functions/FunctionDistinctValues.hpp>
 #include <xqilla/functions/FunctionDoc.hpp>
@@ -244,33 +243,21 @@ const ExternalFunction *FunctionLookup::lookUpGlobalExternalFunction(
 #include <xqilla/functions/FunctionDocAvailable.hpp>
 #include <xqilla/functions/FunctionDocumentURI.hpp>
 #include <xqilla/functions/FunctionEmpty.hpp>
-#include <xqilla/functions/FunctionEncodeForUri.hpp>
 #include <xqilla/functions/FunctionEndsWith.hpp>
 #include <xqilla/functions/FunctionError.hpp>
-#include <xqilla/functions/FunctionEscapeHtmlUri.hpp>
-#include <xqilla/functions/FunctionExactlyOne.hpp>
-#include <xqilla/functions/FunctionExists.hpp>
 #include <xqilla/functions/FunctionQName.hpp>
-#include <xqilla/functions/FunctionFalse.hpp>
 #include <xqilla/functions/FunctionFloor.hpp>
-#include <xqilla/functions/FunctionInScopePrefixes.hpp>
 #include <xqilla/functions/FunctionPrefixFromQName.hpp>
 #include <xqilla/functions/FunctionLocalNameFromQName.hpp>
 #include <xqilla/functions/FunctionNamespaceURIFromQName.hpp>
-#include <xqilla/functions/FunctionNamespaceURIForPrefix.hpp>
 #include <xqilla/functions/FunctionId.hpp>
 #include <xqilla/functions/FunctionIdref.hpp>
 #include <xqilla/functions/FunctionImplicitTimezone.hpp>
-#include <xqilla/functions/FunctionIndexOf.hpp>
-#include <xqilla/functions/FunctionInsertBefore.hpp>
-#include <xqilla/functions/FunctionIriToUri.hpp>
 #include <xqilla/functions/FunctionLang.hpp>
 #include <xqilla/functions/FunctionLast.hpp>
 #include <xqilla/functions/FunctionLocalname.hpp>
 #include <xqilla/functions/FunctionLowerCase.hpp>
 #include <xqilla/functions/FunctionMatches.hpp>
-#include <xqilla/functions/FunctionMax.hpp>
-#include <xqilla/functions/FunctionMin.hpp>
 #include <xqilla/functions/FunctionName.hpp>
 #include <xqilla/functions/FunctionNamespaceUri.hpp>
 #include <xqilla/functions/FunctionNilled.hpp>
@@ -279,35 +266,23 @@ const ExternalFunction *FunctionLookup::lookUpGlobalExternalFunction(
 #include <xqilla/functions/FunctionNormalizeUnicode.hpp>
 #include <xqilla/functions/FunctionNot.hpp>
 #include <xqilla/functions/FunctionNumber.hpp>
-#include <xqilla/functions/FunctionOneOrMore.hpp>
 #include <xqilla/functions/FunctionPosition.hpp>
-#include <xqilla/functions/FunctionRemove.hpp>
 #include <xqilla/functions/FunctionReplace.hpp>
-#include <xqilla/functions/FunctionResolveQName.hpp>
 #include <xqilla/functions/FunctionResolveURI.hpp>
-#include <xqilla/functions/FunctionReverse.hpp>
 #include <xqilla/functions/FunctionRoot.hpp>
 #include <xqilla/functions/FunctionRound.hpp>
 #include <xqilla/functions/FunctionRoundHalfToEven.hpp>
 #include <xqilla/functions/FunctionStartsWith.hpp>
 #include <xqilla/functions/FunctionStaticBaseURI.hpp>
 #include <xqilla/functions/FunctionString.hpp>
-#include <xqilla/functions/FunctionStringJoin.hpp>
 #include <xqilla/functions/FunctionStringLength.hpp>
 #include <xqilla/functions/FunctionStringToCodepoints.hpp>
 #include <xqilla/functions/FunctionCodepointsToString.hpp>
-#include <xqilla/functions/FunctionSubsequence.hpp>
 #include <xqilla/functions/FunctionSubstring.hpp>
-#include <xqilla/functions/FunctionSubstringAfter.hpp>
-#include <xqilla/functions/FunctionSubstringBefore.hpp>
-#include <xqilla/functions/FunctionSum.hpp>
 #include <xqilla/functions/FunctionTokenize.hpp>
 #include <xqilla/functions/FunctionTrace.hpp>
-#include <xqilla/functions/FunctionTranslate.hpp>
-#include <xqilla/functions/FunctionTrue.hpp>
 #include <xqilla/functions/FunctionUnordered.hpp>
 #include <xqilla/functions/FunctionUpperCase.hpp>
-#include <xqilla/functions/FunctionZeroOrOne.hpp>
 #include <xqilla/functions/FunctionYearsFromDuration.hpp>
 #include <xqilla/functions/FunctionMonthsFromDuration.hpp>
 #include <xqilla/functions/FunctionDaysFromDuration.hpp>
@@ -332,6 +307,8 @@ const ExternalFunction *FunctionLookup::lookUpGlobalExternalFunction(
 #include <xqilla/functions/FunctionAdjustDateTimeToTimezone.hpp>
 #include <xqilla/functions/FunctionAdjustDateToTimezone.hpp>
 #include <xqilla/functions/FunctionAdjustTimeToTimezone.hpp>
+#include <xqilla/functions/FunctionHead.hpp>
+#include <xqilla/functions/FunctionTail.hpp>
 // Updates
 #include <xqilla/update/FunctionPut.hpp>
 // XSLT 2.0 functions
@@ -344,10 +321,24 @@ const ExternalFunction *FunctionLookup::lookUpGlobalExternalFunction(
 #include <xqilla/functions/FunctionParseHTML.hpp>
 #include <xqilla/functions/FunctionTime.hpp>
 #include <xqilla/functions/FunctionAnalyzeString.hpp>
+#include <xqilla/functions/FunctionCaseFold.hpp>
+#include <xqilla/functions/FunctionWords.hpp>
+#include <xqilla/functions/FunctionSentences.hpp>
+#include <xqilla/functions/FunctionExplain.hpp>
 // Higher Order Functions extension functions
-#include <xqilla/functions/FunctionPartialApply.hpp>
 #include <xqilla/functions/FunctionFunctionArity.hpp>
 #include <xqilla/functions/FunctionFunctionName.hpp>
+//exslt math functions
+#include <xqilla/functions/FunctionSqrt.hpp>
+#include <xqilla/functions/FunctionSin.hpp>
+#include <xqilla/functions/FunctionCos.hpp>
+#include <xqilla/functions/FunctionTan.hpp>
+#include <xqilla/functions/FunctionAcos.hpp>
+#include <xqilla/functions/FunctionAsin.hpp>
+#include <xqilla/functions/FunctionAtan.hpp>
+#include <xqilla/functions/FunctionLog.hpp>
+#include <xqilla/functions/FunctionExp.hpp>
+#include <xqilla/functions/FunctionPower.hpp>
 
 void FunctionLookup::insertUpdateFunctions(XPath2MemoryManager *memMgr)
 {
@@ -367,8 +358,6 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionNilled>(memMgr));
   //   fn:string
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionString>(memMgr));
-  //   fn:data
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionData>(memMgr));
   //   fn:base-uri
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionBaseURI>(memMgr));
   //   fn:document-uri
@@ -404,12 +393,8 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionStringToCodepoints>(memMgr));
   //   fn:compare
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionCompare>(memMgr));
-  //   fn:codepoint-equal
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionCodepointEqual>(memMgr));
   //   fn:concat
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionConcat>(memMgr));
-  //   fn:string-join
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionStringJoin>(memMgr));
   //   fn:substring
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSubstring>(memMgr));
   //   fn:string-length
@@ -422,24 +407,12 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionUpperCase>(memMgr));
   //   fn:lower-case
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionLowerCase>(memMgr));
-  //   fn:translate
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTranslate>(memMgr));
-  //   fn:encode-for-uri
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionEncodeForUri>(memMgr));
-  //   fn:iri-to-uri
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionIriToUri>(memMgr));
-  //   fn:escape-html-uri
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionEscapeHtmlUri>(memMgr));
   //   fn:contains
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionContains>(memMgr));
   //   fn:starts-with
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionStartsWith>(memMgr));
   //   fn:ends-with
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionEndsWith>(memMgr));
-  //   fn:substring-before
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSubstringBefore>(memMgr));
-  //   fn:substring-after
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSubstringAfter>(memMgr));
 
   //   fn:matches
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionMatches>(memMgr));
@@ -449,10 +422,6 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTokenize>(memMgr));
 
   // Functions on boolean values
-  //   fn:true
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTrue>(memMgr));
-  //   fn:false
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionFalse>(memMgr));
   //   fn:not
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionNot>(memMgr));
 
@@ -507,8 +476,6 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAdjustTimeToTimezone>(memMgr));
 
   // Functions on QName values
-  //   fn:resolve-QName
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionResolveQName>(memMgr));
   //   fn:QName
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionQName>(memMgr));
   //   fn:prefix-from-QName
@@ -517,10 +484,6 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionLocalNameFromQName>(memMgr));
   //   fn:namespace-uri-from-QName
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionNamespaceURIFromQName>(memMgr));
-  //   fn:namespace-uri-for-prefix
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionNamespaceURIForPrefix>(memMgr));
-  //   fn:in-scope-prefixes
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionInScopePrefixes>(memMgr));
 
   // Functions on anyURI values
   //   fn:resolve-URI
@@ -541,46 +504,19 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionRoot>(memMgr));
 
   // Functions on sequences
-  //   fn::zero-or-one
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionZeroOrOne>(memMgr));
-  //   fn::one-or-more
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionOneOrMore>(memMgr));
-  //   fn::exactly-one
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionExactlyOne>(memMgr));
-  //   fn:boolean
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionBoolean>(memMgr));
-  //   fn:index-of
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionIndexOf>(memMgr));
   //   fn:empty
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionEmpty>(memMgr));
-  //   fn:exists
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionExists>(memMgr));
   //   fn:distinct-values
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionDistinctValues>(memMgr));
-  //   fn:insert-before
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionInsertBefore>(memMgr));
-  //   fn:remove
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionRemove>(memMgr));
-  //   fn:reverse
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionReverse>(memMgr));
-  //   fn:subsequence
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSubsequence>(memMgr));
   //   fn:unordered
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionUnordered>(memMgr));
+  //   fn:head
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionHead>(memMgr));
+  //   fn:tail
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTail>(memMgr));
   
-  //   fn:deep-equal
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionDeepEqual>(memMgr));
-
   //   fn:count
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionCount>(memMgr));
-  //   fn:avg
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAvg>(memMgr));
-  //   fn:max
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionMax>(memMgr));
-  //   fn:min
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionMin>(memMgr));
-  //   fn:sum
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSum>(memMgr));
 
   //   fn:id
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionId>(memMgr));
@@ -622,13 +558,30 @@ static void initGlobalTable(FunctionLookup *t, MemoryManager *memMgr)
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSerializeJSON>(memMgr));
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTime>(memMgr));
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAnalyzeString>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionCaseFold>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionWords>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSentences>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionExplain>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionDrop>(memMgr));
 
 #ifdef HAVE_LIBTIDY
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionParseHTML>(memMgr));
 #endif
 
   // Higher Order Functions extension functions
-  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionPartialApply>(memMgr));
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionFunctionArity>(memMgr));
   t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionFunctionName>(memMgr));
+
+  //exslt math functions
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAbs>(EXSLTMathFunction::XMLChFunctionURI, FunctionAbs::name, 1, 1, memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSqrt>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionSin>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionCos>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionTan>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAcos>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAsin>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionAtan>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionLog>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionExp>(memMgr));
+  t->insertFunction(new (memMgr) FuncFactoryTemplate<FunctionPower>(memMgr));
 }
