@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -38,7 +36,7 @@ const unsigned int FunctionNot::maxArgs = 1;
 **/
 
 FunctionNot::FunctionNot(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : ConstantFoldingFunction(name, minArgs, maxArgs, "item()*", args, memMgr)
+  : XQFunction(name, "($arg as item()*) as xs:boolean", args, memMgr)
 {
 }
 
@@ -48,21 +46,17 @@ ASTNode* FunctionNot::staticResolution(StaticContext *context)
 
   _args[0] = new (mm) XQEffectiveBooleanValue(_args[0], mm);
   _args[0]->setLocationInfo(this);
-  _args[0] = _args[0]->staticResolution(context);
 
+  resolveArguments(context);
   return this;
 }
 
-ASTNode *FunctionNot::staticTypingImpl(StaticContext *context)
+BoolResult FunctionNot::boolResult(DynamicContext* context) const
 {
-  _src.clear();
-  _src.copy(_args[0]->getStaticAnalysis());
-  return this;
+  return !_args[0]->boolResult(context);
 }
 
-Sequence FunctionNot::createSequence(DynamicContext* context, int flags) const
+Result FunctionNot::createResult(DynamicContext* context, int flags) const
 {
-  bool result = !((ATBooleanOrDerived*)getParamNumber(1,context)->next(context).get())->isTrue();
-  XPath2MemoryManager* memMgr = context->getMemoryManager();
-  return Sequence(context->getItemFactory()->createBoolean(result, context), memMgr);
+  return (Item::Ptr)context->getItemFactory()->createBoolean(boolResult(context), context);
 }
