@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -50,7 +48,7 @@ const unsigned int FunctionNumber::maxArgs = 1;
 **/
 
 FunctionNumber::FunctionNumber(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "anyAtomicType?", args, memMgr)
+  : XQFunction(name, "($arg as xs:anyAtomicType?) as xs:double", args, memMgr)
 {
 }
 
@@ -64,14 +62,8 @@ ASTNode* FunctionNumber::staticResolution(StaticContext *context)
     _args.push_back(ci);
   }
 
-  return resolveArguments(context);
-}
-
-ASTNode *FunctionNumber::staticTypingImpl(StaticContext *context)
-{
-  _src.clear();
-  _src.getStaticType() = StaticType::DOUBLE_TYPE;
-  return calculateSRCForArguments(context);
+  resolveArguments(context);
+  return this;
 }
 
 Sequence FunctionNumber::createSequence(DynamicContext* context, int flags) const
@@ -107,7 +99,7 @@ Numeric::Ptr FunctionNumber::number(const Item::Ptr &item, DynamicContext *conte
     tmp = atomized->next(context);
 
     if(tmp.notNull() && atomized->next(context).notNull()) {
-      XQThrow3(XPath2TypeMatchException, X("XPath1CompatConvertFunctionArgResult::next"),
+      XQThrow3(XPath2TypeMatchException, X("FunctionNumber::number"),
                X("Sequence does not match type xs:anyAtomicType? - found more than one item [err:XPTY0004]"), location);
     }
   }
@@ -117,7 +109,7 @@ Numeric::Ptr FunctionNumber::number(const Item::Ptr &item, DynamicContext *conte
     buf.append(X(" - found item of type "));
     item->typeToBuffer(context, buf);
     buf.append(X(" [err:XPTY0004]"));
-    XQThrow3(XPath2TypeMatchException, X("XPath1CompatConvertFunctionArgResult::next"), buf.getRawBuffer(), location);
+    XQThrow3(XPath2TypeMatchException, X("FunctionNumber::number"), buf.getRawBuffer(), location);
   }
 
   return number((AnyAtomicType*)tmp.get(), context);
