@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -38,24 +36,26 @@ const unsigned int FunctionParseXML::minArgs = 1;
 const unsigned int FunctionParseXML::maxArgs = 1;
 
 /**
- * xqilla:parse-xml($xml as xs:string?) as document?
+ * xqilla:parse-xml($xml as xs:string?) as document-node()?
  */
 FunctionParseXML::FunctionParseXML(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQillaFunction(name, minArgs, maxArgs, "string?", args, memMgr),
+  : XQillaFunction(name, "($xml as xs:string?) as document-node()?", args, memMgr),
     queryPathTree_(0)
 {
 }
 
 ASTNode *FunctionParseXML::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
+  _src.clearExceptType();
+
+  calculateSRCForArguments(context);
 
   _src.setProperties(StaticAnalysis::DOCORDER | StaticAnalysis::GROUPED |
                      StaticAnalysis::PEER | StaticAnalysis::SUBTREE | StaticAnalysis::ONENODE);
   _src.getStaticType() = StaticType(StaticType::DOCUMENT_TYPE, 0, 1);
   _src.creative(true);
 
-  return calculateSRCForArguments(context);
+  return this;
 }
 
 Sequence FunctionParseXML::createSequence(DynamicContext* context, int flags) const
