@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -24,14 +22,16 @@
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/ATDecimalOrDerived.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
-#include <xqilla/exceptions/FunctionException.hpp>
+#include <xqilla/exceptions/DynamicErrorException.hpp>
 #include <xqilla/ast/StaticAnalysis.hpp>
 #include <xqilla/runtime/ResultImpl.hpp>
 
+XERCES_CPP_NAMESPACE_USE;
+
 const XMLCh FunctionPosition::name[] = {
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_t, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, 
-  XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_n, XERCES_CPP_NAMESPACE_QUALIFIER chNull 
+  chLatin_p, chLatin_o, chLatin_s, 
+  chLatin_i, chLatin_t, chLatin_i, 
+  chLatin_o, chLatin_n, chNull 
 };
 const unsigned int FunctionPosition::minArgs = 0;
 const unsigned int FunctionPosition::maxArgs = 0;
@@ -41,41 +41,28 @@ const unsigned int FunctionPosition::maxArgs = 0;
  */
 
 FunctionPosition::FunctionPosition(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "empty()", args, memMgr)
+  : XQFunction(name, "() as xs:integer", args, memMgr)
 {
 }
 
-ASTNode* FunctionPosition::staticResolution(StaticContext *context) {
-  return resolveArguments(context);
-}
-
 ASTNode *FunctionPosition::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
+  _src.clearExceptType();
 
-  _src.getStaticType() = StaticType::DECIMAL_TYPE;
-  _src.contextPositionUsed(true);
-  return calculateSRCForArguments(context);
-}
-
-class FunctionPositionResult : public ResultImpl
-{
-public:
-  FunctionPositionResult(const LocationInfo *o)
-    : ResultImpl(o) {}
-
-  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context)
-  {
-    if(context->getContextItem().isNull())
-      XQThrow(FunctionException,X("FunctionPosition::createSequence"),
-              X("Undefined context item in fn:position [err:XPDY0002]"));
-
-    tail = 0;
-    return context->getItemFactory()->createInteger((long)context->getContextPosition(), context);
+  if(context && !context->getContextItemType().containsType(StaticType::ITEM_TYPE)) {
+    XQThrow(DynamicErrorException,X("XQContextItem::staticTyping"),
+            X("It is an error for the context item to be undefined when using it [err:XPDY0002]"));
   }
-};
+
+  _src.contextPositionUsed(true);
+  calculateSRCForArguments(context);
+  return this;
+}
 
 Result FunctionPosition::createResult(DynamicContext* context, int flags) const
 {
-  return new FunctionPositionResult(this);
+  if(context->getContextItem().isNull())
+    XQThrow(DynamicErrorException,X("FunctionPosition::createSequence"),
+            X("Undefined context item in fn:position [err:XPDY0002]"));
+  return (Item::Ptr)context->getItemFactory()->createInteger((long)context->getContextPosition(), context);
 }
