@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -24,9 +22,12 @@
 #include <xqilla/functions/FunctionResolveURI.hpp>
 #include <xqilla/items/ATStringOrDerived.hpp>
 #include <xqilla/exceptions/FunctionException.hpp>
+#include <xqilla/exceptions/StaticErrorException.hpp>
 #include "../exceptions/InvalidLexicalSpaceException.hpp"
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
+#include <xqilla/ast/XQLiteral.hpp>
+
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/util/XMLUri.hpp>
 
@@ -47,26 +48,27 @@ const unsigned int FunctionResolveURI::maxArgs = 2;
  **/
 
 FunctionResolveURI::FunctionResolveURI(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "string?, string", args, memMgr),
-    baseURI_(0)
+  : XQFunction(name, "($relative as xs:string?, $base as xs:string) as xs:anyURI?", args, memMgr)
 {
 }
 
 ASTNode* FunctionResolveURI::staticResolution(StaticContext *context)
 {
-  if (getNumArgs() == 1) {
-    baseURI_ = context->getBaseURI();
+  if(getNumArgs() == 1) {
+    if(!context->getBaseURI())
+      XQThrow(StaticErrorException, X("FunctionResolveURI::staticResolution"),
+              X("Base uri undefined in the static context [err:FONS0005]"));
+
+    XPath2MemoryManager* mm = context->getMemoryManager();
+
+    ASTNode *baseURI = new (mm) XQLiteral(SchemaSymbols::fgURI_SCHEMAFORSCHEMA, SchemaSymbols::fgDT_STRING,
+                                          context->getBaseURI(), AnyAtomicType::STRING, mm);
+    baseURI->setLocationInfo(this);
+    _args.push_back(baseURI);
   }
 
-  return resolveArguments(context);
-}
-
-ASTNode *FunctionResolveURI::staticTypingImpl(StaticContext *context)
-{
-  _src.clear();
-
-  _src.getStaticType() = StaticType(StaticType::ANY_URI_TYPE, 0, 1);
-  return calculateSRCForArguments(context);
+  resolveArguments(context);
+  return this;
 }
 
 Sequence FunctionResolveURI::createSequence(DynamicContext* context, int flags) const
@@ -86,16 +88,7 @@ Sequence FunctionResolveURI::createSequence(DynamicContext* context, int flags)
   }
 
   try {
-    const XMLCh *baseURI;
-    if(getNumArgs() == 1) {
-      baseURI = baseURI_;
-      if(!baseURI)
-        XQThrow(FunctionException, X("FunctionResolveURI::createSequence"),
-                X("Base uri undefined in the static context [err:FONS0005]"));
-    }
-    else {
-      baseURI = getParamNumber(2, context)->next(context)->asString(context);
-    }
+    const XMLCh *baseURI = getParamNumber(2, context)->next(context)->asString(context);
 
     if(!XMLUri::isValidURI(true, relativeURI))
       XQThrow(FunctionException, X("FunctionResolveURI::createSequence"),
