@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -50,7 +48,7 @@ const unsigned int FunctionStringLength::maxArgs = 1;
 **/
 
 FunctionStringLength::FunctionStringLength(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "string?", args, memMgr)
+  : XQFunction(name, "($arg as xs:string?) as xs:integer", args, memMgr)
 {
 }
 
@@ -65,15 +63,8 @@ ASTNode* FunctionStringLength::staticResolution(StaticContext *context)
     _args.push_back(arg);
   }
 
-  return resolveArguments(context);
-}
-
-ASTNode *FunctionStringLength::staticTypingImpl(StaticContext *context)
-{
-  _src.clear();
-
-  _src.getStaticType() = StaticType::DECIMAL_TYPE;
-  return calculateSRCForArguments(context);
+  resolveArguments(context);
+  return this;
 }
 
 Sequence FunctionStringLength::createSequence(DynamicContext* context, int flags) const
