@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -40,13 +38,12 @@ const unsigned int FunctionTimezoneFromDateTime::minArgs = 1;
 const unsigned int FunctionTimezoneFromDateTime::maxArgs = 1;
 
 /*
- * fn:timezone-from-dateTime($arg as xs:dateTime?) as xdt:dayTimeDuration?
+ * fn:timezone-from-dateTime($arg as xs:dateTime?) as xs:dayTimeDuration?
  */
 
 FunctionTimezoneFromDateTime::FunctionTimezoneFromDateTime(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : ConstantFoldingFunction(name, minArgs, maxArgs, "dateTime?", args, memMgr)
+  : XQFunction(name, "($arg as xs:dateTime?) as xs:dayTimeDuration?", args, memMgr)
 {
-  _src.getStaticType() = StaticType(StaticType::DAY_TIME_DURATION_TYPE, 0, 1);
 }
 
 Sequence FunctionTimezoneFromDateTime::createSequence(DynamicContext* context, int flags) const
