@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -40,25 +38,23 @@ const unsigned int FunctionUnordered::maxArgs = 1;
 **/
 
 FunctionUnordered::FunctionUnordered(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "item()*", args, memMgr)
+  : XQFunction(name, "($sourceSeq as item()*) as item()*", args, memMgr)
 {
 }
 
 ASTNode* FunctionUnordered::staticResolution(StaticContext *context)
 {
   AutoNodeSetOrderingReset orderReset(context);
-  return resolveArguments(context);
+  resolveArguments(context);
+  return this;
 }
 
 ASTNode *FunctionUnordered::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
-
-  ASTNode *result = calculateSRCForArguments(context);
-  if(result == this) {
-    _src.getStaticType() = _args.front()->getStaticAnalysis().getStaticType();
-  }
-  return result;
+  _src.clearExceptType();
+  calculateSRCForArguments(context);
+  _src.getStaticType() = _args.front()->getStaticAnalysis().getStaticType();
+  return this;
 }
 
 Result FunctionUnordered::createResult(DynamicContext* context, int flags) const
