@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,12 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
 #include <xqilla/functions/FunctionUpperCase.hpp>
 #include <xqilla/context/DynamicContext.hpp>
-#include <xqilla/utils/UCANormalizer.hpp>
+#include <xqilla/utils/UnicodeTransformer.hpp>
 
 #include <xercesc/util/XMLUni.hpp>
 
@@ -44,9 +42,8 @@ const unsigned int FunctionUpperCase::maxArgs = 1;
  */
 
 FunctionUpperCase::FunctionUpperCase(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : ConstantFoldingFunction(name, minArgs, maxArgs, "string?", args, memMgr)
+  : XQFunction(name, "($arg as xs:string?) as xs:string", args, memMgr)
 {
-  _src.getStaticType() = StaticType::STRING_TYPE;
 }
 
 Sequence FunctionUpperCase::createSequence(DynamicContext* context, int flags) const
@@ -58,7 +55,6 @@ Sequence FunctionUpperCase::createSequence(DynamicContext* context, int flags) c
     return Sequence(context->getItemFactory()->createString(XMLUni::fgZeroLenString, context), memMgr);
   }
 
-  XMLBuffer buf(1023, context->getMemoryManager());
-  Normalizer::upperCase(arg->asString(context), buf);
-  return Sequence(context->getItemFactory()->createString(buf.getRawBuffer(), context), memMgr);
+  AutoDeallocate<XMLCh> buf(UnicodeTransformer::upperCase(arg->asString(context), memMgr), memMgr);
+  return Sequence(context->getItemFactory()->createString(buf.get(), context), memMgr);
 }
