@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include <assert.h>
@@ -29,6 +27,7 @@
 #include <xqilla/items/impl/ATDoubleOrDerivedImpl.hpp>
 #include <xqilla/mapm/m_apm.h>
 #include <xqilla/exceptions/IllegalArgumentException.hpp>
+#include <xqilla/exceptions/XQillaException.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/context/ItemFactory.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
@@ -282,7 +281,24 @@ const XMLCh *Numeric::asDecimalString(int significantDigits, const StaticContext
 
 const XMLCh *Numeric::asDecimalString(const MAPM &number, int significantDigits, const StaticContext* context)
 {
-  char obuf[1024];
+  // Init a buf with an array. Most of the time 1024 digits is enough for
+  // decimals. So we only need malloc() in extreme case (huge decimal).
+  char buf[1024];
+  char *obuf = buf;
+
+  // Calculate the required buf size
+  int bufSize = significantDigits + 1;
+  if (bufSize < (number.significant_digits() + 1))
+    bufSize = (number.significant_digits() + 1);
+
+  // Malloc a larger one as needed
+  if (bufSize > sizeof(buf)) {
+    obuf = (char *)malloc(bufSize);
+    if (obuf == NULL)
+      throw XQillaException(XQillaException::RUNTIME_ERR,
+                            X("Numeric::asDecimalString(): Out of memory"));
+  }
+
   if(number.is_integer())
     number.toIntegerString(obuf);
   else {
@@ -302,7 +318,11 @@ const XMLCh *Numeric::asDecimalString(const MAPM &number, int significantDigits,
         *lastChar=0;
     }
   }
-  return context->getMemoryManager()->getPooledString(obuf);
+
+  const XMLCh* ret = context->getMemoryManager()->getPooledString(obuf);
+  if (obuf != buf) // Free "obuf" if it has been malloced
+    free(obuf);
+  return ret;
 }
 
 const XMLCh *Numeric::asDoubleString(int significantDigits, const StaticContext* context) const
@@ -396,3 +416,8 @@ int Numeric::asInt() const
 {
   return (int)asMAPM().toDouble();
 }
+
+bool Numeric::isInteger() const
+{
+  return asMAPM().is_integer();
+}
