@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: ATDurationOrDerivedImpl.cpp 734 2009-01-30 18:08:37Z gmfeinberg $
  */
 
 #include "../../config/xqilla_config.h"
@@ -28,7 +26,6 @@
 #include <xqilla/exceptions/IllegalArgumentException.hpp>
 #include <xqilla/framework/XPath2MemoryManager.hpp>
 #include <xqilla/context/DynamicContext.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/framework/XMLBuffer.hpp>
@@ -36,6 +33,7 @@
 #include <xqilla/mapm/m_apm.h>
 #include "../../utils/DateUtils.hpp"
 #include <xqilla/utils/XPath2Utils.hpp>
+#include <xqilla/context/ItemFactory.hpp>
 
 #if defined(XERCES_HAS_CPP_NAMESPACE)
 XERCES_CPP_NAMESPACE_USE
@@ -80,10 +78,10 @@ void ATDurationOrDerivedImpl::init(const DynamicContext* context)
     }
   }
 
-  if(this->isInstanceOfType (FunctionConstructor::XMLChXPath2DatatypesURI,
+  if(this->isInstanceOfType (SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
                              ATDurationOrDerived::fgDT_DAYTIMEDURATION, context)) {
     _durationType = DAY_TIME_DURATION;
-  } else if (this->isInstanceOfType (FunctionConstructor::XMLChXPath2DatatypesURI,
+  } else if (this->isInstanceOfType (SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
                                      ATDurationOrDerived::fgDT_YEARMONTHDURATION, context)) {
     _durationType = YEAR_MONTH_DURATION;
   } else {
