@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,15 +15,16 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../../config/xqilla_config.h"
 #include "FunctionRefImpl.hpp"
 #include <xqilla/ast/XQVariable.hpp>
+#include <xqilla/ast/XQFunction.hpp>
+#include <xqilla/ast/XQCastAs.hpp>
 #include <xqilla/exceptions/XPath2TypeMatchException.hpp>
 #include <xqilla/functions/FuncFactory.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 #include <xqilla/context/impl/VarStoreImpl.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/context/ContextHelpers.hpp>
@@ -33,59 +34,49 @@
 #include <xqilla/events/EventHandler.hpp>
 #include <xqilla/runtime/ClosureResult.hpp>
 
+#include <xercesc/validators/schema/SchemaSymbols.hpp>
+
 XERCES_CPP_NAMESPACE_USE;
 using namespace std;
 
-XMLCh FunctionRefImpl::argVarPrefix[] = { '#', 'a', 'r', 'g', 0 };
+static const XMLCh constructorArgName[] = { 'a', 'r', 'g', 0 };
 
 FunctionRefImpl::FunctionRefImpl(const XMLCh *prefix, const XMLCh *uri, const XMLCh *localname,
-                                 const ASTNode *instance, size_t numArgs, DynamicContext *context)
+                                 const FunctionSignature *signature, const ASTNode *instance,
+                                 const StaticAnalysis &sa, DynamicContext *context)
   : prefix_(prefix),
     uri_(uri),
     name_(localname),
-    instance_(instance),
-    varStore_(context->getMemoryManager())
-{
-  for(unsigned int i = 0; i < numArgs; ++i) {
-    args_.push_back(i);
-  }  
-}
-
-FunctionRefImpl::FunctionRefImpl(const ASTNode *instance, size_t numArgs, const StaticAnalysis &sa, DynamicContext *context)
-  : prefix_(0),
-    uri_(0),
-    name_(0),
+    signature_(signature),
+    signatureOwned_(false),
     instance_(instance),
     varStore_(context->getMemoryManager())
 {
   // Copy the variables we need into our local storage
   varStore_.cacheVariableStore(sa, context->getVariableStore());
-
-  for(unsigned int i = 0; i < numArgs; ++i) {
-    args_.push_back(i);
-  }  
 }
 
 FunctionRefImpl::FunctionRefImpl(const FunctionRefImpl *other, const Result &argument, unsigned int argNum, DynamicContext *context)
   : prefix_(other->prefix_),
     uri_(other->uri_),
     name_(other->name_),
+    signature_(new (context->getMemoryManager()) FunctionSignature(other->signature_, argNum, context->getMemoryManager())),
+    signatureOwned_(true),
     instance_(other->instance_),
-    args_(other->args_),
     varStore_(other->varStore_, context->getMemoryManager())
 {
-  vector<unsigned int>::iterator argsIt = args_.begin();
+  ArgumentSpecs::const_iterator argsIt = other->signature_->argSpecs->begin();
   for(unsigned int i = 0; i < argNum; ++i) {
     ++argsIt;
   }
 
-  XMLBuffer buf(20);
-  buf.set(FunctionRefImpl::argVarPrefix);
-  XPath2Utils::numToBuf(*argsIt, buf);
+  varStore_.setVar((*argsIt)->getURI(), (*argsIt)->getName(), argument);
+}
 
-  args_.erase(argsIt);
-
-  varStore_.setVar(0, context->getMemoryManager()->getPooledString(buf.getRawBuffer()), argument);
+FunctionRefImpl::~FunctionRefImpl()
+{
+  if(signatureOwned_)
+    const_cast<FunctionSignature*>(signature_)->release();
 }
 
 class FunctionRefScope : public VariableStore
@@ -95,17 +86,12 @@ public:
     : func_(func),
       scope_(context->getMemoryManager(), context->getVariableStore())
   {
-    XPath2MemoryManager *mm = context->getMemoryManager();
-
-    VectorOfResults::const_iterator i = args.begin();
-    vector<unsigned int>::const_iterator argsIt = func_->args_.begin();
-    for(; i != args.end(); ++i) {
-      XMLBuffer buf(20);
-      buf.set(FunctionRefImpl::argVarPrefix);
-      XPath2Utils::numToBuf(*argsIt, buf);
-      ++argsIt;
-
-      scope_.setVar(0, mm->getPooledString(buf.getRawBuffer()), *i);
+    if(func_->signature_->argSpecs) {
+      VectorOfResults::const_iterator i = args.begin();
+      ArgumentSpecs::const_iterator argsIt = func_->signature_->argSpecs->begin();
+      for(; i != args.end(); ++i, ++argsIt) {
+        scope_.setVar((*argsIt)->getURI(), (*argsIt)->getName(), *i);
+      }
     }
   }
 
@@ -135,12 +121,13 @@ Result FunctionRefImpl::execute(const VectorOfResults &args, DynamicContext *con
     XPath2Utils::numToBuf((unsigned int)args.size(), buf);
     buf.append(X(" arguments - found item of type "));
     typeToBuffer(context, buf);
-    buf.append(X(" [err:TBD]"));
+    buf.append(X(" [err:XPTY0004]"));
     XQThrow3(XPath2TypeMatchException, X("FunctionRefImpl::execute"), buf.getRawBuffer(), location);
   }
 
   FunctionRefScope scope(this, args, context);
-  return ClosureResult::create(instance_, context, &scope);
+  AutoVariableStoreReset vsReset(context, &scope);
+  return instance_->createResult(context);
 }
 
 ATQNameOrDerived::Ptr FunctionRefImpl::getName(const DynamicContext *context) const
@@ -149,6 +136,12 @@ ATQNameOrDerived::Ptr FunctionRefImpl::getName(const DynamicContext *context) co
   return context->getItemFactory()->createQName(uri_, prefix_, name_, context);
 }
 
+size_t FunctionRefImpl::getNumArgs() const
+{
+  if(signature_->argSpecs == 0) return 0;
+  return signature_->argSpecs->size();
+}
+
 FunctionRef::Ptr FunctionRefImpl::partialApply(const Result &arg, unsigned int argNum, DynamicContext *context, const LocationInfo *location) const
 {
   if(getNumArgs() < argNum) {
@@ -174,17 +167,13 @@ void FunctionRefImpl::generateEvents(EventHandler *events, const DynamicContext
 const XMLCh *FunctionRefImpl::asString(const DynamicContext *context) const
 {
   XMLBuffer buf;
-  buf.append(X("function(#"));
-  XPath2Utils::numToBuf((unsigned int)getNumArgs(), buf);
-  buf.append(')');
+  signature_->toBuffer(buf, /*typeSyntax*/false);
   return context->getMemoryManager()->getPooledString(buf.getRawBuffer());
 }
 
 void FunctionRefImpl::typeToBuffer(DynamicContext *context, XMLBuffer &buffer) const
 {
-  buffer.append(X("function("));
-  XPath2Utils::numToBuf((unsigned int)getNumArgs(), buffer);
-  buffer.append(X(")"));
+  signature_->toBuffer(buffer, /*typeSyntax*/true);
 }
 
 void *FunctionRefImpl::getInterface(const XMLCh *name) const
@@ -192,17 +181,18 @@ void *FunctionRefImpl::getInterface(const XMLCh *name) const
   return 0;
 }
 
-ASTNode *FunctionRefImpl::createInstance(const FuncFactory *factory, unsigned int numArgs, XPath2MemoryManager *mm, const LocationInfo *location)
+ASTNode *FunctionRefImpl::createInstance(const FuncFactory *factory, const FunctionSignature *signature,
+                                         XPath2MemoryManager *mm, const LocationInfo *location)
 {
   VectorOfASTNodes newArgs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
-  for(unsigned int i = 0; i < numArgs; ++i) {
-    XMLBuffer buf(20);
-    buf.set(argVarPrefix);
-    XPath2Utils::numToBuf(i, buf);
-
-    XQVariable *var = new (mm) XQVariable(0, mm->getPooledString(buf.getRawBuffer()), mm);
-    var->setLocationInfo(location);
-    newArgs.push_back(var);
+  if(signature->argSpecs) {
+    ArgumentSpecs::const_iterator argsIt = signature->argSpecs->begin();
+    for(; argsIt != signature->argSpecs->end(); ++argsIt) {
+      assert((*argsIt)->getName() != 0);
+      XQVariable *var = new (mm) XQVariable((*argsIt)->getURI(), (*argsIt)->getName(), mm);
+      var->setLocationInfo(location);
+      newArgs.push_back(var);
+    }
   }
 
   ASTNode *instance = factory->createInstance(newArgs, mm);
@@ -210,22 +200,67 @@ ASTNode *FunctionRefImpl::createInstance(const FuncFactory *factory, unsigned in
   return instance;
 }
 
-ASTNode *FunctionRefImpl::createInstance(const XMLCh *uri, const XMLCh *name, unsigned int numArgs, StaticContext *context, const LocationInfo *location)
+ASTNode *FunctionRefImpl::createInstance(const XMLCh *uri, const XMLCh *name, unsigned int numArgs,
+                                         StaticContext *context, const LocationInfo *location,
+                                         FunctionSignature *&signature)
 {
   XPath2MemoryManager *mm = context->getMemoryManager();
 
   VectorOfASTNodes newArgs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
   for(unsigned int i = 0; i < numArgs; ++i) {
-    XMLBuffer buf(20);
-    buf.set(argVarPrefix);
-    XPath2Utils::numToBuf(i, buf);
-
-    XQVariable *var = new (mm) XQVariable(0, mm->getPooledString(buf.getRawBuffer()), mm);
-    var->setLocationInfo(location);
-    newArgs.push_back(var);
+    newArgs.push_back(0); // Dummy argument
   }
 
-  ASTNode *instance = context->lookUpFunction(uri, name, newArgs);
-  if(instance) instance->setLocationInfo(location);
-  return instance;
+  ASTNode *result = context->lookUpFunction(uri, name, newArgs, location);
+  if(!result) return 0;
+
+  switch(result->getType()) {
+  case ASTNode::FUNCTION:
+  case ASTNode::USER_FUNCTION: {
+    XQFunction *function = (XQFunction*)result;
+    function->parseSignature(context);
+    signature = new (mm) FunctionSignature(function->getSignature(), mm);
+
+    if(signature->argSpecs) {
+      // Fill in the arguments with XQVariable objects
+      ArgumentSpecs::const_iterator argsIt = signature->argSpecs->begin();
+      VectorOfASTNodes &args = const_cast<VectorOfASTNodes&>(function->getArguments());
+      for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i, ++argsIt) {
+        (*i) = new (mm) XQVariable((*argsIt)->getURI(), (*argsIt)->getName(), mm);
+        (*i)->setLocationInfo(location);
+      }
+    }
+
+    break;
+  }
+  case ASTNode::CAST_AS: {
+    XQCastAs *cast = (XQCastAs*)result;
+
+    // Fill in the argument with an XQVariable object
+    XQVariable *var = new (mm) XQVariable(0, constructorArgName, mm);
+    var->setLocationInfo(location);
+    cast->setExpression(var);
+
+    // Create a signature for the constructor function
+    SequenceType *argType = new (mm) SequenceType(SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
+                                                  AnyAtomicType::fgDT_ANYATOMICTYPE,
+                                                  SequenceType::QUESTION_MARK, mm);
+    argType->setLocationInfo(location);
+
+    ArgumentSpec *arg = new (mm) ArgumentSpec(constructorArgName, argType, mm);
+    arg->setLocationInfo(location);
+
+    ArgumentSpecs *args = new (mm) ArgumentSpecs(XQillaAllocator<ArgumentSpec*>(mm));
+    args->push_back(arg);
+
+    signature = new (mm) FunctionSignature(args, cast->getSequenceType(), mm);
+    signature->staticResolution(context);
+    break;
+  }
+  default:
+    assert(false);
+    break;
+  }
+
+  return result;
 }
