@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2007
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2007
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef _XQILLAFUNCTIONREFIMPL_HPP
@@ -34,15 +32,15 @@ class XQILLA_API FunctionRefImpl : public FunctionRef
 public:
   typedef RefCountPointer<const FunctionRefImpl> Ptr;
 
-  FunctionRefImpl(const XMLCh *prefix, const XMLCh *uri, const XMLCh *localname,
-                  const ASTNode *instance, size_t numArgs, DynamicContext *context);
   /** Initialises closure from the StaticAnalysis of variables that are needed */
-  FunctionRefImpl(const ASTNode *instance, size_t numArgs, const StaticAnalysis &sa, DynamicContext *context);
-  /** Partially evaluates the given FunctionRefImpl with the given argument */
-  FunctionRefImpl(const FunctionRefImpl *other, const Result &argument, unsigned int argNum, DynamicContext *context);
+  FunctionRefImpl(const XMLCh *prefix, const XMLCh *uri, const XMLCh *localname,
+                  const FunctionSignature *signature, const ASTNode *instance,
+                  const StaticAnalysis &sa, DynamicContext *context);
+  ~FunctionRefImpl();
 
   virtual ATQNameOrDerived::Ptr getName(const DynamicContext *context) const;
-  virtual size_t getNumArgs() const { return args_.size(); }
+  virtual size_t getNumArgs() const;
+  virtual const FunctionSignature *getSignature() const { return signature_; }
   virtual Result execute(const VectorOfResults &args, DynamicContext *context, const LocationInfo *location) const;
   virtual FunctionRef::Ptr partialApply(const Result &arg, unsigned int argNum, DynamicContext *context, const LocationInfo *location) const;
 
@@ -55,15 +53,19 @@ public:
 
   virtual void *getInterface(const XMLCh *name) const;
 
-  static ASTNode *createInstance(const FuncFactory *factory, unsigned int numArgs, XPath2MemoryManager *mm, const LocationInfo *location);
-  static ASTNode *createInstance(const XMLCh *uri, const XMLCh *name, unsigned int numArgs, StaticContext *context, const LocationInfo *location);
-
-  static XMLCh argVarPrefix[];
+  static ASTNode *createInstance(const FuncFactory *factory, const FunctionSignature *signature, XPath2MemoryManager *mm,
+                                 const LocationInfo *location);
+  static ASTNode *createInstance(const XMLCh *uri, const XMLCh *name, unsigned int numArgs, StaticContext *context,
+                                 const LocationInfo *location, FunctionSignature *&signature);
 
 protected:
+  /** Partially evaluates the given FunctionRefImpl with the given argument */
+  FunctionRefImpl(const FunctionRefImpl *other, const Result &argument, unsigned int argNum, DynamicContext *context);
+
   const XMLCh *prefix_, *uri_, *name_;
+  const FunctionSignature *signature_;
+  bool signatureOwned_;
   const ASTNode *instance_;
-  std::vector<unsigned int> args_;
   VarStoreImpl varStore_;
 
   friend class FunctionRefScope;
