@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #ifndef XQLEXER_H
@@ -91,6 +89,7 @@ typedef struct yyltype
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/parser/QName.hpp>
 #include <xqilla/functions/XQUserFunction.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 #include <xqilla/ast/XQTypeswitch.hpp>
 #include <xqilla/ast/XQPredicate.hpp>
 #include <xqilla/fulltext/FTSelection.hpp>
@@ -107,8 +106,9 @@ typedef union {
   ASTNode* astNode;
   XQSequence *parenExpr;
   XQUserFunction* functDecl;
-  XQUserFunction::ArgumentSpec* argSpec;
-  XQUserFunction::ArgumentSpecs* argSpecs;
+  ArgumentSpec* argSpec;
+  ArgumentSpecs* argSpecs;
+  FunctionSignature *signature;
   XQUserFunction::Mode* mode;
   XQUserFunction::ModeList* modeList;
   XQGlobalVariable *globalVar;
@@ -179,13 +179,23 @@ public:
   bool isXSLT() const { return (m_language & XQilla::XSLT2) != 0; }
   bool isFullText() const { return (m_language & XQilla::FULLTEXT) != 0; }
   bool isUpdate() const { return (m_language & XQilla::UPDATE) != 0; }
+  bool isVersion3() const { return (m_language & XQilla::VERSION3) != 0; }
   bool isExtensions() const { return (m_language & XQilla::EXTENSIONS) != 0; }
 
+  void setVersion3(bool value)
+  {
+    m_language = (XQilla::Language)((m_language & ~(XQilla::VERSION3)) |
+                                    (value ? XQilla::VERSION3 : 0));
+  }
+
   void setGenerateErrorException(bool bEnable) { m_bGenerateErrorException=bEnable; }
 
   virtual void undoLessThan() = 0;
   virtual XERCES_CPP_NAMESPACE_QUALIFIER DOMXPathNSResolver *getNSResolver() const { return 0; }
 
+  const XMLCh *getFile() const { return m_szQueryFile; }
+  XPath2MemoryManager *getMemoryManager() const { return mm_; }
+
 protected:
   Lexer(XPath2MemoryManager *mm, XQilla::Language lang, const XMLCh *queryFile, int line, int column)
     : mm_(mm),
@@ -226,6 +236,7 @@ public:
   };
 
   XQLexer(XPath2MemoryManager* memMgr, const XMLCh *queryFile, const XMLCh* query, XQilla::Language lang);
+  XQLexer(XPath2MemoryManager* memMgr, int firstToken, const XMLCh *queryFile, int line, int column, const XMLCh *query);
   XQLexer(XPath2MemoryManager* memMgr, const XMLCh *queryFile, int line, int column, const XMLCh* query,
           unsigned int length, const std::vector<ValueOffset> &offsets, XQilla::Language lang, StartMode mode = MODE_NORMAL);
   ~XQLexer();
@@ -279,22 +290,44 @@ protected:
   int m_nOpenComments;
 };
 
-class XQParserArgs
+class XQILLA_API XQParserArgs
 {
 public:
-  XQParserArgs()
-    : _lexer(0), _context(0), _query(0), _flags(32), _namespaceDecls(13) {}
+  XQParserArgs(Lexer *lexer, XQQuery *query)
+    : _lexer(lexer),
+      _context((DynamicContext*)query->getStaticContext()),
+      _query(query),
+      _function(0),
+      _moduleName(0),
+      _flags(32),
+      _namespaceDecls(13)
+  {
+  }
+
+  XQParserArgs(Lexer *lexer)
+    : _lexer(lexer),
+      _context(0),
+      _query(0),
+      _function(0),
+      _moduleName(0),
+      _flags(32),
+      _namespaceDecls(13)
+  {
+  }
 
   Lexer* _lexer;
   DynamicContext* _context;
   XQQuery* _query;
+  XQUserFunction *_function;
+  FunctionSignature *_signature;
+  const XMLCh *_moduleName;
   XERCES_CPP_NAMESPACE_QUALIFIER BitSet _flags;
   XERCES_CPP_NAMESPACE_QUALIFIER RefHashTableOf<XMLCh> _namespaceDecls;
 };
 
 namespace XQParser {
-  extern int yyparse(void *);
-  extern int yydebug;
+  extern XQILLA_API int yyparse(void *);
+  extern XQILLA_API int yydebug;
 }
 
 
