@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -39,8 +37,11 @@
 #include <xqilla/ast/XQAtomize.hpp>
 #include <xqilla/context/ContextHelpers.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
+#include <xqilla/runtime/Sequence.hpp>
 
-/*static*/ const XMLCh GeneralComp::name[]={ XERCES_CPP_NAMESPACE_QUALIFIER chLatin_c, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_o, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_m, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_p, XERCES_CPP_NAMESPACE_QUALIFIER chNull };
+XERCES_CPP_NAMESPACE_USE;
+
+/*static*/ const XMLCh GeneralComp::name[]={ 'G', 'e', 'n', 'e', 'r', 'a', 'l', 'C', 'o', 'm', 'p', 'a', 'r', 'i', 's', 'o', 'n', 0 };
 
 GeneralComp::GeneralComp(ComparisonOperation operation, const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
   : XQOperator(name, args, memMgr),
@@ -160,61 +161,44 @@ ASTNode *GeneralComp::staticTypingImpl(StaticContext *context)
   return this;
 }
 
-class GeneralCompResult : public ResultImpl
+BoolResult GeneralComp::boolResult(DynamicContext* context) const
 {
-public:
-  GeneralCompResult(const GeneralComp *op)
-    : ResultImpl(op),
-      _op(op)
-  {
-  }
+  // Atomization is applied to each operand of a general comparison.
+  Result arg1 = getArgument(0)->createResult(context);
+  Result arg2 = getArgument(1)->createResult(context);
 
-  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context)
-  {
-    // Atomization is applied to each operand of a general comparison.
-    Result arg1 = _op->getArgument(0)->createResult(context);
-    Result arg2 = _op->getArgument(1)->createResult(context);
+  // The result of the comparison is true if and only if there is a pair of atomic values, 
+  // one belonging to the result of atomization of the first operand and the other belonging 
+  // to the result of atomization of the second operand, that have the required magnitude relationship.
+  // Otherwise the result of the general comparison is false.
 
-    // The result of the comparison is true if and only if there is a pair of atomic values, 
-    // one belonging to the result of atomization of the first operand and the other belonging 
-    // to the result of atomization of the second operand, that have the required magnitude relationship.
-    // Otherwise the result of the general comparison is false.
-
-    AnyAtomicType::Ptr item1 = (const AnyAtomicType::Ptr)arg1->next(context);
-    if(item1 != NULLRCP) {
-      // The first time we loop over arg2, we store it in a sequence
-      AnyAtomicType::Ptr item2;
-      Sequence arg2_sequence(context->getMemoryManager());
-      while((item2 = (const AnyAtomicType::Ptr)arg2->next(context)) != NULLRCP) {
-        if(GeneralComp::compare(_op->getOperation(), item1, item2, _op->getCollation(), context, _op->getXPath1CompatibilityMode(), this)) {
-          tail = 0;
-          return context->getItemFactory()->createBoolean(true, context);
-        }
-        arg2_sequence.addItem(item2);
+  AnyAtomicType::Ptr item1 = (const AnyAtomicType::Ptr)arg1->next(context);
+  if(item1 != NULLRCP) {
+    // The first time we loop over arg2, we store it in a sequence
+    AnyAtomicType::Ptr item2;
+    Sequence arg2_sequence(context->getMemoryManager());
+    while((item2 = (const AnyAtomicType::Ptr)arg2->next(context)) != NULLRCP) {
+      if(GeneralComp::compare(getOperation(), item1, item2, getCollation(), context, getXPath1CompatibilityMode(), this)) {
+        return true;
       }
+      arg2_sequence.addItem(item2);
+    }
 
-      // The second and subsequent times, we iterate over the sequence
-      Sequence::iterator itSecond;
-      while((item1 = (const AnyAtomicType::Ptr)arg1->next(context)) != NULLRCP) {
-        for(itSecond = arg2_sequence.begin(); itSecond != arg2_sequence.end(); ++itSecond) {
-          if(GeneralComp::compare(_op->getOperation(), item1, (const AnyAtomicType::Ptr)*itSecond, _op->getCollation(), context,
-                     _op->getXPath1CompatibilityMode(), this)) {
-            tail = 0;
-            return context->getItemFactory()->createBoolean(true, context);
-          }
+    // The second and subsequent times, we iterate over the sequence
+    Sequence::iterator itSecond;
+    while((item1 = (const AnyAtomicType::Ptr)arg1->next(context)) != NULLRCP) {
+      for(itSecond = arg2_sequence.begin(); itSecond != arg2_sequence.end(); ++itSecond) {
+        if(GeneralComp::compare(getOperation(), item1, (const AnyAtomicType::Ptr)*itSecond, getCollation(), context,
+                                getXPath1CompatibilityMode(), this)) {
+          return true;
         }
       }
     }
-    tail = 0;
-    return context->getItemFactory()->createBoolean(false, context);
   }
-
-private:
-  const GeneralComp *_op;
-};
+  return false;
+}
 
 Result GeneralComp::createResult(DynamicContext* context, int flags) const
 {
-  return new GeneralCompResult(this);
+  return (Item::Ptr)context->getItemFactory()->createBoolean(boolResult(context), context);
 }
-
