@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -29,6 +27,7 @@
 #include <xqilla/ast/XQTreatAs.hpp>
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
+#include <xqilla/runtime/Sequence.hpp>
 
 XERCES_CPP_NAMESPACE_USE;
 
@@ -92,41 +91,27 @@ ASTNode *Intersect::staticTypingImpl(StaticContext *context)
   return this;
 }
 
-class IntersectResult : public ResultImpl
-{
-public:
-  IntersectResult(const Intersect *ast) : ResultImpl(ast), ast_(ast) {}
-
-  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context)
-  {
-    Sequence param1 = ast_->getArgument(0)->createResult(context)->toSequence(context);
-    Sequence param2 = ast_->getArgument(1)->createResult(context)->toSequence(context);
-
-    XPath2MemoryManager* memMgr = context->getMemoryManager();
-    Sequence result(param1.getLength(),memMgr);
-
-    Sequence::const_iterator p1It = param1.begin();
-    Sequence::const_iterator p2It;
-    Sequence::const_iterator end1 = param1.end();
-    Sequence::const_iterator end2 = param2.end();
-
-    for(;p1It != end1; ++p1It) {
-      for(p2It = param2.begin();p2It != end2; ++p2It) {
-        if(((Node*)p1It->get())->equals((Node*)p2It->get())) {
-          result.addItem(*p1It);
-        }
-      }
-    }
-
-    tail = result;
-    return 0;
-  }
-
-private:
-  const Intersect *ast_;
-};
-
 Result Intersect::createResult(DynamicContext* context, int flags) const
 {
-  return new IntersectResult(this);
+  // TBD Improve this implementation - jpcs
+  Sequence param1 = getArgument(0)->createResult(context)->toSequence(context);
+  Sequence param2 = getArgument(1)->createResult(context)->toSequence(context);
+
+  XPath2MemoryManager* memMgr = context->getMemoryManager();
+  Sequence result(param1.getLength(),memMgr);
+
+  Sequence::const_iterator p1It = param1.begin();
+  Sequence::const_iterator p2It;
+  Sequence::const_iterator end1 = param1.end();
+  Sequence::const_iterator end2 = param2.end();
+
+  for(;p1It != end1; ++p1It) {
+    for(p2It = param2.begin();p2It != end2; ++p2It) {
+      if(((Node*)p1It->get())->equals((Node*)p2It->get())) {
+        result.addItem(*p1It);
+      }
+    }
+  }
+
+  return result;
 }
