@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -32,7 +30,7 @@
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
 
-/*static*/ const XMLCh NodeComparison::name[]={ XERCES_CPP_NAMESPACE_QUALIFIER chLatin_i, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chNull };
+/*static*/ const XMLCh NodeComparison::name[]={ XERCES_CPP_NAMESPACE_QUALIFIER chLatin_I, XERCES_CPP_NAMESPACE_QUALIFIER chLatin_s, XERCES_CPP_NAMESPACE_QUALIFIER chNull };
 
 NodeComparison::NodeComparison(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
   : XQOperator(name, args, memMgr)
@@ -76,24 +74,17 @@ ASTNode *NodeComparison::staticTypingImpl(StaticContext *context)
   return this;
 }
 
+BoolResult NodeComparison::boolResult(DynamicContext* context) const
+{
+  Item::Ptr arg1 = getArgument(0)->createResult(context)->next(context);
+  if(arg1.isNull()) return BoolResult::Null;
+  Item::Ptr arg2 = getArgument(1)->createResult(context)->next(context);
+  if(arg2.isNull()) return BoolResult::Null;
+  
+	return ((Node*)arg1.get())->equals((Node*)arg2.get());
+}
+
 Result NodeComparison::createResult(DynamicContext* context, int flags) const
 {
-  return new NodeComparisonResult(this);
+  return (Item::Ptr)context->getItemFactory()->createBoolean(boolResult(context), context);
 }
-
-NodeComparison::NodeComparisonResult::NodeComparisonResult(const NodeComparison *op)
-  : SingleResult(op),
-    _op(op)
-{
-}
-
-Item::Ptr NodeComparison::NodeComparisonResult::getSingleResult(DynamicContext *context) const
-{
-  Item::Ptr arg1 = _op->getArgument(0)->createResult(context)->next(context);
-  if(arg1.isNull()) return 0;
-  Item::Ptr arg2 = _op->getArgument(1)->createResult(context)->next(context);
-  if(arg2.isNull()) return 0;
-  
-	return context->getItemFactory()->createBoolean(((Node*)arg1.get())->equals((Node*)arg2.get()), context);
-}
-
