@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: ASTVisitor.cpp 531 2008-04-10 23:23:07Z jpcs $
  */
 
 #include <xqilla/optimizer/ASTCopier.hpp>
@@ -50,13 +48,16 @@
 #include <xqilla/operators/Or.hpp>
 #include <xqilla/operators/OrderComparison.hpp>
 #include <xqilla/operators/Plus.hpp>
-#include <xqilla/operators/Range.hpp>
 #include <xqilla/operators/UnaryMinus.hpp>
 #include <xqilla/operators/Union.hpp>
 
-#include <xqilla/functions/FunctionCodepointEqual.hpp>
 #include <xqilla/update/FunctionPut.hpp>
 
+#include "xqilla/functions/FunctionMatches.hpp"
+#include "xqilla/functions/FunctionReplace.hpp"
+#include "xqilla/functions/FunctionTokenize.hpp"
+#include <xqilla/functions/FunctionSignature.hpp>
+
 XERCES_CPP_NAMESPACE_USE;
 
 ASTCopier::ASTCopier()
@@ -135,6 +136,11 @@ TupleNode *ASTCopier::optimizeUnknownTupleNode(TupleNode *item)
   return ASTVisitor::optimizeUnknownTupleNode(item);
 }
 
+FTSelection *ASTCopier::optimizeUnknownFTSelection(FTSelection *selection)
+{
+  return ASTVisitor::optimizeUnknownFTSelection(selection);
+}
+
 #define COPY_IMPL() { \
   result->setLocationInfo(item); \
   const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(item->getStaticAnalysis()); \
@@ -257,12 +263,12 @@ COPY_XQ5(QNameLiteral, TypeURI, TypeName, URI, Prefix, Localname)
 COPY_XQ4(NumericLiteral, TypeURI, TypeName, Value, PrimitiveType)
 COPY_XQ1(Sequence, Children)
 COPY_XQ2(Step, Axis, NodeTest)
-COPY_XQ2(Variable, URI, Name)
+COPY_XQ4(Variable, Prefix, URI, Name, Global)
 COPY_XQ3(If, Test, WhenTrue, WhenFalse)
-COPY_XQ2(InstanceOf, Expression, SequenceType) // SequenceType
 COPY_XQ4(CastableAs, Expression, SequenceType, IsPrimitive, TypeIndex)
 COPY_XQ4(CastAs, Expression, SequenceType, IsPrimitive, TypeIndex)
-COPY_XQ8(TreatAs, Expression, SequenceType, ErrorCode, DoTypeCheck, DoCardinalityCheck, FuncConvert, TreatType, IsExact)
+COPY_XQ7(TreatAs, Expression, SequenceType, ErrorCode, DoTypeCheck, DoCardinalityCheck, TreatType, IsExact)
+COPY_XQ4(FunctionCoercion, Expression, SequenceType, FuncConvert, TreatType)
 COPY_XQ0(ContextItem)
 COPY_XQ2(Return, Parent, Expression)
 COPY_XQ3(Quantified, QuantifierType, Parent, Expression)
@@ -270,7 +276,6 @@ COPY_XQ2(Validate, Expression, Mode)
 COPY_XQ2(OrderingChange, OrderingValue, Expr)
 COPY_XQ2(Atomize, Expression, DoPSVI)
 COPY_XQ1(EffectiveBooleanValue, Expression)
-COPY2(XPath1CompatConvertFunctionArg, Expression, SequenceType) // SequenceType
 COPY_XQ5(PromoteUntyped, Expression, TypeURI, TypeName, IsPrimitive, TypeIndex)
 COPY_XQ4(PromoteNumeric, Expression, TypeURI, TypeName, TypeIndex)
 COPY_XQ3(PromoteAnyURI, Expression, TypeURI, TypeName)
@@ -284,8 +289,8 @@ COPY_XQ2(FunctionConversion, Expression, SequenceType)
 COPY_XQ5(AnalyzeString, Expression, Regex, Flags, Match, NonMatch)
 COPY_XQ2(CopyOf, Expression, CopyNamespaces)
 COPY_XQ4(Copy, Expression, Children, CopyNamespaces, InheritNamespaces)
+COPY_XQ2(FunctionRef, QName, NumArgs)
 COPY1(ASTDebugHook, Expression)
-COPY_XQ4(FunctionRef, URI, Name, NumArgs, Instance)
 COPY1(UDelete, Expression)
 COPY2(URename, Target, Name)
 COPY2(UReplace, Target, Expression)
@@ -303,7 +308,10 @@ COPY_FULL3(UserFunction, XQUserFunctionInstance, FunctionDefinition, Arguments,
 
 ASTNode *ASTCopier::optimizeInlineFunction(XQInlineFunction *item)
 {
-  XQInlineFunction *result = new (mm_) XQInlineFunction(item->getUserFunction(), item->getNumArgs(), item->getInstance(), mm_);
+  XQInlineFunction *result = new (mm_) XQInlineFunction(item->getUserFunction(), item->getPrefix(),
+                                                        item->getURI(), item->getName(), item->getNumArgs(),
+                                                        new (mm_) FunctionSignature(item->getSignature(), mm_),
+                                                        item->getInstance(), mm_);
   ASTVisitor::optimizeInlineFunction(result);
 
   if(result->getUserFunction()) {
@@ -323,15 +331,18 @@ ASTNode *ASTCopier::optimizeFunction(XQFunction *item)
   if(uri == XQFunction::XMLChFunctionURI) {
     if(name == FunctionPut::name) {
       result = new (mm_) FunctionPut(((FunctionPut*)item)->getBaseURI(), item->getArguments(), mm_);
-    }
-    else if(name == FunctionCodepointEqual::name) {
-      result = new (mm_) FunctionCodepointEqual(((FunctionCodepointEqual*)item)->getCollation(), item->getArguments(), mm_);
+    } else if (name == FunctionMatches::name || name == FunctionReplace::name || name == FunctionTokenize::name) {
+      RegExpFunction* newFunc = (RegExpFunction*)context_->lookUpFunction(item->getFunctionURI(), item->getFunctionName(), item->getArguments());
+      newFunc->copyRegExp((RegExpFunction*) item, mm_);
+      result = newFunc;
     }
   }
 
   if(result == 0)
     result = (XQFunction*)context_->lookUpFunction(item->getFunctionURI(), item->getFunctionName(), item->getArguments());
 
+  result->setSignature(new (mm_) FunctionSignature(item->getSignature(), mm_));
+
   ASTVisitor::optimizeFunction(result);
   COPY_IMPL();
 }
@@ -373,7 +384,6 @@ ASTNode *ASTCopier::optimizeOperator(XQOperator *item)
   OP_CHECK(NotEquals);
   OP_CHECK(Or);
   OP_CHECK(Plus);
-  OP_CHECK(Range);
   OP_CHECK(Union);
 
   ASTVisitor::optimizeOperator(result);
@@ -382,7 +392,8 @@ ASTNode *ASTCopier::optimizeOperator(XQOperator *item)
 
 static XQTypeswitch::Case *copyCase(const XQTypeswitch::Case *in, XPath2MemoryManager *mm)
 {
-  XQTypeswitch::Case *result = new (mm) XQTypeswitch::Case(in->getQName(), in->getURI(), in->getName(), in->getSequenceType(), in->getExpression());
+  XQTypeswitch::Case *result = new (mm) XQTypeswitch::Case(in->getQName(), in->getURI(), in->getName(), in->getSequenceType(),
+                                                           in->getTreatType(), in->getIsExact(), in->getExpression());
   result->setLocationInfo(in);
   return result;
 }
@@ -406,7 +417,7 @@ ASTNode *ASTCopier::optimizeFunctionCall(XQFunctionCall *item)
   VectorOfASTNodes *newArgs = new (mm_) VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm_));
   *newArgs = *item->getArguments();
 
-  XQFunctionCall *result = new (mm_) XQFunctionCall(item->getQName(), newArgs, mm_);
+  XQFunctionCall *result = new (mm_) XQFunctionCall(item->getPrefix(), item->getURI(), item->getName(), newArgs, mm_);
 
   ASTVisitor::optimizeFunctionCall(result);
   COPY_IMPL();
@@ -536,6 +547,21 @@ ASTNode *ASTCopier::optimizeFunctionDeref(XQFunctionDeref *item)
   COPY_IMPL();
 }
 
+ASTNode *ASTCopier::optimizePartialApply(XQPartialApply *item)
+{
+  VectorOfASTNodes *newArgs = 0;
+  VectorOfASTNodes *args = const_cast<VectorOfASTNodes *>(item->getArguments());
+  if(args) {
+    newArgs = new (mm_) VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm_));
+    *newArgs = *args;
+  }
+
+  XQPartialApply *result = new (mm_) XQPartialApply(item->getExpression(), newArgs, mm_);
+
+  ASTVisitor::optimizePartialApply(item);
+  COPY_IMPL();
+}
+
 ASTNode *ASTCopier::optimizeUTransform(UTransform *item)
 {
   VectorOfCopyBinding *newBindings = new (mm_) VectorOfCopyBinding(XQillaAllocator<CopyBinding*>(mm_));
@@ -595,6 +621,16 @@ TupleNode *ASTCopier::optimizeWhereTuple(WhereTuple *item)
   return result;
 }
 
+TupleNode *ASTCopier::optimizeCountTuple(CountTuple *item)
+{
+  CountTuple *result = new (mm_) CountTuple(item->getParent(), item->getVarURI(), item->getVarName(), mm_);
+  ASTVisitor::optimizeCountTuple(result);
+  result->setLocationInfo(item);
+  result->setMin(item->getMin());
+  result->setMax(item->getMax());
+  return result;
+}
+
 TupleNode *ASTCopier::optimizeOrderByTuple(OrderByTuple *item)
 {
   OrderByTuple *result = new (mm_) OrderByTuple(item->getParent(), item->getExpression(), item->getModifiers(),
@@ -616,3 +652,124 @@ TupleNode *ASTCopier::optimizeTupleDebugHook(TupleDebugHook *item)
   result->setMax(item->getMax());
   return result;
 }
+
+FTSelection *ASTCopier::optimizeFTWords(FTWords *selection)
+{
+  FTWords *result = new (mm_) FTWords(selection->getExpr(), selection->getOption(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTWords(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTWord(FTWord *selection)
+{
+  FTWord *result = new (mm_) FTWord(selection->getQueryString(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTWord(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTOr(FTOr *selection)
+{
+  FTOr *result = new (mm_) FTOr(mm_);
+  result->setLocationInfo(selection);
+  const_cast<VectorOfFTSelections&>(result->getArguments()) = selection->getArguments();
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTOr(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTAnd(FTAnd *selection)
+{
+  FTAnd *result = new (mm_) FTAnd(mm_);
+  result->setLocationInfo(selection);
+  const_cast<VectorOfFTSelections&>(result->getArguments()) = selection->getArguments();
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTAnd(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTMildnot(FTMildnot *selection)
+{
+  FTMildnot *result = new (mm_) FTMildnot(selection->getLeft(), selection->getRight(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTMildnot(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTUnaryNot(FTUnaryNot *selection)
+{
+  FTUnaryNot *result = new (mm_) FTUnaryNot(selection->getArgument(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTUnaryNot(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTOrder(FTOrder *selection)
+{
+  FTOrder *result = new (mm_) FTOrder(selection->getArgument(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTOrder(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTDistance(FTDistance *selection)
+{
+  FTDistance *result = new (mm_) FTDistance(selection->getRange(), selection->getUnit(), selection->getArgument(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTDistance(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTDistanceLiteral(FTDistanceLiteral *selection)
+{
+  FTDistanceLiteral *result = new (mm_) FTDistanceLiteral(selection->getArgument(), selection->getType(), selection->getDistance(),
+                                                          selection->getDistance2(), selection->getUnit(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTDistanceLiteral(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTScope(FTScope *selection)
+{
+  FTScope *result = new (mm_) FTScope(selection->getArgument(), selection->getType(), selection->getUnit(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTScope(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTContent(FTContent *selection)
+{
+  FTContent *result = new (mm_) FTContent(selection->getArgument(), selection->getType(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTContent(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTWindow(FTWindow *selection)
+{
+  FTWindow *result = new (mm_) FTWindow(selection->getArgument(), selection->getExpr(), selection->getUnit(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTWindow(selection);
+  return result;
+}
+
+FTSelection *ASTCopier::optimizeFTWindowLiteral(FTWindowLiteral *selection)
+{
+  FTWindowLiteral *result = new (mm_) FTWindowLiteral(selection->getArgument(), selection->getDistance(), selection->getUnit(), mm_);
+  result->setLocationInfo(selection);
+  const_cast<StaticAnalysis&>(result->getStaticAnalysis()).copy(selection->getStaticAnalysis());
+  ASTVisitor::optimizeFTWindowLiteral(selection);
+  return result;
+}
+
