@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: ASTVisitor.cpp 680 2008-11-24 16:02:25Z jpcs $
  */
 
 #include <xqilla/optimizer/ASTVisitor.hpp>
@@ -25,9 +23,11 @@ using namespace std;
 
 void ASTVisitor::optimize(XQQuery *query)
 {
-  ImportedModules &modules = const_cast<ImportedModules&>(query->getImportedModules());
-  for(ImportedModules::iterator it2 = modules.begin(); it2 != modules.end(); ++it2) {
-    optimize(*it2);
+  if(query->isModuleCacheOwned()) {
+    ImportedModules &modules = const_cast<ImportedModules&>(query->getModuleCache()->ordered_);
+    for(ImportedModules::iterator it2 = modules.begin(); it2 != modules.end(); ++it2) {
+      optimize(*it2);
+    }
   }
 
   vector<XQGlobalVariable*> newVars;
@@ -100,14 +100,14 @@ ASTNode *ASTVisitor::optimize(ASTNode *item)
     return optimizeStep((XQStep *)item);
   case ASTNode::IF:
     return optimizeIf((XQIf *)item);
-  case ASTNode::INSTANCE_OF:
-    return optimizeInstanceOf((XQInstanceOf *)item);
   case ASTNode::CASTABLE_AS:
     return optimizeCastableAs((XQCastableAs *)item);
   case ASTNode::CAST_AS:
     return optimizeCastAs((XQCastAs *)item);
   case ASTNode::TREAT_AS:
     return optimizeTreatAs((XQTreatAs *)item);
+  case ASTNode::FUNCTION_COERCION:
+    return optimizeFunctionCoercion((XQFunctionCoercion *)item);
   case ASTNode::OPERATOR:
     return optimizeOperator((XQOperator *)item);
   case ASTNode::CONTEXT_ITEM:
@@ -134,8 +134,6 @@ ASTNode *ASTVisitor::optimize(ASTNode *item)
     return optimizeEffectiveBooleanValue((XQEffectiveBooleanValue *)item);
   case ASTNode::MAP:
     return optimizeMap((XQMap *)item);
-  case ASTNode::XPATH1_CONVERT:
-    return optimizeXPath1CompatConvertFunctionArg((XPath1CompatConvertFunctionArg *)item);
   case ASTNode::PROMOTE_UNTYPED:
     return optimizePromoteUntyped((XQPromoteUntyped *)item);
   case ASTNode::PROMOTE_NUMERIC:
@@ -200,6 +198,8 @@ ASTNode *ASTVisitor::optimize(ASTNode *item)
     return optimizeFunctionRef((XQFunctionRef *)item);
   case ASTNode::FUNCTION_DEREF:
     return optimizeFunctionDeref((XQFunctionDeref *)item);
+  case ASTNode::PARTIAL_APPLY:
+    return optimizePartialApply((XQPartialApply *)item);
   }
   return optimizeUnknown(item);
 }
@@ -274,12 +274,6 @@ ASTNode *ASTVisitor::optimizeIf(XQIf *item)
   return item;
 }
 
-ASTNode *ASTVisitor::optimizeInstanceOf(XQInstanceOf *item)
-{
-  item->setExpression(optimize(const_cast<ASTNode *>(item->getExpression())));
-  return item;
-}
-
 ASTNode *ASTVisitor::optimizeCastableAs(XQCastableAs *item)
 {
   item->setExpression(optimize(const_cast<ASTNode *>(item->getExpression())));
@@ -293,6 +287,12 @@ ASTNode *ASTVisitor::optimizeCastAs(XQCastAs *item)
 }
 
 ASTNode *ASTVisitor::optimizeTreatAs(XQTreatAs *item)
+{
+  item->setExpression(optimize(item->getExpression()));
+  return item;
+}
+
+ASTNode *ASTVisitor::optimizeFunctionCoercion(XQFunctionCoercion *item)
 {
   item->setExpression(optimize(item->getExpression()));
   if(item->getFuncConvert())
@@ -445,12 +445,6 @@ ASTNode *ASTVisitor::optimizeMap(XQMap *item)
   return item;
 }
 
-ASTNode *ASTVisitor::optimizeXPath1CompatConvertFunctionArg(XPath1CompatConvertFunctionArg *item)
-{
-  item->setExpression(optimize(const_cast<ASTNode *>(item->getExpression())));
-  return item;
-}
-
 ASTNode *ASTVisitor::optimizePromoteUntyped(XQPromoteUntyped *item)
 {
   item->setExpression(optimize(const_cast<ASTNode *>(item->getExpression())));
@@ -566,7 +560,6 @@ ASTNode *ASTVisitor::optimizeInlineFunction(XQInlineFunction *item)
 
 ASTNode *ASTVisitor::optimizeFunctionRef(XQFunctionRef *item)
 {
-  item->setInstance(optimize(item->getInstance()));
   return item;
 }
 
@@ -584,6 +577,21 @@ ASTNode *ASTVisitor::optimizeFunctionDeref(XQFunctionDeref *item)
   return item;
 }
 
+ASTNode *ASTVisitor::optimizePartialApply(XQPartialApply *item)
+{
+  item->setExpression(optimize(item->getExpression()));
+
+  VectorOfASTNodes *args = const_cast<VectorOfASTNodes*>(item->getArguments());
+  if(args) {
+    for(VectorOfASTNodes::iterator i = args->begin(); i != args->end(); ++i) {
+      if(*i != 0)
+        *i = optimize(*i);
+    }
+  }
+
+  return item;
+}
+
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
@@ -672,14 +680,141 @@ ASTNode *ASTVisitor::optimizeUApplyUpdates(UApplyUpdates *item)
 
 ASTNode *ASTVisitor::optimizeFTContains(FTContains *item)
 {
-  // TBD implement optimization of FTSelection objects - jpcs
   item->setArgument(optimize(item->getArgument()));
-  //   item->setSelection(optimizeFTSelection(item->getSelection()));
+  item->setSelection(optimizeFTSelection(item->getSelection()));
   if(item->getIgnore())
     item->setIgnore(optimize(item->getIgnore()));
   return item;
 }
 
+FTSelection *ASTVisitor::optimizeFTSelection(FTSelection *selection)
+{
+  switch(selection->getType()) {
+  case FTSelection::OR:
+    return optimizeFTOr((FTOr*)selection);
+  case FTSelection::AND:
+    return optimizeFTAnd((FTAnd*)selection);
+  case FTSelection::MILD_NOT:
+    return optimizeFTMildnot((FTMildnot*)selection);
+  case FTSelection::UNARY_NOT:
+    return optimizeFTUnaryNot((FTUnaryNot*)selection);
+  case FTSelection::WORDS:
+    return optimizeFTWords((FTWords*)selection);
+  case FTSelection::WORD:
+    return optimizeFTWord((FTWord*)selection);
+  case FTSelection::ORDER:
+    return optimizeFTOrder((FTOrder*)selection);
+  case FTSelection::DISTANCE:
+    return optimizeFTDistance((FTDistance*)selection);
+  case FTSelection::DISTANCE_LITERAL:
+    return optimizeFTDistanceLiteral((FTDistanceLiteral*)selection);
+  case FTSelection::SCOPE:
+    return optimizeFTScope((FTScope*)selection);
+  case FTSelection::CONTENT:
+    return optimizeFTContent((FTContent*)selection);
+  case FTSelection::WINDOW:
+    return optimizeFTWindow((FTWindow*)selection);
+  case FTSelection::WINDOW_LITERAL:
+    return optimizeFTWindowLiteral((FTWindowLiteral*)selection);
+  }
+  return optimizeUnknownFTSelection(selection);
+}
+
+FTSelection *ASTVisitor::optimizeUnknownFTSelection(FTSelection *selection)
+{
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTWords(FTWords *selection)
+{
+  selection->setExpr(optimize(selection->getExpr()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTWord(FTWord *selection)
+{
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTOr(FTOr *selection)
+{
+  VectorOfFTSelections &args = const_cast<VectorOfFTSelections&>(selection->getArguments());
+  for(VectorOfFTSelections::iterator i = args.begin(); i != args.end(); ++i) {
+    *i = optimizeFTSelection(*i);
+  }
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTAnd(FTAnd *selection)
+{
+  VectorOfFTSelections &args = const_cast<VectorOfFTSelections&>(selection->getArguments());
+  for(VectorOfFTSelections::iterator i = args.begin(); i != args.end(); ++i) {
+    *i = optimizeFTSelection(*i);
+  }
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTMildnot(FTMildnot *selection)
+{
+  selection->setLeft(optimizeFTSelection(selection->getLeft()));
+  selection->setRight(optimizeFTSelection(selection->getRight()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTUnaryNot(FTUnaryNot *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTOrder(FTOrder *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTDistance(FTDistance *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+
+  FTRange &range = const_cast<FTRange&>(selection->getRange());
+  range.arg1 = optimize(range.arg1);
+  if(range.arg2) range.arg2 = optimize(range.arg2);
+
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTDistanceLiteral(FTDistanceLiteral *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTScope(FTScope *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTContent(FTContent *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTWindow(FTWindow *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  selection->setExpr(optimize(selection->getExpr()));
+  return selection;
+}
+
+FTSelection *ASTVisitor::optimizeFTWindowLiteral(FTWindowLiteral *selection)
+{
+  selection->setArgument(optimizeFTSelection(selection->getArgument()));
+  return selection;
+}
+
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 TupleNode *ASTVisitor::optimizeTupleNode(TupleNode *item)
@@ -693,6 +828,8 @@ TupleNode *ASTVisitor::optimizeTupleNode(TupleNode *item)
     return optimizeLetTuple((LetTuple*)item);
   case TupleNode::WHERE:
     return optimizeWhereTuple((WhereTuple*)item);
+  case TupleNode::COUNT:
+    return optimizeCountTuple((CountTuple*)item);
   case TupleNode::ORDER_BY:
     return optimizeOrderByTuple((OrderByTuple*)item);
   case TupleNode::DEBUG_HOOK:
@@ -734,6 +871,12 @@ TupleNode *ASTVisitor::optimizeWhereTuple(WhereTuple *item)
   return item;
 }
 
+TupleNode *ASTVisitor::optimizeCountTuple(CountTuple *item)
+{
+  item->setParent(optimizeTupleNode(const_cast<TupleNode*>(item->getParent())));
+  return item;
+}
+
 TupleNode *ASTVisitor::optimizeOrderByTuple(OrderByTuple *item)
 {
   item->setParent(optimizeTupleNode(const_cast<TupleNode*>(item->getParent())));
