@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -43,37 +41,35 @@
 #include <xqilla/operators/NodeComparison.hpp>
 #include <xqilla/operators/OrderComparison.hpp>
 #include <xqilla/operators/GeneralComp.hpp>
-#include <xqilla/operators/Range.hpp>
 #include <xqilla/operators/Intersect.hpp>
 #include <xqilla/operators/Except.hpp>
 #include <xqilla/operators/Union.hpp>
 
 #include <xqilla/functions/FunctionBaseURI.hpp>
 #include <xqilla/functions/FunctionCollection.hpp>
-#include <xqilla/functions/FunctionDeepEqual.hpp>
 #include <xqilla/functions/FunctionDoc.hpp>
 #include <xqilla/functions/FunctionDocument.hpp>
-#include <xqilla/functions/FunctionExactlyOne.hpp>
 #include <xqilla/functions/FunctionId.hpp>
 #include <xqilla/functions/FunctionIdref.hpp>
-#include <xqilla/functions/FunctionInsertBefore.hpp>
 #include <xqilla/functions/FunctionLang.hpp>
 #include <xqilla/functions/FunctionNormalizeSpace.hpp>
 #include <xqilla/functions/FunctionNumber.hpp>
-#include <xqilla/functions/FunctionOneOrMore.hpp>
 #include <xqilla/functions/FunctionParseXML.hpp>
 #include <xqilla/functions/FunctionParseJSON.hpp>
-#include <xqilla/functions/FunctionRemove.hpp>
-#include <xqilla/functions/FunctionReverse.hpp>
+#include <xqilla/functions/FunctionExplain.hpp>
 #include <xqilla/functions/FunctionRoot.hpp>
 #include <xqilla/functions/FunctionString.hpp>
 #include <xqilla/functions/FunctionStringLength.hpp>
-#include <xqilla/functions/FunctionSubsequence.hpp>
 #include <xqilla/functions/FunctionTrace.hpp>
 #include <xqilla/functions/FunctionUnordered.hpp>
-#include <xqilla/functions/FunctionZeroOrOne.hpp>
+#include <xqilla/functions/FunctionHead.hpp>
+#include <xqilla/functions/FunctionTail.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 
 #include <xqilla/update/FunctionPut.hpp>
+#include <xqilla/utils/XPath2Utils.hpp>
+
+#include "../items/impl/FunctionRefImpl.hpp"
 
 #include <xercesc/util/XMLUri.hpp>
 
@@ -162,11 +158,11 @@ void QueryPathTreeGenerator::optimize(XQQuery *query)
 XQUserFunction *QueryPathTreeGenerator::optimizeFunctionDef(XQUserFunction *item)
 {
   if(item->getFunctionBody() != 0) {
-    const XQUserFunction::ArgumentSpecs *params = item->getArgumentSpecs();
+    const ArgumentSpecs *params = item->getSignature()->argSpecs;
     if(params) {
       varStore_.addScope(VarStore::MyScope::LOCAL_SCOPE);
 
-      for(XQUserFunction::ArgumentSpecs::const_iterator it = params->begin();
+      for(ArgumentSpecs::const_iterator it = params->begin();
           it != params->end(); ++it) {
           PathResult paramRes;
           if((*it)->getStaticAnalysis().getStaticType().containsType(StaticType::NODE_TYPE))
@@ -645,52 +641,15 @@ ASTNode *QueryPathTreeGenerator::optimizeFunction(XQFunction *item)
       result = generate(args[0]);
       generate(args[1]);
     }
-    else if(name == FunctionZeroOrOne::name ||
-            name == FunctionOneOrMore::name ||
-            name == FunctionExactlyOne::name ||
-            name == FunctionReverse::name ||
+    else if(name == FunctionHead::name ||
+            name == FunctionTail::name ||
             name == FunctionUnordered::name) {
       // return their argument
       result = generate(args[0]);
     }
-    else if(name == FunctionInsertBefore::name) {
-      // behaves a bit like union
-      PathResult ret = generate(args[0]);
-      result.join(ret);
-
-      generate(args[1]);
-
-      PathResult ret2 = generate(args[2]);
-      result.join(ret2);
-    }
-    else if(name == FunctionRemove::name) {
-      result = generate(args[0]);
-      generate(args[1]);
-    }
-    else if(name == FunctionSubsequence::name) {
-      result = generate(args[0]);
-      generate(args[1]);
-      if(args.size() > 2) {
-        generate(args[2]);
-      }
-    }
-    else if(name == FunctionDeepEqual::name) {
-      generate(args[0]).markSubtreeResult();
-      generate(args[1]).markSubtreeResult();
-      if(args.size() > 2) {
-        generate(args[2]);
-      }
-    }
     else if(name == FunctionId::name ||
             name == FunctionIdref::name) {
-      PathResult nodes;
-      if(args.size() == 1) {
-        const PathResult &currentContext = getCurrentContext();
-        nodes = currentContext;
-      }
-      else {
-        nodes = generate(args[1]);
-      }
+      PathResult nodes = generate(args[1]);
 
       // We don't handle these, so we mark the trees as unoptimisable
       nodes.markRoot();
@@ -700,9 +659,7 @@ ASTNode *QueryPathTreeGenerator::optimizeFunction(XQFunction *item)
     else if(name == FunctionLang::name) {
       generate(args[0]);
 
-      PathResult nodeResult;
-      if(args.size() == 1) nodeResult = getCurrentContext();
-      else nodeResult = generate(args[1]);
+      PathResult nodeResult = generate(args[1]);
 
       // We want all the xml:lang attributes from all our lookup node's ancestors
 
@@ -724,9 +681,7 @@ ASTNode *QueryPathTreeGenerator::optimizeFunction(XQFunction *item)
       attrResult.markSubtreeValue();
     }
     else if(name == FunctionBaseURI::name) {
-      PathResult nodeResult;
-      if(args.size() == 0) nodeResult = getCurrentContext();
-      else nodeResult = generate(args[0]);
+      PathResult nodeResult = generate(args[0]);
 
       // We want all the xml:base attributes from all our lookup node's ancestors
 
@@ -799,6 +754,20 @@ ASTNode *QueryPathTreeGenerator::optimizeFunction(XQFunction *item)
       result.join(root);
     }
 
+    else if(name == FunctionExplain::name) {
+      // Returns a sequence of elements
+      generate(args[0]);
+
+      QueryPathNode *root = ((FunctionExplain*)item)->getQueryPathTree();
+      if(!root) {
+        NodeTest *nt = createNodeTest(Node::element_string);
+        root = createQueryPathNode(nt, QueryPathNode::CHILD);
+        ((FunctionExplain*)item)->setQueryPathTree(root);
+      }
+
+      result.join(root);
+    }
+
     else {
       // We don't know about this function, assume it will only need the descendants of it's node arguments
       for(VectorOfASTNodes::iterator i = args.begin(); i != args.end(); ++i) {
@@ -836,9 +805,9 @@ ASTNode *QueryPathTreeGenerator::optimizeUserFunction(XQUserFunctionInstance *it
 
   // Evaluate the arguments in the current scope, declare them in the function's scope
   vector<ArgHolder> evaluatedArgs;
-  if(functionDef->getArgumentSpecs()) {
-    XQUserFunction::ArgumentSpecs::const_iterator binding = functionDef->getArgumentSpecs()->begin();
-    for(VectorOfASTNodes::iterator arg = args.begin(); arg != args.end() && binding != functionDef->getArgumentSpecs()->end(); ++arg, ++binding) {
+  if(functionDef->getSignature()->argSpecs) {
+    ArgumentSpecs::const_iterator binding = functionDef->getSignature()->argSpecs->begin();
+    for(VectorOfASTNodes::iterator arg = args.begin(); arg != args.end() && binding != functionDef->getSignature()->argSpecs->end(); ++arg, ++binding) {
 
       PathResult r = generate(*arg);
 
@@ -965,13 +934,6 @@ ASTNode *QueryPathTreeGenerator::optimizeIf(XQIf *item)
   return item;
 }
 
-ASTNode *QueryPathTreeGenerator::optimizeInstanceOf(XQInstanceOf *item)
-{
-  generate(const_cast<ASTNode *>(item->getExpression()));
-  push(PathResult());
-  return item;
-}
-
 TupleNode *QueryPathTreeGenerator::optimizeForTuple(ForTuple *item)
 {
   optimizeTupleNode(const_cast<TupleNode*>(item->getParent()));
@@ -1010,6 +972,12 @@ TupleNode *QueryPathTreeGenerator::optimizeWhereTuple(WhereTuple *item)
   return item;
 }
 
+TupleNode *QueryPathTreeGenerator::optimizeCountTuple(CountTuple *item)
+{
+  optimizeTupleNode(const_cast<TupleNode*>(item->getParent()));
+  return item;
+}
+
 TupleNode *QueryPathTreeGenerator::optimizeOrderByTuple(OrderByTuple *item)
 {
   optimizeTupleNode(const_cast<TupleNode*>(item->getParent()));
@@ -1224,6 +1192,12 @@ ASTNode *QueryPathTreeGenerator::optimizeNumericLiteral(XQNumericLiteral *item)
   return item;
 }
 
+ASTNode *QueryPathTreeGenerator::optimizeFunctionRef(XQFunctionRef *item)
+{
+  push(PathResult());
+  return item;
+}
+
 ASTNode *QueryPathTreeGenerator::optimizeAtomize(XQAtomize *item)
 {
   generate(const_cast<ASTNode *>(item->getExpression())).markSubtreeValue();
@@ -1388,12 +1362,10 @@ ASTNode *QueryPathTreeGenerator::optimizeFunctionCall(XQFunctionCall *item)
 
 ASTNode *QueryPathTreeGenerator::optimizeFTContains(FTContains *item)
 {
-  // TBD implement optimization of FTSelection objects - jpcs
-  PathResult result = generate(item->getArgument());
-  //   item->setSelection(optimizeFTSelection(item->getSelection()));
-  if(item->getIgnore())
-    generate(item->getIgnore());
-  push(result);
+  generate(item->getArgument());
+  optimizeFTSelection(item->getSelection());
+  if(item->getIgnore()) generate(item->getIgnore());
+  push(PathResult());
   return item;
 }
 
@@ -1480,15 +1452,26 @@ ASTNode *QueryPathTreeGenerator::optimizeInlineFunction(XQInlineFunction *item)
   if(item->getUserFunction())
     optimizeFunctionDef(item->getUserFunction());
 
-  // TBD What about the function instance? - jpcs
+  const ArgumentSpecs *params = item->getSignature()->argSpecs;
+  if(params) {
+    varStore_.addScope(VarStore::MyScope::LOCAL_SCOPE);
 
-  push(PathResult());
-  return item;
-}
+    ArgumentSpecs::const_iterator it = params->begin();
+    unsigned int c = 0;
+    for(; it != params->end(); ++it, ++c) {
+      PathResult paramRes;
+      if((*it)->getStaticAnalysis().getStaticType().containsType(StaticType::NODE_TYPE))
+        createAnyNodeResult(paramRes);
+      setVariable((*it)->getURI(), (*it)->getName(), paramRes);
+    }
+  }
+
+  generate(item->getInstance()).markSubtreeResult();
+
+  if(params) {
+    delete varStore_.popScope();
+  }
 
-ASTNode *QueryPathTreeGenerator::optimizeFunctionRef(XQFunctionRef *item)
-{
-  // TBD What about the function instance? - jpcs
   push(PathResult());
   return item;
 }
@@ -1515,6 +1498,29 @@ ASTNode *QueryPathTreeGenerator::optimizeFunctionDeref(XQFunctionDeref *item)
   return item;
 }
 
+ASTNode *QueryPathTreeGenerator::optimizePartialApply(XQPartialApply *item)
+{
+  // TBD Could be better - jpcs
+  context_->setProjection(false);
+
+  // We could trace which function instances could possibly be called here
+  // and treat the function dereference as the union of all the functions
+  // it could call.
+
+  generate(item->getExpression());
+
+  VectorOfASTNodes *args = const_cast<VectorOfASTNodes*>(item->getArguments());
+  if(args) {
+    for(VectorOfASTNodes::iterator i = args->begin(); i != args->end(); ++i) {
+      if(*i != 0)
+        generate(*i).markRoot();
+    }
+  }
+
+  push(PathResult());
+  return item;
+}
+
 #define UNCHANGED_FULL(methodname, classname) \
 ASTNode *QueryPathTreeGenerator::optimize ## methodname (classname *item) \
 { \
@@ -1527,6 +1533,7 @@ ASTNode *QueryPathTreeGenerator::optimize ## methodname (classname *item) \
 UNCHANGED_XQ(NameExpression)
 UNCHANGED_XQ(CastAs)
 UNCHANGED_XQ(TreatAs)
+UNCHANGED_XQ(FunctionCoercion)
 UNCHANGED_XQ(OrderingChange)
 UNCHANGED_XQ(CastableAs)
 UNCHANGED_XQ(PromoteUntyped)
@@ -1534,11 +1541,30 @@ UNCHANGED_XQ(PromoteNumeric)
 UNCHANGED_XQ(PromoteAnyURI)
 UNCHANGED_XQ(DocumentOrder)
 UNCHANGED(UApplyUpdates)
-UNCHANGED(XPath1CompatConvertFunctionArg)
 UNCHANGED_XQ(NamespaceBinding)
 UNCHANGED_XQ(FunctionConversion)
 UNCHANGED(ASTDebugHook)
 
+#define UNCHANGED_FT(classname) \
+FTSelection *QueryPathTreeGenerator::optimize ## classname (classname *item) \
+{ \
+  return ASTVisitor::optimize ## classname (item); \
+}
+
+UNCHANGED_FT(FTWords)
+UNCHANGED_FT(FTWord)
+UNCHANGED_FT(FTMildnot)
+UNCHANGED_FT(FTUnaryNot)
+UNCHANGED_FT(FTOrder)
+UNCHANGED_FT(FTDistance)
+UNCHANGED_FT(FTDistanceLiteral)
+UNCHANGED_FT(FTScope)
+UNCHANGED_FT(FTContent)
+UNCHANGED_FT(FTWindow)
+UNCHANGED_FT(FTWindowLiteral)
+UNCHANGED_FT(FTOr)
+UNCHANGED_FT(FTAnd)
+
 ASTNode *QueryPathTreeGenerator::optimizeUnknown(ASTNode *item)
 {
   return ASTVisitor::optimizeUnknown(item);
@@ -1549,6 +1575,11 @@ TupleNode *QueryPathTreeGenerator::optimizeUnknownTupleNode(TupleNode *item)
   return ASTVisitor::optimizeUnknownTupleNode(item);
 }
 
+FTSelection *QueryPathTreeGenerator::optimizeUnknownFTSelection(FTSelection *selection)
+{
+  return ASTVisitor::optimizeUnknownFTSelection(selection);
+}
+
 TupleNode *QueryPathTreeGenerator::optimizeContextTuple(ContextTuple *item)
 {
   return ASTVisitor::optimizeContextTuple(item);
