@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -27,56 +25,54 @@
 
 using namespace std;
 
-Result ClosureResult::create(const ASTNode *ast, DynamicContext *context, const VariableStore *sourceScope)
+class LazyCreateResult : public ResultImpl
 {
-  // TBD We probably need to store the regex groups here too - jpcs
+public:
+  LazyCreateResult(const ASTNode *ast)
+    : ResultImpl(ast), ast_(ast) {}
 
-  if(!ast->getStaticAnalysis().variablesUsed() &&
-     !ast->getStaticAnalysis().areContextFlagsUsed()) {
-    return ast->createResult(context);
+  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context)
+  {
+    tail = ast_->createResult(context);
+    return 0;
   }
 
-  return new ClosureResult(ast, context, sourceScope);
+private:
+  const ASTNode *ast_;
+};
+
+Result ClosureResult::create(const StaticAnalysis &src, DynamicContext *context, ResultImpl *result)
+{
+  // TBD We probably need to store the regex groups here too - jpcs
+  if(!src.isVariableUsed() && !src.areContextFlagsUsed())
+    return result;
+  return new ClosureResult(src, context, result);
 }
 
-ClosureResult::ClosureResult(const ASTNode *ast, DynamicContext *context, const VariableStore *sourceScope)
-  : ResultImpl(ast),
+Result ClosureResult::create(const ASTNode *ast, DynamicContext *context)
+{
+  return create(ast->getStaticAnalysis(), context, new LazyCreateResult(ast));
+}
+
+ClosureResult::ClosureResult(const StaticAnalysis &src, DynamicContext *context, ResultImpl *result)
+  : ResultImpl(result),
     contextItem_(context->getContextItem()),
     contextPosition_(context->getContextPosition()),
     contextSize_(context->getContextSize()),
     varStore_(context->getMemoryManager()),
-    docCache_(const_cast<DocumentCache*>(context->getDocumentCache())),
-    result_(0)
+    result_(result)
 {
-  if(sourceScope == 0) sourceScope = context->getVariableStore();
-
   // Copy the variables we need into our local storage
-  varStore_.cacheVariableStore(ast->getStaticAnalysis(), sourceScope);
-
-  try {
-    AutoVariableStoreReset vsReset(context, &varStore_);
-    result_ = ast->createResult(context);
-  }
-  catch(...) {
-    varStore_.clear();
-    throw;
-  }
-}
-
-Item::Ptr ClosureResult::next(DynamicContext *context)
-{
-  AutoDocumentCacheReset dcReset(context);
-  context->setDocumentCache(docCache_);
-  AutoContextInfoReset ciReset(context, contextItem_, contextPosition_, contextSize_);
-  AutoVariableStoreReset vsReset(context, &varStore_);
-
-  return result_->next(context);
+  varStore_.cacheVariableStore(src, context->getVariableStore());
 }
 
 Item::Ptr ClosureResult::nextOrTail(Result &tail, DynamicContext *context)
 {
-  AutoDocumentCacheReset dcReset(context);
-  context->setDocumentCache(docCache_);
+  context->testInterrupt();
+
+  // TBD Fix the DocumentCache/module import problem - jpcs
+  // AutoDocumentCacheReset dcReset(context);
+  // context->setDocumentCache(docCache_);
   AutoContextInfoReset ciReset(context, contextItem_, contextPosition_, contextSize_);
   AutoVariableStoreReset vsReset(context, &varStore_);
 
@@ -88,8 +84,7 @@ ClosureEventGenerator::ClosureEventGenerator(const ASTNode *ast, DynamicContext
     contextItem_(context->getContextItem()),
     contextPosition_(context->getContextPosition()),
     contextSize_(context->getContextSize()),
-    varStore_(context->getMemoryManager()),
-    docCache_(const_cast<DocumentCache*>(context->getDocumentCache()))
+    varStore_(context->getMemoryManager())
 {
   // Copy the variables we need into our local storage
   varStore_.cacheVariableStore(ast->getStaticAnalysis(), context->getVariableStore());
@@ -97,8 +92,10 @@ ClosureEventGenerator::ClosureEventGenerator(const ASTNode *ast, DynamicContext
 
 EventGenerator::Ptr ClosureEventGenerator::generateEvents(EventHandler *events, DynamicContext *context)
 {
-  AutoDocumentCacheReset dcReset(context);
-  context->setDocumentCache(docCache_);
+  context->testInterrupt();
+
+  // AutoDocumentCacheReset dcReset(context);
+  // context->setDocumentCache(docCache_);
   AutoContextInfoReset ciReset(context, contextItem_, contextPosition_, contextSize_);
   AutoVariableStoreReset vsReset(context, &varStore_);
 
