@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -41,8 +39,10 @@ Sequence EmptyResult::toSequence(DynamicContext *context)
   return Sequence(context->getMemoryManager());
 }
 
-ResultBufferImpl *EmptyResult::toResultBuffer(unsigned int readCount)
+void EmptyResult::toResultBuffer(unsigned int readCount, ResultBuffer &buffer)
 {
-  return new ResultBufferImpl(Result(0), readCount);
 }
 
+void EmptyResult::skip(unsigned count, DynamicContext *context)
+{
+}
