@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -27,17 +25,21 @@
 #include <xqilla/context/DynamicContext.hpp>
 
 ResultBuffer::ResultBuffer(const Result &result, unsigned int readCount)
-  : _impl(const_cast<ResultImpl*>(result.get())->toResultBuffer(readCount))
+  : _impl(0),
+    _start(0)
 {
+  const_cast<ResultImpl*>(result.get())->toResultBuffer(readCount,*this);
 }
 
 ResultBuffer::ResultBuffer(const Item::Ptr &item, unsigned int readCount)
-  : _impl(new ResultBufferImpl(item, readCount))
+  : _impl(new ResultBufferImpl(item, readCount)),
+    _start(0)
 {
 }
 
-ResultBuffer::ResultBuffer(ResultBufferImpl *impl)
-  : _impl(impl)
+ResultBuffer::ResultBuffer(ResultBufferImpl *impl, unsigned start)
+  : _impl(impl),
+    _start(start)
 {
 }
 
@@ -45,7 +47,7 @@ Result ResultBuffer::createResult()
 {
   if(_impl.isNull()) return 0;
 
-  Result result = _impl->createResult();
+  Result result = _impl->createResult(_start);
 
   if(_impl->getMaxReadCount() != ResultBufferImpl::UNLIMITED_COUNT &&
      _impl->incrementReadCount() >= _impl->getMaxReadCount()) {
