@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -68,6 +66,19 @@ Sequence::Sequence(size_t n, MemoryManager* memMgr)
     _itemList.reserve(n);
 }
 
+Sequence::Sequence(const Sequence&s, size_t start, MemoryManager* memMgr)
+  : _itemList(XQillaAllocator<Item::Ptr>(memMgr))
+{
+  size_t end = s._itemList.size();
+  if(end > start) {
+    _itemList.reserve(end - start);
+    while(end > start) {
+      _itemList.push_back(s._itemList[start]);
+      ++start;
+    }
+  }
+}
+
 Sequence & Sequence::operator=(const Sequence & s) {
 	if( this != &s ) { // no self-assignment
 	  _itemList = s._itemList;
