@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: SequenceType.cpp 734 2009-01-30 18:08:37Z gmfeinberg $
  */
 
 #include "../config/xqilla_config.h"
@@ -39,14 +37,15 @@
 #include <xqilla/items/ATUntypedAtomic.hpp>
 #include <xqilla/items/Node.hpp>
 #include <xqilla/items/FunctionRef.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/functions/FunctionString.hpp>
 #include <xqilla/functions/FunctionNumber.hpp>
+#include <xqilla/functions/FunctionHead.hpp>
+#include <xqilla/functions/FunctionSignature.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
 #include <xqilla/ast/XQAtomize.hpp>
 #include <xqilla/ast/XQTreatAs.hpp>
-#include <xqilla/ast/XPath1Compat.hpp>
 #include <xqilla/ast/ConvertFunctionArg.hpp>
+#include <xqilla/ast/XQFunctionCoercion.hpp>
 
 #include <xercesc/validators/schema/SchemaAttDef.hpp>
 #include <xercesc/validators/schema/SchemaElementDecl.hpp>
@@ -191,7 +190,7 @@ void SequenceType::ItemType::staticResolution(StaticContext *context, const Loca
         XQThrow3(StaticErrorException, X("SequenceType::ItemType::staticResolution"), buf.getRawBuffer(), location);
       }
       if(!context->getDocumentCache()->isTypeOrDerivedFromType(m_TypeURI, m_pType->getName(),
-                                                               FunctionConstructor::XMLChXPath2DatatypesURI,
+                                                               SchemaSymbols::fgURI_SCHEMAFORSCHEMA,
                                                                AnyAtomicType::fgDT_ANYATOMICTYPE)) {
         XMLBuffer buf;
         buf.set(X("Type {"));
@@ -202,6 +201,12 @@ void SequenceType::ItemType::staticResolution(StaticContext *context, const Loca
         XQThrow3(StaticErrorException, X("SequenceType::ItemType::staticResolution"), buf.getRawBuffer(), location);
       }
     }
+    else if(XPath2Utils::equals(m_pType->getName(), SchemaSymbols::fgATTVAL_ANYTYPE) &&
+            XPath2Utils::equals(m_TypeURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+      // xs:anyType is the same as a type wildcard
+      m_pType = 0;
+      m_TypeURI = 0;
+    }
     else if(!context->getDocumentCache()->isTypeDefined(m_TypeURI, m_pType->getName())) {
       XMLBuffer msg;
       msg.set(X("Type {"));
@@ -436,6 +441,11 @@ void SequenceType::ItemType::getStaticType(StaticType &st, const StaticContext *
     isExact = true;
     break;
   }
+  case TEST_NAMESPACE: {
+    st = StaticType::NAMESPACE_TYPE;
+    isExact = true;
+    break;
+  }
   case TEST_NODE: {
     st = StaticType::NODE_TYPE;
     isExact = true;
@@ -616,13 +626,17 @@ void SequenceType::ItemType::toBuffer(XMLBuffer &buffer, bool addBrackets) const
     buffer.append(X("text()"));
     break;
   }
+  case TEST_NAMESPACE: {
+    buffer.append(X("namespace-node()"));
+    break;
+  }
   case TEST_NODE: {
     buffer.append(X("node()"));
     break;
   }
   case TEST_FUNCTION: {
     if(returnType_ == 0) {
-      buffer.append(X("function()"));
+      buffer.append(X("function(*)"));
     }
     else {
       addBrackets = addBrackets && returnType_->getOccurrenceIndicator() == EXACTLY_ONE;
@@ -633,9 +647,9 @@ void SequenceType::ItemType::toBuffer(XMLBuffer &buffer, bool addBrackets) const
       bool doneOne = false;
       for(VectorOfSequenceTypes::const_iterator i = argTypes_->begin();
           i != argTypes_->end(); ++i) {
-        (*i)->toBuffer(buffer);
         if(doneOne) buffer.append(',');
         doneOne = true;
+        (*i)->toBuffer(buffer);
       }
       buffer.append(X(") as "));
       returnType_->toBuffer(buffer);
@@ -826,6 +840,11 @@ bool SequenceType::ItemType::matches(const Node::Ptr &toBeTested, DynamicContext
       return (toBeTested->dmNodeKind() == Node::text_string);
     }
 
+    case TEST_NAMESPACE:
+    {
+      return (toBeTested->dmNodeKind() == Node::namespace_string);
+    }
+
     case TEST_SCHEMA_DOCUMENT:
     case TEST_DOCUMENT:
     {
@@ -865,10 +884,69 @@ bool SequenceType::ItemType::matches(const Node::Ptr &toBeTested, DynamicContext
   return true;
 }
 
+bool SequenceType::ItemType::matches(const FunctionRef::Ptr &toBeTested, DynamicContext* context) const
+{
+  return matches(toBeTested->getSignature(), context);
+}
+
+bool SequenceType::ItemType::matches(const FunctionSignature *sig, DynamicContext* context) const
+{
+  switch(m_nTestType) {
+    case TEST_ELEMENT:
+    case TEST_ATTRIBUTE:
+    case TEST_SCHEMA_ELEMENT:
+    case TEST_SCHEMA_ATTRIBUTE:
+    case TEST_NODE:
+    case TEST_PI:
+    case TEST_COMMENT:
+    case TEST_TEXT:
+    case TEST_DOCUMENT:
+    case TEST_SCHEMA_DOCUMENT:
+    case TEST_NAMESPACE:
+    case TEST_ATOMIC_TYPE:
+    {
+      return false;
+    }
+    
+    case TEST_ANYTHING:
+    {
+      return true;
+    }
+
+    case TEST_FUNCTION:
+    {
+      // function(*) matches any function item.
+      if(returnType_ == 0) return true;
+
+      // A TypedFunctionTest matches an item if it is a function item, and the function
+      // item's type signature is a subtype of the TypedFunctionTest.
+      size_t numArgs = sig->argSpecs ? sig->argSpecs->size() : 0;
+      if(numArgs != argTypes_->size()) return false;
+
+      if(sig->argSpecs) {
+        ArgumentSpecs::const_iterator aa_i = sig->argSpecs->begin();
+        VectorOfSequenceTypes::const_iterator ba_i = argTypes_->begin();
+        for(; aa_i != sig->argSpecs->end() && ba_i != argTypes_->end(); ++aa_i, ++ba_i) {
+          if(!(*ba_i)->isSubtypeOf((*aa_i)->getType(), context)) return false;
+        }
+      }
+
+      if(sig->returnType)
+        return sig->returnType->isSubtypeOf(returnType_, context);
+
+      return returnType_->m_nOccurrence == STAR && returnType_->m_pItemType &&
+        returnType_->m_pItemType->getItemTestType() == TEST_ANYTHING;
+    }
+  }
+  return true;
+}
+
 bool SequenceType::ItemType::matches(const Item::Ptr &toBeTested, DynamicContext* context) const
 {
   if(toBeTested->isNode())
-    return matches((const Node::Ptr)toBeTested, context);
+    return matches((Node::Ptr)toBeTested, context);
+  if(toBeTested->isFunction())
+    return matches((FunctionRef::Ptr)toBeTested, context);
     
   switch(m_nTestType) {
     case TEST_ELEMENT:
@@ -881,6 +959,8 @@ bool SequenceType::ItemType::matches(const Item::Ptr &toBeTested, DynamicContext
     case TEST_TEXT:
     case TEST_DOCUMENT:
     case TEST_SCHEMA_DOCUMENT:
+    case TEST_NAMESPACE:
+    case TEST_FUNCTION:
     {
       return false;
     }
@@ -895,74 +975,76 @@ bool SequenceType::ItemType::matches(const Item::Ptr &toBeTested, DynamicContext
       if(!toBeTested->isAtomicValue()) return false;
       return matchesNameType(toBeTested, context);
     }
-
-    case TEST_FUNCTION:
-    {
-      if(!toBeTested->isFunction()) return false;
-      if(returnType_ == 0) return true;
-
-      FunctionRef *func = (FunctionRef*)toBeTested.get();
-      if(func->getNumArgs() != argTypes_->size()) return false;
-
-      return true;
-    }
-
   }
   return true;
 }
 
-bool SequenceType::matches(const SequenceType *toBeTested, const StaticContext* context) const
+bool SequenceType::isSubtypeOf(const SequenceType *b, const StaticContext* context) const
 {
-  if(m_pItemType == 0)
-    return toBeTested->m_pItemType == 0;
-  if(toBeTested->m_pItemType == 0)
-    return m_nOccurrence == QUESTION_MARK || m_nOccurrence == STAR;
+  const SequenceType *a = this;
 
-  switch(m_nOccurrence) {
+  if(b->m_pItemType == 0)
+    return a->m_pItemType == 0;
+  if(a->m_pItemType == 0)
+    return b->m_nOccurrence == QUESTION_MARK || b->m_nOccurrence == STAR;
+
+  switch(b->m_nOccurrence) {
   case EXACTLY_ONE:
-    if(toBeTested->m_nOccurrence != EXACTLY_ONE) return false;
+    if(a->m_nOccurrence != EXACTLY_ONE) return false;
     break;
   case PLUS:
-    if(toBeTested->m_nOccurrence != EXACTLY_ONE &&
-       toBeTested->m_nOccurrence != PLUS) return false;
+    if(a->m_nOccurrence != EXACTLY_ONE && a->m_nOccurrence != PLUS) return false;
     break;
   case QUESTION_MARK:
-    if(toBeTested->m_nOccurrence == PLUS) return false;
+    if(a->m_nOccurrence != QUESTION_MARK && a->m_nOccurrence != EXACTLY_ONE) return false;
     break;
   case STAR:
     break;
   }
 
-  return m_pItemType->matches(toBeTested->m_pItemType, context);
+  return a->m_pItemType->isSubtypeOf(b->m_pItemType, context);
 }
 
-bool SequenceType::ItemType::matchesNameType(const ItemType *toBeTested, const StaticContext* context) const
+bool SequenceType::ItemType::isSubtypeOfNameType(const ItemType *b, const StaticContext* context) const
 {
-  if(m_pName) {
-    if(toBeTested->m_pName == 0) return false;
-    if(!XPath2Utils::equals(m_pName->getName(), toBeTested->m_pName->getName()) ||
-       !XPath2Utils::equals(m_NameURI, toBeTested->m_NameURI))
+  const ItemType *a = this;
+
+  if(a->m_pName) {
+    if(b->m_pName == 0) return false;
+    if(!XPath2Utils::equals(a->m_pName->getName(), b->m_pName->getName()) ||
+       !XPath2Utils::equals(a->m_NameURI, b->m_NameURI))
       return false;
   }
 
-  if(m_pType) {
-    if(toBeTested->m_pType == 0) return false;
-    if(!context->isTypeOrDerivedFromType(toBeTested->m_TypeURI, toBeTested->m_pType->getName(), m_TypeURI, m_pType->getName()))
+  if(a->m_pType) {
+    if(b->m_pType == 0) return false;
+    if(!context->isTypeOrDerivedFromType(a->m_TypeURI, a->m_pType->getName(), b->m_TypeURI, b->m_pType->getName()))
       return false;
   }
 
   return true;
 }
 
-bool SequenceType::ItemType::matches(const ItemType *toBeTested, const StaticContext* context) const
+bool SequenceType::ItemType::isSubtypeOf(const ItemType *b, const StaticContext* context) const
 {
-  switch(m_nTestType) {
+  const ItemType *a = this;
+
+  switch(b->m_nTestType) {
+
+  case TEST_ATOMIC_TYPE: {
+    // Ai and Bi are AtomicTypes, and derives-from(Ai, Bi) returns true.
+    if(b->m_nTestType != TEST_ATOMIC_TYPE) return false;
+    return a->isSubtypeOfNameType(b, context);
+  }
+
   case TEST_ANYTHING: {
+    // Bi is item().
     return true;
   }
 
   case TEST_NODE: {
-    switch(toBeTested->m_nTestType) {
+    // Bi is node(), and Ai is a KindTest.
+    switch(a->m_nTestType) {
     case TEST_ELEMENT:
     case TEST_ATTRIBUTE:
     case TEST_SCHEMA_ELEMENT:
@@ -979,99 +1061,96 @@ bool SequenceType::ItemType::matches(const ItemType *toBeTested, const StaticCon
     }
   }
 
+  case TEST_TEXT: {
+    // Bi is text() and Ai is also text().
+    return b->m_nTestType == TEST_TEXT;
+  }
+
+  case TEST_COMMENT: {
+    // Bi is comment() and Ai is also comment().
+    return a->m_nTestType == TEST_COMMENT;
+  }
+
+  case TEST_NAMESPACE: {
+    // Bi is namespace-node() and Ai is also namespace-node().
+    return a->m_nTestType == TEST_NAMESPACE;
+  }
+
+  case TEST_PI: {
+    // Bi is processing-instruction() and Ai is either processing-instruction() or processing-instruction(N) for any name N..
+    // Bi is processing-instruction(Bn), and Ai is also processing-instruction(Bn).
+    if(a->m_nTestType != TEST_PI) return false;
+    return a->isSubtypeOfNameType(b, context);
+  }
+
+  case TEST_DOCUMENT:
+    // Bi is document-node() and Ai is either document-node() or document-node(E) for any ElementTest E.
+    // Bi is document-node(Be) and Ai is document-node(Ae), and subtype-itemtype(Ae, Be).
   case TEST_ELEMENT: {
-    if(toBeTested->m_nTestType != TEST_ELEMENT)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    if(!m_bAllowNil && toBeTested->m_bAllowNil) return false;
-    return true;
+    // Bi is either element() or element(*), and Ai is an ElementTest.
+    // Bi is either element(Bn) or element(Bn, xs:anyType), and Ai is either element(Bn), or element(Bn, T) for any type T.
+    // Bi is element(Bn, Bt), Ai is element(Bn, At), and derives-from(At, Bt) returns true.
+    // Bi is element(Bn, Bt?), Ai is either element(Bn, At), or element(Bn, At?), and derives-from(At, Bt) returns true.
+    // Bi is element(*, Bt), Ai is either element(*, At), or element(N, At) for any name N, and derives-from(At, Bt) returns true.
+    // Bi is element(*, Bt?), Ai is either element(*, At), element(*, At?), element(N, At), or element(N, At?) for any name N, and derives-from(At, Bt) returns true.
+    if(a->m_nTestType != b->m_nTestType) return false;
+    if(!a->isSubtypeOfNameType(b, context)) return false;
+    return !a->m_bAllowNil || b->m_bAllowNil;
+  }
+
+  case TEST_SCHEMA_DOCUMENT:
+    // Bi is document-node() and Ai is either document-node() or document-node(E) for any ElementTest E.
+    // Bi is document-node(Be) and Ai is document-node(Ae), and subtype-itemtype(Ae, Be).
+  case TEST_SCHEMA_ELEMENT: {
+    // Bi is schema-element(Bn), Ai is schema-element(An), and either the expanded QName An equals the expanded QName Bn or
+    // the element declaration named An is in the substitution group of the element declaration named Bn.
+    if(a->m_nTestType != b->m_nTestType) return false;
+    if(a->isSubtypeOfNameType(b, context)) return true;
+
+    // Check substitution groups
+    SchemaElementDecl* aElemDecl = context->getDocumentCache()->getElementDecl(a->m_NameURI, a->m_pName->getName());
+    while(aElemDecl) {
+      if(XPath2Utils::equals(aElemDecl->getBaseName(), b->m_pName->getName()) &&
+         XPath2Utils::equals(context->getDocumentCache()->getSchemaUri(aElemDecl->getURI()), b->m_NameURI))
+        return true;
+      aElemDecl = aElemDecl->getSubstitutionGroupElem();
+    }
+    return false;
   }
 
   case TEST_ATTRIBUTE: {
-    if(toBeTested->m_nTestType != TEST_ATTRIBUTE)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
-  }
-
-  case TEST_SCHEMA_ELEMENT: {
-    if(toBeTested->m_nTestType != TEST_SCHEMA_ELEMENT)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
+    // Bi is either attribute() or attribute(*), and Ai is an AttributeTest.
+    // Bi is either attribute(Bn) or attribute(Bn, xs:anyType), and Ai is either attribute(Bn), or attribute(Bn, T) for any type T.
+    // Bi is attribute(Bn, Bt), Ai is attribute(Bn, At), and derives-from(At, Bt) returns true.
+    // Bi is attribute(*, Bt), Ai is either attribute(*, At), or attribute(N, At) for any name N, and derives-from(At, Bt) returns true.
+    if(a->m_nTestType != TEST_ATTRIBUTE) return false;
+    if(!a->isSubtypeOfNameType(b, context)) return false;
     return true;
   }
 
   case TEST_SCHEMA_ATTRIBUTE: {
-    if(toBeTested->m_nTestType != TEST_SCHEMA_ATTRIBUTE)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
-  }
-
-  case TEST_PI: {
-    if(toBeTested->m_nTestType != TEST_PI)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
-  }
-
-  case TEST_COMMENT: {
-    if(toBeTested->m_nTestType != TEST_COMMENT)
-      return false;
-    return true;
-  }
-
-  case TEST_TEXT: {
-    if(toBeTested->m_nTestType != TEST_TEXT)
-      return false;
-    return true;
-  }
-
-  case TEST_DOCUMENT: {
-    if(toBeTested->m_nTestType != TEST_DOCUMENT)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
-  }
-    
-  case TEST_SCHEMA_DOCUMENT: {
-    if(toBeTested->m_nTestType != TEST_SCHEMA_DOCUMENT)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
-  }
-    
-  case TEST_ATOMIC_TYPE: {
-    if(toBeTested->m_nTestType != TEST_ATOMIC_TYPE)
-      return false;
-    if(!matchesNameType(toBeTested, context))
-      return false;
-    return true;
+    // Bi is schema-attribute(Bn) and Ai is also schema-attribute(Bn).
+    if(a->m_nTestType != TEST_SCHEMA_ATTRIBUTE) return false;
+    return a->isSubtypeOfNameType(b, context);
   }
 
   case TEST_FUNCTION: {
-    if(toBeTested->m_nTestType != TEST_FUNCTION) return false;
-    if(returnType_ == 0) return true;
-    if(toBeTested->returnType_ == 0) return false;
+    // Bi is function(*), and Ai is a FunctionTest.
+    if(a->m_nTestType != TEST_FUNCTION) return false;
+    if(b->returnType_ == 0) return true;
+    if(a->returnType_ == 0) return false;
 
-    if(toBeTested->argTypes_->size() != argTypes_->size()) return false;
+    // Bi is function(Ba_1, Ba_2, ... Ba_N) as Br, Ai is function(Aa_1, Aa_2, ... Aa_M) as Ar, N (arity of Bi) equals M (arity of Ai),
+    // subtype(Ar, Br), and for values of I between 1 and N, subtype(Ba_I, Aa_I).
+    if(a->argTypes_->size() != b->argTypes_->size()) return false;
 
-    VectorOfSequenceTypes::const_iterator i = toBeTested->argTypes_->begin();
-    VectorOfSequenceTypes::const_iterator j = argTypes_->begin();
-    for(; i != toBeTested->argTypes_->end() && j != argTypes_->end(); ++i, ++j) {
-      if(!(*i)->matches(*j, context)) return false;
+    VectorOfSequenceTypes::const_iterator aa_i = a->argTypes_->begin();
+    VectorOfSequenceTypes::const_iterator ba_i = b->argTypes_->begin();
+    for(; aa_i != a->argTypes_->end() && ba_i != b->argTypes_->end(); ++aa_i, ++ba_i) {
+      if(!(*ba_i)->isSubtypeOf(*aa_i, context)) return false;
     }
 
-    if(!returnType_->matches(toBeTested->returnType_, context))
-      return false;
-    return true;
+    return a->returnType_->isSubtypeOf(b->returnType_, context);
   }
   }
   return true;
@@ -1100,7 +1179,31 @@ Result SequenceType::convertFunctionArg(const Result &input, DynamicContext *con
     // conversions are applied sequentially to the argument value V:
     if(xpath1Compat) {
       if(m_nOccurrence == SequenceType::EXACTLY_ONE || m_nOccurrence == SequenceType::QUESTION_MARK) {
-        result = new XPath1CompatConvertFunctionArgResult(location, result, this);
+        // 1. If the expected type calls for a single item or optional single item (examples: xs:string,
+        //    xs:string?, xdt:untypedAtomic, xdt:untypedAtomic?, node(), node()?, item(), item()?), then the
+        //    value V is effectively replaced by V[1].
+        Item::Ptr item = result->next(context);
+
+        if(testType == ItemType::TEST_ATOMIC_TYPE) {
+          const XMLCh* typeURI = m_pItemType->getTypeURI();
+          const XMLCh* typeName = m_pItemType->getType()->getName();
+
+          // 2. If the expected type is xs:string or xs:string?, then the value V is effectively replaced by
+          //    fn:string(V).
+          if(XPath2Utils::equals(typeName, SchemaSymbols::fgDT_STRING) &&
+             XPath2Utils::equals(typeURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+            item = FunctionString::string_item(item, context);
+          }
+
+          // 3. If the expected type is xs:double or xs:double?, then the value V is effectively replaced by
+          //    fn:number(V).
+          else if(XPath2Utils::equals(typeName, SchemaSymbols::fgDT_DOUBLE) &&
+                  XPath2Utils::equals(typeURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+            item = FunctionNumber::number(item, context, this);
+          }
+        }
+
+        result = item;
       }
     }
     // If the expected type is a sequence of an atomic type (possibly with an occurrence indicator *, +, or ?),
@@ -1149,23 +1252,48 @@ ASTNode *SequenceType::convertFunctionArg(ASTNode *arg, StaticContext *context,
   if(m_pItemType!=NULL)
   {
     SequenceType::ItemType::ItemTestType testType = getItemTestType();
-    // FS says we atomize first if the sequence type is atomic, and I think that's sensible - jpcs
-    if(testType == ItemType::TEST_ATOMIC_TYPE) {
-      arg = new (mm) XQAtomize(arg, mm);
-      arg->setLocationInfo(location);
-    }
 
     // If XPath 1.0 compatibility mode is true and an argument is not of the expected type, then the following
     // conversions are applied sequentially to the argument value V:
     if(context->getXPath1CompatibilityMode()) {
       if(m_nOccurrence == SequenceType::EXACTLY_ONE || m_nOccurrence == SequenceType::QUESTION_MARK) {
-          arg = new (mm) XPath1CompatConvertFunctionArg(arg, this, mm);
-          arg->setLocationInfo(location);
+        // 1. If the expected type calls for a single item or optional single item (examples: xs:string,
+        //    xs:string?, xdt:untypedAtomic, xdt:untypedAtomic?, node(), node()?, item(), item()?), then the
+        //    value V is effectively replaced by V[1].
+        VectorOfASTNodes headargs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
+        headargs.push_back(arg);
+        arg = new (mm) FunctionHead(headargs, mm);
+        arg->setLocationInfo(location);
+
+        if(testType == ItemType::TEST_ATOMIC_TYPE) {
+          const XMLCh *typeURI = m_pItemType->getTypeURI();
+          const XMLCh *typeName = m_pItemType->getType()->getName();
+
+          // 2. If the expected type is xs:string or xs:string?, then the value V is effectively replaced by
+          //    fn:string(V).
+          if(XPath2Utils::equals(typeName, SchemaSymbols::fgDT_STRING) &&
+             XPath2Utils::equals(typeURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+            VectorOfASTNodes stringargs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
+            stringargs.push_back(arg);
+            arg = new (mm) FunctionString(stringargs, mm);
+            arg->setLocationInfo(location);
+          }
+
+          // 3. If the expected type is xs:double or xs:double?, then the value V is effectively replaced by
+          //    fn:number(V).
+          else if(XPath2Utils::equals(typeName, SchemaSymbols::fgDT_DOUBLE) &&
+                  XPath2Utils::equals(typeURI, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+            VectorOfASTNodes numberargs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
+            numberargs.push_back(arg);
+            arg = new (mm) FunctionNumber(numberargs, mm);
+            arg->setLocationInfo(location);
+          }
+        }
       }
     }
     // If the expected type is a sequence of an atomic type (possibly with an occurrence indicator *, +, or ?),
     // the following conversions are applied:
-    else if(testType == ItemType::TEST_ATOMIC_TYPE) {
+    if(testType == ItemType::TEST_ATOMIC_TYPE) {
       arg = new (mm) XQAtomize(arg, mm);
       arg->setLocationInfo(location);
 
@@ -1191,6 +1319,9 @@ ASTNode *SequenceType::convertFunctionArg(ASTNode *arg, StaticContext *context,
       arg = new (mm) XQPromoteAnyURI(arg, uri, name, mm);
       arg->setLocationInfo(location);
     }
+    else if(testType == ItemType::TEST_FUNCTION && m_pItemType->getReturnType() != 0) {
+      arg = new (mm) XQFunctionCoercion(arg, this, mm);
+    }
   }
   // If, after the above conversions, the resulting value does not match the expected type according to the
   // rules for SequenceType Matching, a type error is raised. [err:XPTY0004] Note that the rules for
