@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 #include "../config/xqilla_config.h"
 #include <xqilla/update/FunctionPut.hpp>
@@ -45,13 +43,13 @@ const unsigned int FunctionPut::maxArgs = 2;
  * fn:put($node as node(), $uri as xs:string?) as empty-sequence()
  */
 FunctionPut::FunctionPut(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "node(), string?", args, memMgr),
+  : XQFunction(name, "updating ($node as node(), $uri as xs:string?) as empty-sequence()", args, memMgr),
     baseURI_(0)
 {
 }
 
 FunctionPut::FunctionPut(const XMLCh *baseURI, const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "node(), string?", args, memMgr),
+  : XQFunction(name, "updating ($node as node(), $uri as xs:string?) as empty-sequence()", args, memMgr),
     baseURI_(baseURI)
 {
 }
@@ -59,7 +57,8 @@ FunctionPut::FunctionPut(const XMLCh *baseURI, const VectorOfASTNodes &args, XPa
 ASTNode* FunctionPut::staticResolution(StaticContext *context)
 {
   baseURI_ = context->getBaseURI();
-  return resolveArguments(context);
+  resolveArguments(context);
+  return this;
 }
 
 ASTNode *FunctionPut::staticTypingImpl(StaticContext *context)
@@ -68,7 +67,8 @@ ASTNode *FunctionPut::staticTypingImpl(StaticContext *context)
 
   _src.forceNoFolding(true);
   _src.updating(true);
-  return calculateSRCForArguments(context);
+  calculateSRCForArguments(context);
+  return this;
 }
 
 PendingUpdateList FunctionPut::createUpdateList(DynamicContext *context) const
