@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -61,25 +59,10 @@ ASTNode *UApplyUpdates::staticTypingImpl(StaticContext *context)
   return this;
 }
 
-class ApplyUpdatesResult : public ResultImpl
-{
-public:
-  ApplyUpdatesResult(const UApplyUpdates *ast) : ResultImpl(ast), ast_(ast) {}
-
-  virtual Item::Ptr nextOrTail(Result &tail, DynamicContext *context)
-  {
-    AutoDelete<UpdateFactory> ufactory(context->createUpdateFactory());
-    ufactory->applyUpdates(ast_->getExpression()->createUpdateList(context), context,
-                           ast_->getRevalidationMode());
-    tail = 0;
-    return 0;
-  }
-
-private:
-  const UApplyUpdates *ast_;
-};
-
 Result UApplyUpdates::createResult(DynamicContext* context, int flags) const
 {
-  return new ApplyUpdatesResult(this);
+  AutoDelete<UpdateFactory> ufactory(context->createUpdateFactory());
+  ufactory->applyUpdates(getExpression()->createUpdateList(context), context,
+                         getRevalidationMode());
+  return 0;
 }
