@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,13 +15,12 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
 
 #include <xqilla/update/UInsertAfter.hpp>
+#include <xqilla/update/UInsertAsLast.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/Node.hpp>
 #include <xqilla/ast/XQTreatAs.hpp>
@@ -32,7 +31,6 @@
 #include <xqilla/update/PendingUpdateList.hpp>
 #include <xqilla/ast/XQDOMConstructor.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
-#include <xqilla/functions/FunctionNamespaceURIForPrefix.hpp>
 #include <xqilla/exceptions/DynamicErrorException.hpp>
 
 XERCES_CPP_NAMESPACE_USE;
@@ -134,17 +132,14 @@ PendingUpdateList UInsertAfter::createUpdateList(DynamicContext *context) const
       //    b. No attribute node in $alist may have a QName whose implied namespace binding conflicts with a namespace binding in the
       //       "namespaces" property of parent($target) [err:XUDY0023].
       ATQNameOrDerived::Ptr qname = ((Node*)item.get())->dmNodeName(context);
-      if(qname->getURI() != 0 && *(qname->getURI()) != 0) {
-        ATAnyURIOrDerived::Ptr uri = FunctionNamespaceURIForPrefix::uriForPrefix(qname->getPrefix(), parentNode, context, this);
-        if(uri.notNull() && !XPath2Utils::equals(uri->asString(context), qname->getURI())) {
-          XMLBuffer buf;
-          buf.append(X("Implied namespace binding for the insert after expression (\""));
-          buf.append(qname->getPrefix());
-          buf.append(X("\" -> \""));
-          buf.append(qname->getURI());
-          buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
-          XQThrow3(DynamicErrorException, X("UInsertAfter::createUpdateList"), buf.getRawBuffer(), this);
-        }
+      if(!UInsertAsLast::checkNamespaceBinding(qname, parentNode, context, this)) {
+        XMLBuffer buf;
+        buf.append(X("Implied namespace binding for the insert after expression (\""));
+        buf.append(qname->getPrefix());
+        buf.append(X("\" -> \""));
+        buf.append(qname->getURI());
+        buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
+        XQThrow3(DynamicErrorException, X("UInsertAfter::createUpdateList"), buf.getRawBuffer(), this);
       }
 
       alist.addItem(item);
