@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,21 +15,19 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
 
 #include <xqilla/update/URename.hpp>
 #include <xqilla/update/PendingUpdateList.hpp>
+#include <xqilla/update/UInsertAsLast.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/Node.hpp>
 #include <xqilla/ast/XQTreatAs.hpp>
 #include <xqilla/ast/XQDOMConstructor.hpp>
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
-#include <xqilla/functions/FunctionNamespaceURIForPrefix.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
 #include <xqilla/exceptions/XPath2TypeMatchException.hpp>
 #include <xqilla/exceptions/DynamicErrorException.hpp>
@@ -126,32 +124,28 @@ PendingUpdateList URename::createUpdateList(DynamicContext *context) const
   //    a. If $target is an element node, the "namespaces" property of $target must not include any namespace binding that conflicts
   //       with the implied namespace binding of $QName [err:XUDY0023].
   if(node->dmNodeKind() == Node::element_string) {
-    ATAnyURIOrDerived::Ptr uri = FunctionNamespaceURIForPrefix::uriForPrefix(qname->getPrefix(), node, context, this);
-    if(uri.notNull() && !XPath2Utils::equals(uri->asString(context), qname->getURI())) {
+    if(!UInsertAsLast::checkNamespaceBinding(qname, node, context, this)) {
       XMLBuffer buf;
       buf.append(X("Implied namespace binding for the rename expression (\""));
       buf.append(qname->getPrefix());
       buf.append(X("\" -> \""));
       buf.append(qname->getURI());
       buf.append(X("\") conflicts with those already existing on the target element [err:XUDY0023]"));
-      XQThrow3(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer(), this);
+      XQThrow(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer());
     }
   }
   //    b. If $target is an attribute node that has a parent, the "namespaces" property of parent($target) must not include any
   //       namespace binding that conflicts with the implied namespace binding of $QName [err:XUDY0023].
   else if(node->dmNodeKind() == Node::attribute_string) {
     Node::Ptr parentNode = node->dmParent(context);
-    if(parentNode.notNull() && qname->getURI() != 0 && *(qname->getURI()) != 0) {
-      ATAnyURIOrDerived::Ptr uri = FunctionNamespaceURIForPrefix::uriForPrefix(qname->getPrefix(), parentNode, context, this);
-      if(uri.notNull() && !XPath2Utils::equals(uri->asString(context), qname->getURI())) {
-        XMLBuffer buf;
-        buf.append(X("Implied namespace binding for the rename expression (\""));
-        buf.append(qname->getPrefix());
-        buf.append(X("\" -> \""));
-        buf.append(qname->getURI());
-        buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
-        XQThrow3(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer(), this);
-      }
+    if(parentNode.notNull() && !UInsertAsLast::checkNamespaceBinding(qname, parentNode, context, this)) {
+      XMLBuffer buf;
+      buf.append(X("Implied namespace binding for the rename expression (\""));
+      buf.append(qname->getPrefix());
+      buf.append(X("\" -> \""));
+      buf.append(qname->getURI());
+      buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
+      XQThrow(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer());
     }
   }
   //    c. If $target is processing instruction node, $QName must not include a non-empty namespace prefix. [err:XUDY0025].
