@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,21 +15,19 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
 
 #include <xqilla/update/UReplace.hpp>
 #include <xqilla/update/PendingUpdateList.hpp>
+#include <xqilla/update/UInsertAsLast.hpp>
 #include <xqilla/context/DynamicContext.hpp>
 #include <xqilla/items/Node.hpp>
 #include <xqilla/ast/XQTreatAs.hpp>
 #include <xqilla/ast/XQDOMConstructor.hpp>
 #include <xqilla/schema/SequenceType.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
-#include <xqilla/functions/FunctionNamespaceURIForPrefix.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
 #include <xqilla/exceptions/XPath2TypeMatchException.hpp>
 #include <xqilla/exceptions/DynamicErrorException.hpp>
@@ -144,17 +142,14 @@ PendingUpdateList UReplace::createUpdateList(DynamicContext *context) const
       //    b. No attribute node in $rlist may have a QName whose implied namespace binding conflicts with a namespace binding in the
       //       "namespaces" property of $parent [err:XUDY0023].
       ATQNameOrDerived::Ptr qname = ((Node*)tmp.get())->dmNodeName(context);
-      if(qname->getURI() != 0 && *(qname->getURI()) != 0) {
-        ATAnyURIOrDerived::Ptr uri = FunctionNamespaceURIForPrefix::uriForPrefix(qname->getPrefix(), parentNode, context, this);
-        if(uri.notNull() && !XPath2Utils::equals(uri->asString(context), qname->getURI())) {
-          XMLBuffer buf;
-          buf.append(X("Implied namespace binding for the replace expression (\""));
-          buf.append(qname->getPrefix());
-          buf.append(X("\" -> \""));
-          buf.append(qname->getURI());
-          buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
-          XQThrow3(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer(), this);
-        }
+      if(!UInsertAsLast::checkNamespaceBinding(qname, parentNode, context, this)) {
+        XMLBuffer buf;
+        buf.append(X("Implied namespace binding for the replace expression (\""));
+        buf.append(qname->getPrefix());
+        buf.append(X("\" -> \""));
+        buf.append(qname->getURI());
+        buf.append(X("\") conflicts with those already existing on the parent element of the target attribute [err:XUDY0023]"));
+        XQThrow(DynamicErrorException, X("URename::createUpdateList"), buf.getRawBuffer());
       }
 
       value.addItem(tmp);
