@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -34,13 +32,13 @@
 #include <xqilla/items/ATQNameOrDerived.hpp>
 #include <xqilla/items/ATUntypedAtomic.hpp>
 #include <xqilla/utils/XPath2NSUtils.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/items/DatatypeFactory.hpp>
 #include <xqilla/schema/DocumentCache.hpp>
 #include <xqilla/utils/XPath2Utils.hpp>
 #include <xqilla/events/EventHandler.hpp>
 #include <xqilla/events/EventSerializer.hpp>
 #include <xqilla/events/NSFixupFilter.hpp>
+#include <xqilla/context/ItemFactory.hpp>
 
 #include "AncestorAxis.hpp"
 #include "AncestorOrSelfAxis.hpp"
@@ -54,7 +52,6 @@
 #include "ParentAxis.hpp"
 #include "PrecedingAxis.hpp"
 #include "PrecedingSiblingAxis.hpp"
-#include <xqilla/axis/SelfAxis.hpp>
 #include <xqilla/axis/NodeTest.hpp>
 
 #include <xqilla/xerces/XercesConfiguration.hpp>
@@ -228,6 +225,9 @@ static void toEventsImpl(const DOMNode *node, EventHandler *events,
                              node->getNodeValue(), typeURI, typeName);
     }
     break;
+  case (DOMNode::NodeType)DOMXPathNamespace::XPATH_NAMESPACE_NODE:
+    events->namespaceEvent(emptyToNull(node->getNodeName()), emptyToNull(node->getNamespaceURI()));
+    break;
   default:
     break;
   }
@@ -427,7 +427,7 @@ const XMLCh* XercesNodeImpl::dmStringValue(const DynamicContext* context) const
         //   returns the concatenation of the string-values of all its text node descendants in document order. 
         //   It returns "" if the element has no text node descendants.
         // - If the element has a complex type with empty content, returns "".
-        if(XPath2Utils::equals(typeName, DocumentCache::g_szUntyped) && XPath2Utils::equals(typeUri, FunctionConstructor::XMLChXPath2DatatypesURI)
+        if((XPath2Utils::equals(typeName, DocumentCache::g_szUntyped) && XPath2Utils::equals(typeUri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA))
            || context->getDocumentCache()->getComplexTypeInfo(typeUri, typeName)!=NULL)
           addStringValueToBuffer(fNode,str);
         else
@@ -481,8 +481,8 @@ const XMLCh* XercesNodeImpl::dmStringValue(const DynamicContext* context) const
             getTypeUriAndName(typeUri,typeName);
             // Returns the value calculated as follows:
             //  - If the attribute type is xdt:untypedAtomic, xs:string, or a type derived from xs:string, returns that string.
-            if(XPath2Utils::equals(typeName, ATUntypedAtomic::fgDT_UNTYPEDATOMIC) &&
-               XPath2Utils::equals(typeUri, FunctionConstructor::XMLChXPath2DatatypesURI) ||
+            if((XPath2Utils::equals(typeName, ATUntypedAtomic::fgDT_UNTYPEDATOMIC) &&
+                XPath2Utils::equals(typeUri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) ||
                context->getDocumentCache()->isTypeOrDerivedFromType(typeUri,typeName,SchemaSymbols::fgURI_SCHEMAFORSCHEMA, SchemaSymbols::fgDT_STRING))
                 return context->getMemoryManager()->getPooledString(fNode->getNodeValue());
             //  - If the attribute type is xs:anyURI, returns the characters of the URI.
@@ -616,7 +616,7 @@ Sequence XercesNodeImpl::dmTypedValue(DynamicContext* context) const {
 
             // If the attribute is of type xdt:untypedAtomic: its typed-value is its dm:string-value as an xdt:untypedAtomic
             if(XPath2Utils::equals(typeName, ATUntypedAtomic::fgDT_UNTYPEDATOMIC) &&
-               XPath2Utils::equals(typeUri, FunctionConstructor::XMLChXPath2DatatypesURI))
+               XPath2Utils::equals(typeUri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA))
             {
                 item = (const Item::Ptr)context->getItemFactory()->createUntypedAtomic(dmStringValue(context), context);
                 return Sequence(item, context->getMemoryManager()); 
@@ -661,7 +661,7 @@ Sequence XercesNodeImpl::dmTypedValue(DynamicContext* context) const {
             getMemberTypeUriAndName(typeUri,typeName);
 
             // If the element is of type xdt:untyped or xs:anyType, its typed-value is its dm:string-value as an xdt:untypedAtomic.
-            if((XPath2Utils::equals(typeName, DocumentCache::g_szUntyped) && XPath2Utils::equals(typeUri, FunctionConstructor::XMLChXPath2DatatypesURI)) ||
+            if((XPath2Utils::equals(typeName, DocumentCache::g_szUntyped) && XPath2Utils::equals(typeUri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) ||
                (XPath2Utils::equals(typeName, SchemaSymbols::fgATTVAL_ANYTYPE) && XPath2Utils::equals(typeUri, SchemaSymbols::fgURI_SCHEMAFORSCHEMA))
               )
             {
@@ -990,7 +990,7 @@ Result XercesNodeImpl::getAxisResult(XQStep::Axis axis, const NodeTest *nodeTest
     return new PrecedingSiblingAxis(info, fNode, this, nodeTest, *this);
   }
   case XQStep::SELF: {
-    return nodeTest->filterResult(new SelfAxis(info, this), info);
+    return nodeTest->filterResult((Item::Ptr)this, info);
   }
   }
 
@@ -1131,7 +1131,7 @@ void XercesNodeImpl::typeUriAndName(const DOMNode *node, const XMLCh*& uri, cons
             // ignore it; the implementation of getInterface for Xerces < 2.6 will throw it
         }
         // we are xdt:untyped
-        uri=FunctionConstructor::XMLChXPath2DatatypesURI;
+        uri=SchemaSymbols::fgURI_SCHEMAFORSCHEMA;
         name=DocumentCache::g_szUntyped;
         return;
     }
@@ -1183,13 +1183,13 @@ void XercesNodeImpl::typeUriAndName(const DOMNode *node, const XMLCh*& uri, cons
             }
         }
         // we are xdt:untypedAtomic
-        uri=FunctionConstructor::XMLChXPath2DatatypesURI;
+        uri=SchemaSymbols::fgURI_SCHEMAFORSCHEMA;
         name=ATUntypedAtomic::fgDT_UNTYPEDATOMIC;
         return;
     }
     else if(nodeType == DOMNode::TEXT_NODE || 
             nodeType == DOMNode::CDATA_SECTION_NODE) {
-        uri=FunctionConstructor::XMLChXPath2DatatypesURI;
+        uri=SchemaSymbols::fgURI_SCHEMAFORSCHEMA;
         name=ATUntypedAtomic::fgDT_UNTYPEDATOMIC;
         return;
     }
