@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: XercesUpdateFactory.cpp 814 2009-11-13 14:21:12Z jpcs $
  */
 
 #include "../config/xqilla_config.h"
@@ -90,7 +88,8 @@ static DOMNode *importNodeFix(DOMDocument *doc, DOMNode *node, bool deep)
   // Xerces-C has a bug that doesn't copy the prefix correctly - jpcs
   if((node->getNodeType() == DOMNode::ELEMENT_NODE ||
       node->getNodeType() == DOMNode::ATTRIBUTE_NODE) &&
-     node->getNamespaceURI() != 0 && *node->getNamespaceURI() != 0)
+     node->getNamespaceURI() != 0 && *node->getNamespaceURI() != 0 &&
+     node->getPrefix() != 0 && *node->getPrefix() != 0)
     newNode->setPrefix(node->getPrefix());
   return newNode;
 }
