@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: TestSuiteParser.cpp 736 2009-02-05 21:37:38Z lauren_foutz $
  */
 
 #ifdef _MSC_VER
@@ -115,10 +113,14 @@ void TestSuiteParser::startElement(const XMLCh* const uri, const XMLCh* const lo
 
         urlXQTSQueriesDirectory_.setURL(urlXQTSCatalog_, attributes.getValue(X("XQueryQueryOffsetPath")));
         urlXQTSResultsDirectory_.setURL(urlXQTSCatalog_, attributes.getValue(X("ResultOffsetPath")));
+
+        // detect runner's parser language and conf
+        runner_->detectParserLang(szURI);
       }
       else if(szName=="testcases") {
         // XSLT test suite
         xslt_ = true;
+        runner_->detectParserLang(szURI); 
 
         runner_->getResultListener()->reportVersion(UTF8(attributes.getValue(X("testSuiteVersion"))), false);
 
@@ -479,8 +481,10 @@ void TestSuiteParser::endElement(const XMLCh* const uri, const XMLCh* const loca
       else if(szName == "output-file")
       {
         readingChars_=false;
-        chars_=UTF8(XMLURL(urlBasePathReferenceFiles_, chars_.c_str()).getURLText());
-        testCase_.outputFiles[chars_]=compareMethod_;
+        if(chars_ != "") {
+          chars_=UTF8(XMLURL(urlBasePathReferenceFiles_, chars_.c_str()).getURLText());
+          testCase_.outputFiles[chars_]=compareMethod_;
+        }
         compareMethod_="";
       }
       else if(szName == "expected-error")
