@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2011,
+ *     Oracle and/or its affiliates. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id: TestSuiteResultListener.cpp 663 2008-10-08 17:51:30Z jpcs $
  */
 #ifdef _MSC_VER
 #pragma warning(disable: 4786)
@@ -300,6 +298,19 @@ bool KnownErrorChecker::saveErrors(const std::string &errorFile) const
   return true;
 }
 
+bool KnownErrorChecker::isSkippedTest(const std::string &testName) const
+{
+  if(errors_.empty())
+    return false;
+
+  std::map<std::string, Error>::const_iterator it = errors_.find(testName);
+ 
+  if(it != errors_.end() && (*it).second.action == "skip")
+    return true;
+  else
+    return false;
+
+}
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 ConsoleResultListener::ConsoleResultListener()
